function [result, ruleIndex] = checkLeftLegFirst( ...
    firstPlateMLStanceSamples, ...
    secondPlateMLStanceSamples, toBeVerbose)

% checkLeftLegFirst checks if the first of two given sequences of samples 
% of the medio-lateral force is related to the left leg-foot.  
%
% Given sequences should be related to two force plates, 
% one of which contacted the left foot, while another contacted the right foot.
%
% Given sequences must comprise only stance-related samples, 
% but need not to be time-normalized, or to comprise as many samples.
%
% The algorithm is based on trying decision rules, from more to less strict ones. 
% Rules are based on comparing the means of the beginning, mid, and ending
% framents of time-normalized versions of the input sequences.
%
% This function allows for automatically associating participant's legs 
% with force plates and measurements.
% During experiments participants were allowed to put either foot 
% on the first of two force plates, which were arranged 
% one after another along the walkway.
% The leg order was not observed, so our data set of gait measurements 
% contains no explicite information about leg-plate relations.
% 
% Input:
% - firstPlateMLForceStanceSamples : a vector of stance-related samples 
%       of the medio-lateral ground reaction force of one force plate.
% - secondPlateMLForceStanceSamples : a signal (vector of samples) 
%       of the medio-lateral ground reaction force of another force plate
% - toBeVerbose : 0 or 1. Disables/enables displaying diagnostic messages, 
%                 which were used to debug the code.
%
% Output:
% - result : 1 if firstPlateMLForceStanceSamples are related to the left foot
%            0 otherwise
% - ruleIndex : index of the decision rule behind the result
%
% Created on May 04, 2022
% 
% Author: Marek Parfieniuk

% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>


if (nargin < 3)
    toBeVerbose = 0;
end

% Downsampled (to the same length) versions of input sequences will be
% compared.
compareLength = 100; 

sampleCount = numel(firstPlateMLStanceSamples) - 1; 
firstPlateSamples = interp1((0 : sampleCount) ./ sampleCount, firstPlateMLStanceSamples, ...
    (0 : (compareLength - 1)) ./ (compareLength - 1));

sampleCount = numel(secondPlateMLStanceSamples) - 1; 
secondPlateSamples = interp1((0 : sampleCount) ./ sampleCount, secondPlateMLStanceSamples, ...
    (0 : (compareLength - 1)) ./ (compareLength - 1));

ruleIndex = 0;

% Indexes of three fragments of a sample sequence.
% Fragments are obtained by dividing a sequence into 7 parts.
% Two no-care parts are neglected, 
% while 3 parts are considered to form one longer fragment.

d = round(compareLength / 7);
begIdx = 1 : d;
midIdx = (2 * d) : (5 * d);
endIdx = (6 * d) : compareLength;

midMean1 = mean(firstPlateSamples(midIdx));
midStd1 = std(firstPlateSamples(midIdx));
midMean2 = mean(secondPlateSamples(midIdx));
midStd2 = std(secondPlateSamples(midIdx));

frc1 = 0.45;    % Tuned based on record no. 93355.

% A rule is applied if no previous rule gives a decision.

if (ruleIndex == 0)     
    begGreaterCount1 = sum(firstPlateSamples(begIdx) > (midMean1 + midStd1));
    endGreaterCount1 = sum(firstPlateSamples(endIdx) > (midMean1 + midStd1));
    begLowerCount2 = sum(secondPlateSamples(begIdx) < (midMean2 - midStd2));
    endLowerCount2 = sum(secondPlateSamples(endIdx) < (midMean2 - midStd2));
    
    if ((begGreaterCount1 > (frc1 * d)) && (endGreaterCount1 > (frc1 * d)) ... 
            && (begLowerCount2 > (frc1 * d)) && (endLowerCount2 > (frc1 * d)))
        ruleIndex = -1; % right on first

    end
end
if (ruleIndex == 0)     
    begLowerCount1 = sum(firstPlateSamples(begIdx) < (midMean1 - midStd1));
    endLowerCount1 = sum(firstPlateSamples(endIdx) < (midMean1 - midStd1));
    begGreaterCount2 = sum(secondPlateSamples(begIdx) > (midMean2 + midStd2));
    endGreaterCount2 = sum(secondPlateSamples(endIdx) > (midMean2 + midStd2));
    
    if ((begGreaterCount2 > (frc1 * d)) && (endGreaterCount2 > (frc1 * d)) ... 
            && (begLowerCount1 > (frc1 * d)) && (endLowerCount1 > (frc1 * d)))
        ruleIndex = 1; % left on first

    end
end
if (ruleIndex == 0)     
    begGreaterCount1 = sum(firstPlateSamples(begIdx) > (midMean1 + midStd1));   
    begLowerCount2 = sum(secondPlateSamples(begIdx) < (midMean2 - midStd2));
    
    if ((begGreaterCount1 > (frc1 * d)) && (begLowerCount2 > (frc1 * d))) 
        ruleIndex = -3; % right on first
    end
end
if (ruleIndex == 0)     
    begLowerCount1 = sum(firstPlateSamples(begIdx) < (midMean1 - midStd1));
    begGreaterCount2 = sum(secondPlateSamples(begIdx) > (midMean2 + midStd2));
   
    if ((begLowerCount1 > (frc1 * d)) && (begGreaterCount2 > (frc1 * d)))
        ruleIndex = 3; % left on first
    end
end
if (ruleIndex == 0)     
    begGreaterCount1 = sum(firstPlateSamples(begIdx) > midMean1);
    endGreaterCount1 = sum(firstPlateSamples(endIdx) > midMean1);
    begLowerCount2 = sum(secondPlateSamples(begIdx) < midMean2);
    endLowerCount2 = sum(secondPlateSamples(endIdx) < midMean2);
    
    if ((begGreaterCount1 > (frc1 * d)) && (endGreaterCount1 > (frc1 * d)) ... 
            && (begLowerCount2 > (frc1 * d)) && (endLowerCount2 > (frc1 * d)))
        ruleIndex = -5; % right on first

    end
end
if (ruleIndex == 0)     
    begLowerCount1 = sum(firstPlateSamples(begIdx) < midMean1);
    endLowerCount1 = sum(firstPlateSamples(endIdx) < midMean1);
    begGreaterCount2 = sum(secondPlateSamples(begIdx) > midMean2);
    endGreaterCount2 = sum(secondPlateSamples(endIdx) > midMean2);
    
    if ((begGreaterCount2 > (frc1 * d)) && (endGreaterCount2 > (frc1 * d)) ... 
            && (begLowerCount1 > (frc1 * d)) && (endLowerCount1 > (frc1 * d)))
        ruleIndex = 5; % left on first

    end
end
if (ruleIndex == 0)  % This last rule is decisive.
    if (midMean1 < midMean2)
        ruleIndex = -4; % right on first
    else
        ruleIndex = 4;
    end            
end

result = (ruleIndex >= 0);
