function patternData = convertStancesIntoPatterns(stanceData, patternSize)

% convertStancesIntoPatterns converts data of stance-related fragments 
% of GRF signals into patterns suitable for recognizing persons and shoe
% types.
%
% The input structure should be prepared by using the extractStancesFromMeasurements
% function.
%
% The main effects of this function are
% - it returns data assigned to legs, taking as input data assigned to force plates.
% - it returns sample series of the same, given length, regardless of how many
%   samples were in input signals; it time-normalizes GRF signals 
%   by interpolating samples,
%
% This function returns a structure of fiels
% each of which is a vector that contains the corresponding data
% (numbers or vectors) of all input records.
%
% Input:
% - stanceData : array of records of gait measurements
% - patternSize : number of samples that result from signal
%   time-normalization.
%
% Output:
% - patternData : structure of column vectors with pattern data 
%
% Created on Sep. 28, 2021
% 
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

measurementCount = length(stanceData.fileNames);

patternData.fileNames = cell(measurementCount, 1);

patternData.leftVForces = zeros(measurementCount, patternSize);
patternData.rightVForces = zeros(measurementCount, patternSize);
patternData.leftAPForces = zeros(measurementCount, patternSize);
patternData.rightAPForces = zeros(measurementCount, patternSize);
patternData.leftMLForces = zeros(measurementCount, patternSize);
patternData.rightMLForces = zeros(measurementCount, patternSize);

patternData.leftStrikeIndexes = zeros(measurementCount, 1);
patternData.rightStrikeIndexes = zeros(measurementCount, 1);
patternData.leftOffIndexes = zeros(measurementCount, 1);
patternData.rightOffIndexes = zeros(measurementCount, 1);

patternData.leftVStrikeMeans = zeros(measurementCount, 1);
patternData.leftAPStrikeMeans = zeros(measurementCount, 1);
patternData.leftMLStrikeMeans = zeros(measurementCount, 1);
patternData.leftVOffMeans = zeros(measurementCount, 1);
patternData.leftAPOffMeans = zeros(measurementCount, 1);
patternData.leftMLOffMeans = zeros(measurementCount, 1);
patternData.rightVStrikeMeans = zeros(measurementCount, 1);
patternData.rightAPStrikeMeans = zeros(measurementCount, 1);
patternData.rightMLStrikeMeans = zeros(measurementCount, 1);
patternData.rightVOffMeans = zeros(measurementCount, 1);
patternData.rightAPOffMeans = zeros(measurementCount, 1);
patternData.rightMLOffMeans = zeros(measurementCount, 1);

normalizedLength = patternSize;
normalizedRelativeTimes = (0 : (normalizedLength - 1)) ./ (normalizedLength - 1);

for measurementIndex = 1 : measurementCount    
    if (mod(measurementIndex, 10) == 0)
        fprintf('Processing measurement no. %d\n', measurementIndex);
    end   

    patternData.fileNames(measurementIndex) = stanceData.fileNames(measurementIndex);    
    
    %keyboard
    
%     if (length(stanceData.firstVForce{measurementIndex}) > normalizedLength)
%        error('Normalization would shorten record') 
%     end
    
    originalRelativeTimes = (0 : (length(stanceData.firstVForces{measurementIndex}) - 1)) ./ (length(stanceData.firstVForces{measurementIndex}) - 1);   
    firstNormalizedVForce = interp1(originalRelativeTimes, stanceData.firstVForces{measurementIndex}, normalizedRelativeTimes);
    firstNormalizedAPForce = interp1(originalRelativeTimes, stanceData.firstAPForces{measurementIndex}, normalizedRelativeTimes);
    firstNormalizedMLForce = interp1(originalRelativeTimes, stanceData.firstMLForces{measurementIndex}, normalizedRelativeTimes);

%     if (length(stanceData.secondVForce{measurementIndex}) > normalizedLength)
%        error('Normalization would shorten record') 
%     end    
    
    originalRelativeTimes = (0 : (length(stanceData.secondVForces{measurementIndex}) - 1)) ./ (length(stanceData.secondVForces{measurementIndex}) - 1);   
    secondNormalizedVForce = interp1(originalRelativeTimes, stanceData.secondVForces{measurementIndex}, normalizedRelativeTimes);
    secondNormalizedAPForce = interp1(originalRelativeTimes, stanceData.secondAPForces{measurementIndex}, normalizedRelativeTimes);
    secondNormalizedMLForce = interp1(originalRelativeTimes, stanceData.secondMLForces{measurementIndex}, normalizedRelativeTimes);

    
    %[result, ruleIndex] = isLeftLegFirst(firstMLWave, secondMLWave, 0);     

     %if (isLeftLegMedioLateralForce(recordData.secondVForce))
     
     if (stanceData.leftLegFirstFlags(measurementIndex))         
         patternData.leftVForces(measurementIndex, :) = firstNormalizedVForce;
         patternData.leftAPForces(measurementIndex, :) = firstNormalizedAPForce;
         patternData.leftMLForces(measurementIndex, :) = firstNormalizedMLForce;
         
         patternData.leftStrikeIndexes(measurementIndex) = stanceData.firstStrikeIndexes(measurementIndex);
         patternData.leftOffIndexes(measurementIndex) = stanceData.firstOffIndexes(measurementIndex);
         
         patternData.leftVStrikeMeans(measurementIndex) = stanceData.firstVStrikeMeans(measurementIndex);
         patternData.leftAPStrikeMeans(measurementIndex) = stanceData.firstAPStrikeMeans(measurementIndex);
         patternData.leftMLStrikeMeans(measurementIndex) = stanceData.firstMLStrikeMeans(measurementIndex);
         patternData.leftVOffMeans(measurementIndex) = stanceData.firstVOffMeans(measurementIndex);
         patternData.leftAPOffMeans(measurementIndex) = stanceData.firstAPOffMeans(measurementIndex);
         patternData.leftMLOffMeans(measurementIndex) = stanceData.firstMLOffMeans(measurementIndex);         
         
         patternData.rightVForces(measurementIndex, :) = secondNormalizedVForce;
         patternData.rightAPForces(measurementIndex, :) = secondNormalizedAPForce;
         patternData.rightMLForces(measurementIndex, :) = secondNormalizedMLForce;
         
         patternData.rightStrikeIndexes(measurementIndex) = stanceData.secondStrikeIndexes(measurementIndex);
         patternData.rightOffIndexes(measurementIndex) = stanceData.secondOffIndexes(measurementIndex);

         patternData.rightVStrikeMeans(measurementIndex) = stanceData.secondVStrikeMeans(measurementIndex);
         patternData.rightAPStrikeMeans(measurementIndex) = stanceData.secondAPStrikeMeans(measurementIndex);
         patternData.rightMLStrikeMeans(measurementIndex) = stanceData.secondMLStrikeMeans(measurementIndex);
         patternData.rightVOffMeans(measurementIndex) = stanceData.secondVOffMeans(measurementIndex);
         patternData.rightAPOffMeans(measurementIndex) = stanceData.secondAPOffMeans(measurementIndex);
         patternData.rightMLOffMeans(measurementIndex) = stanceData.secondMLOffMeans(measurementIndex);         
     else
         patternData.leftVForces(measurementIndex, :) = secondNormalizedVForce;
         patternData.leftAPForces(measurementIndex, :) = secondNormalizedAPForce;
         patternData.leftMLForces(measurementIndex, :) = secondNormalizedMLForce;
         patternData.leftStrikeIndexes(measurementIndex) = stanceData.secondStrikeIndexes(measurementIndex);
         patternData.leftOffIndexes(measurementIndex) = stanceData.secondOffIndexes(measurementIndex);

         patternData.leftVStrikeMeans(measurementIndex) = stanceData.secondVStrikeMeans(measurementIndex);
         patternData.leftAPStrikeMeans(measurementIndex) = stanceData.secondAPStrikeMeans(measurementIndex);
         patternData.leftMLStrikeMeans(measurementIndex) = stanceData.secondMLStrikeMeans(measurementIndex);
         patternData.leftVOffMeans(measurementIndex) = stanceData.secondVOffMeans(measurementIndex);
         patternData.leftAPOffMeans(measurementIndex) = stanceData.secondAPOffMeans(measurementIndex);
         patternData.leftMLOffMeans(measurementIndex) = stanceData.secondMLOffMeans(measurementIndex);         
         
         patternData.rightVForces(measurementIndex, :) = firstNormalizedVForce;
         patternData.rightAPForces(measurementIndex, :) = firstNormalizedAPForce;
         patternData.rightMLForces(measurementIndex, :) = firstNormalizedMLForce;
         
         patternData.rightStrikeIndexes(measurementIndex) = stanceData.firstStrikeIndexes(measurementIndex);
         patternData.rightOffIndexes(measurementIndex) = stanceData.firstOffIndexes(measurementIndex);    
         
         patternData.rightVStrikeMeans(measurementIndex) = stanceData.firstVStrikeMeans(measurementIndex);
         patternData.rightAPStrikeMeans(measurementIndex) = stanceData.firstAPStrikeMeans(measurementIndex);
         patternData.rightMLStrikeMeans(measurementIndex) = stanceData.firstMLStrikeMeans(measurementIndex);
         patternData.rightVOffMeans(measurementIndex) = stanceData.firstVOffMeans(measurementIndex);
         patternData.rightAPOffMeans(measurementIndex) = stanceData.firstAPOffMeans(measurementIndex);
         patternData.rightMLOffMeans(measurementIndex) = stanceData.firstMLOffMeans(measurementIndex);          
     end
end




   