function [stanceSamples, strikeIndex, offIndex] ...
    = cutStance(vForce, strikeInitialIndex, offInitialIndex, toSmoothBoundaries, toBeVerbose)

% cutStance cuts out stance-related samples
% from measurement of the vertical ground reaction force.
% The stance-related fragment of the GRF signal must be indicated by
% providing estimated indexes of boundary samples related to foot strike/off events. 
% These indexes should be determined by using the findStance function.
% 
% cutStance can refine boundary indexes 
% and replace original samples with extrapolated values,
% so as to return a series of non-negative values which has smoothed tails 
% and begins and ends with zero-valued samples.
%
% The output refined indexes allow for mapping the resulting sample series
% onto the original signal.
%
% Input:
% - vForce : a vector of samples of the vertical ground reaction force.
% - strikeInitialIndex : The initial estimate of the index of the sample 
%                   that begins the foot strike.                 .
% - offInitialIndex : The initial estimate of the index of the sample
%                   that ends the foot off.
% - toSmoothBoundaries : 0 or 1. Disables/enables smoothing of boundaries 
%                   of sample series
% - toBeVerbose : 0 or 1. Disables/enables displaying diagnostic messages, 
%                   which were used for debugging.
%
% Output:
% - stanceSamples : A series of samples related to the stance phase of gait.
% - strikeIndex : The index of the (possibly extrapolated) zero sample 
%                   that is estimated to precede the foot strike.
% - offIndex : The index of the (possibly extrapolated) zero sample 
%                  that is estimated to follow the foot off.
%
%
% Created on Sep. 27, 2021
%
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

if (toSmoothBoundaries == 0)                 % No wave refinement
    strikeIndex = strikeInitialIndex;
    offIndex = offInitialIndex;
    stanceSamples = vForce(strikeIndex : offIndex);
    return;
end

% Refinement of the foot-strike-related tail of the wave

strikeValue = vForce(strikeInitialIndex);
strikeIndex = strikeInitialIndex;
if (strikeValue > 0)                   % Positive sample extrapolation
    
    % We analyze the signal-slope 
    % at the initial estimate of the foot-strike moment
    % in order to estimate the difference for linear extrapolation
    
    % We consider two samples before and after the the foot-strike moment
    % so as to get the average difference between subsequent samples
    
    dd = diff(vForce(strikeIndex + (-2 : 2)));    
    d = mean(dd(dd > 0));   % Values should decrease toward wave tail   
    
    if (d > 0)
        % Difference estimated successfully 
        if (d < 0.1 * strikeValue)      % but values cannot
            d = strikeValue / 10;       % decrease too slowly
        end
    else
        if (toBeVerbose)
            warning('cutStance: unsuccessful estimation of difference for strike extrapolation %f', d)
        end
        %keyboard
        d = strikeValue;    % Difference set so as to cut the wave in one step
    end
        
    while (1)                       % Samples extrapolated one-by-one
        if (strikeValue >= d)       % values decreased by the difference
            strikeValue = strikeValue - d;
            strikeIndex = strikeIndex - 1;
            vForce(strikeIndex) = strikeValue;
        else                            % until crossing zero
            if (strikeValue > abs(strikeValue - d))                    
                strikeIndex = strikeIndex - 1;
            end
            vForce(strikeIndex) = 0;
            break;
        end
    end
elseif (strikeValue < 0)        % When the initial sample is negative
    while (1)                           % we go backward and omit samples
        t = vForce(strikeIndex + 1);     % until crossing zero
        if (t < 0)
            strikeValue = t;
            strikeIndex = strikeIndex + 1;
        else 
            if (abs(strikeValue) >= t)
                strikeIndex = strikeIndex + 1;
            end
            vForce(strikeIndex) = 0;
            break;            
        end
    end

    
end

% In analogous way, we refine the foot-off-related tail of the wave

offValue = vForce(offInitialIndex);
offIndex = offInitialIndex;
if (offValue > 0)
    dd = diff(vForce(offIndex - (-2 : 2)));
    d = mean(dd(dd > 0));

    if (d > 0)
        if (d < 0.1 * offValue)
            d = offValue / 10;
        end    
    else
        if (toBeVerbose)
            warning('cutStance: unsuccessful estimation of difference for off extrapolation %f', d)
        end
        %keyboard
        d = offValue;
    end
    
    while (1)
        if (offValue >= d)
            offValue = offValue - d;
            offIndex = offIndex + 1;
            vForce(offIndex) = offValue;
        else
            if (offValue > abs(offValue - d))                    
                offIndex = offIndex + 1;
            end
            vForce(offIndex) = 0;
            break;
        end
    end
elseif (offValue < 0)
    while (1)
        t = vForce(offIndex - 1);
        if (t < 0)
            offValue = t;
            offIndex = offIndex - 1;
        else 
            if (abs(offValue) >= t)
                offIndex = offIndex - 1;
            end
            vForce(offIndex) = 0;
            break;            
        end
    end        
end

stanceSamples = vForce(strikeIndex : offIndex);
stanceSamples(stanceSamples < 0) = 0;

