function stanceData = extractStancesFromMeasurements(gaitMeasurements, meanOffset, toSmoothBoundaries)

% extractStancesFromMeasurements processes raw measurements of gait 
% so as to obtain stance-related GRF samples and related information:
% indexes/values of boundary/maximum samples, leg order, file name.
% Its results can be used to recognize patterns, 
% to assess data integrity, and to identify trends and outliers.
% 
% Raw measurements are provided via an array of structures 
% that should be created by using the loadGaitData function.
%
% Stance-related samples are determined by using the findStance and cutStance functions. 
% The latter function can smooth tails of stance-related fragment taken of GRF vertical force, 
% so boundary samples can differ from the correspoding samples of raw measurements.
% The "Initial" sub-word is used to distinguish results 
% of the findStance function from those of the cutStance results.
%
% extractStancesFromMeasurements returns a structure with fields
% each of which is an array that contains the corresponding data
% (numbers or vectors) of all raw measurements.
%
% Examples of accessing output data:
% - stanceData.firstVForce{13} : gives stance-related samples 
%   of vertical force of first plate of the 13th measurement
% - stanceData.secondStrikeIndex(3) : gives the index of the sample
%   estimated to start the foot strike on the first plate  
%
% Input:
% - gaitMeasurements : array of structures that describe raw measurements
% - meanOffset : how many samples have to be used to compute means
% - toSmoothBoundaries : 0 or 1, disables/enables refinement 
%           of boundaries of the stance-related fragment of the vertical force;
%           this argument is passed to the cutStance function
%
% Output:
% - stanceData : structure of column vectors with stance-relatd data 
%
% Created on May 8, 2022
% 
% Author: Marek Parfieniuk

% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

fileNames = cell(size(gaitMeasurements));
measurementLengths = zeros(size(gaitMeasurements));
leftLegFirstFlags = zeros(size(gaitMeasurements));
legOrderRuleIndexes = zeros(size(gaitMeasurements));

firstVForces = cell(size(gaitMeasurements));
secondVForces = cell(size(gaitMeasurements));
firstMLForces = cell(size(gaitMeasurements));
secondMLForces = cell(size(gaitMeasurements));
firstAPForces = cell(size(gaitMeasurements));
secondAPForces = cell(size(gaitMeasurements));

firstStrikeInitialIndexes = zeros(size(gaitMeasurements));
firstOffInitialIndexes = zeros(size(gaitMeasurements));
firstStrikeInitialValues = zeros(size(gaitMeasurements));
firstOffInitialValues = zeros(size(gaitMeasurements));
firstMaxIndexes = zeros(size(gaitMeasurements));
firstMaxValues = zeros(size(gaitMeasurements));
firstStrikeIndexes = zeros(size(gaitMeasurements));
firstOffIndexes = zeros(size(gaitMeasurements));
firstVStrikeMeans = zeros(size(gaitMeasurements));
firstAPStrikeMeans = zeros(size(gaitMeasurements));
firstMLStrikeMeans = zeros(size(gaitMeasurements));
firstVOffMeans = zeros(size(gaitMeasurements));
firstAPOffMeans = zeros(size(gaitMeasurements));
firstMLOffMeans = zeros(size(gaitMeasurements));

secondStrikeInitialIndexes = zeros(size(gaitMeasurements));
secondOffInitialIndexes = zeros(size(gaitMeasurements));
secondStrikeInitialValues = zeros(size(gaitMeasurements));
secondOffInitialValues = zeros(size(gaitMeasurements));
secondMaxIndexes = zeros(size(gaitMeasurements));
secondMaxValues = zeros(size(gaitMeasurements));
secondStrikeIndexes = zeros(size(gaitMeasurements));
secondOffIndexes = zeros(size(gaitMeasurements));
secondVStrikeMeans = zeros(size(gaitMeasurements));
secondAPStrikeMeans = zeros(size(gaitMeasurements));
secondMLStrikeMeans = zeros(size(gaitMeasurements));
secondVOffMeans = zeros(size(gaitMeasurements));
secondAPOffMeans = zeros(size(gaitMeasurements));
secondMLOffMeans = zeros(size(gaitMeasurements));

for measurementIndex = 1 : length(gaitMeasurements)
    measurementRecord = gaitMeasurements{measurementIndex};
    
    if (mod(measurementIndex, 10) == 0)
        fprintf('Processing measurement no. %d\n', measurementIndex);
    end

    fileNames{measurementIndex} = measurementRecord.fileName;    
    measurementLengths(measurementIndex) = length(measurementRecord.firstVForce);
    
    try       
        [firstStrikeInitialIndexes(measurementIndex), firstOffInitialIndexes(measurementIndex), ...
            firstStrikeInitialValues(measurementIndex), firstOffInitialValues(measurementIndex), ...
            firstMaxIndexes(measurementIndex), firstMaxValues(measurementIndex)] ...
            = findStance(measurementRecord.firstVForce, 0);
    
        [firstVForces{measurementIndex}, firstStrikeIndexes(measurementIndex), ...
            firstOffIndexes(measurementIndex)] ...
            = cutStance(measurementRecord.firstVForce, firstStrikeInitialIndexes(measurementIndex), ...
            firstOffInitialIndexes(measurementIndex), toSmoothBoundaries, 0); 

        [secondStrikeInitialIndexes(measurementIndex), secondOffInitialIndexes(measurementIndex), ...
            secondStrikeInitialValues(measurementIndex), secondOffInitialValues(measurementIndex), ...
            secondMaxIndexes(measurementIndex), secondMaxValues(measurementIndex)] ...
            = findStance(measurementRecord.secondVForce, 0);
    
        [secondVForces{measurementIndex}, secondStrikeIndexes(measurementIndex), ...
            secondOffIndexes(measurementIndex)] ...
            = cutStance(measurementRecord.secondVForce, secondStrikeInitialIndexes(measurementIndex), ...
            secondOffInitialIndexes(measurementIndex), toSmoothBoundaries, 0);
        
    catch ex        
        fprintf(getReport(ex,'extended','hyperlinks','off'))        
        fprintf('\n\nError while processing %s\n', measurementRecord.fileName)
        keyboard
        % showRecord(gaitData, recordData.fileName, gcf)
    end

    stanceIndexes = firstStrikeIndexes(measurementIndex) : firstOffIndexes(measurementIndex); % of stance samples
    firstAPForces{measurementIndex} = measurementRecord.firstAPForce(stanceIndexes);
    firstMLForces{measurementIndex} = measurementRecord.firstMLForce(stanceIndexes);

    [firstVStrikeMeans(measurementIndex), firstVOffMeans(measurementIndex)] ...
            = evaluateMean(measurementRecord.firstVForce, stanceIndexes, meanOffset);
    [firstAPStrikeMeans(measurementIndex), firstAPOffMeans(measurementIndex)] ...
            = evaluateMean(measurementRecord.firstAPForce, stanceIndexes, meanOffset);
    [firstMLStrikeMeans(measurementIndex), firstMLOffMeans(measurementIndex)] ...
            = evaluateMean(measurementRecord.firstMLForce, stanceIndexes, meanOffset);
        
    stanceIndexes = secondStrikeIndexes(measurementIndex) : secondOffIndexes(measurementIndex); % of stance samples   
    secondAPForces{measurementIndex} = measurementRecord.secondAPForce(stanceIndexes);
    secondMLForces{measurementIndex} = measurementRecord.secondMLForce(stanceIndexes);

    [secondVStrikeMeans(measurementIndex), secondVOffMeans(measurementIndex)] = evaluateMean(measurementRecord.secondVForce, stanceIndexes, meanOffset);
    [secondAPStrikeMeans(measurementIndex), secondAPOffMeans(measurementIndex)] = evaluateMean(measurementRecord.secondAPForce, stanceIndexes, meanOffset);
    [secondMLStrikeMeans(measurementIndex), secondMLOffMeans(measurementIndex)] = evaluateMean(measurementRecord.secondMLForce, stanceIndexes, meanOffset);
    
    [leftLegFirstFlags(measurementIndex), legOrderRuleIndexes(measurementIndex)] = checkLeftLegFirst(firstMLForces{measurementIndex}, secondMLForces{measurementIndex}, 0);
end

stanceData.fileNames = fileNames;
stanceData.measurementLengths = measurementLengths;
stanceData.leftLegFirstFlags = leftLegFirstFlags;

stanceData.firstVForces = firstVForces;
stanceData.firstAPForces = firstAPForces;
stanceData.firstMLForces = firstMLForces;
stanceData.firstStrikeIndexes = firstStrikeIndexes;
stanceData.firstStrikeInitialIndexes = firstStrikeInitialIndexes;
stanceData.firstStrikeInitialValues = firstStrikeInitialValues;
stanceData.firstOffIndexes = firstOffIndexes;
stanceData.firstOffInitialIndexes = firstOffInitialIndexes;
stanceData.firstOffInitialValues = firstOffInitialValues;
stanceData.firstMaxValues = firstMaxValues;
stanceData.firstMaxIndexes = firstMaxIndexes;

stanceData.firstVStrikeMeans = firstVStrikeMeans;
stanceData.firstAPStrikeMeans = firstAPStrikeMeans;
stanceData.firstMLStrikeMeans = firstMLStrikeMeans;
stanceData.firstVOffMeans = firstVOffMeans;
stanceData.firstAPOffMeans = firstAPOffMeans;
stanceData.firstMLOffMeans = firstMLOffMeans;

stanceData.secondVForces = secondVForces;
stanceData.secondAPForces = secondAPForces;
stanceData.secondMLForces = secondMLForces;
stanceData.secondStrikeIndexes = secondStrikeIndexes;
stanceData.secondStrikeInitialIndexes = secondStrikeInitialIndexes;
stanceData.secondStrikeInitialValues = secondStrikeInitialValues;
stanceData.secondOffIndexes = secondOffIndexes;
stanceData.secondOffInitialIndexes = secondOffInitialIndexes;
stanceData.secondOffInitialValues = secondOffInitialValues;
stanceData.secondOffIndexes = secondOffIndexes;
stanceData.secondMaxValues = secondMaxValues;
stanceData.secondMaxIndexes = secondMaxIndexes;

stanceData.secondVStrikeMeans = secondVStrikeMeans;
stanceData.secondAPStrikeMeans = secondAPStrikeMeans;
stanceData.secondMLStrikeMeans = secondMLStrikeMeans;
stanceData.secondVOffMeans = secondVOffMeans;
stanceData.secondAPOffMeans = secondAPOffMeans;
stanceData.secondMLOffMeans = secondMLOffMeans;

end

function [strikeMean, offMean] = evaluateMean(forceSamples, stanceIndexes, meanOffset)
    meanIndexes = max(stanceIndexes(1) - meanOffset, 1) : stanceIndexes(1);
    strikeMean = mean(abs(forceSamples(meanIndexes)));
    
    meanIndexes = stanceIndexes(end) : min(stanceIndexes(end) + meanOffset, stanceIndexes(end));
    offMean = mean(abs(forceSamples(meanIndexes)));
end