function [strikeIndex, offIndex, strikeValue, offValue, maxIndex, maxValue] ...
            = findStance(vForce, toBeVerbose)

% findStance determines boundary samples related to the stance phase of gait, 
% of a raw measurement data of the vertical ground reaction force.
% This function assumes that the measurement comprises 
% a single wave between series of insignificant samples.
% 
% This function uses constants that have been tuned 
% to match a particular source of measurements 
% (sampling freqency, signal dynamic range). 
% One must refine the constants in order to adapt this function 
% to another source of GRF signals.
%
% Input:
% - vForce : a vector of samples of the vertical ground reaction force.
% - toBeVerbose : 0 or 1. Disables/enables displaying diagnostic messages, 
%                 which were used to debug the code.
%
% Output:
% - strikeIndex : The index of the sample that begins the foot strike.
% - offIndex : The index of the sample that ends the foot off.
% - strikeValue : The value of the sample that begins the foot strike.
% - offValue : The value of the sample that ends the foot off.
% - maxIndex: The index of the maximum sample.
% - maxValue: The value of the maximum sample.
%
% Created on Sep. 27, 2021
%
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

% Firstly, the maximum sample is localized.
[maxValue, maxIndex] = max(vForce);
if (toBeVerbose)
    fprintf('findStance: maximum %f at %d\n', maxValue, maxIndex)
end

% By going back from the maximum sample, we look for the foot strike

strikeIndex = 0;
for k = maxIndex : -1 : 1

    % We assume that foots strike is related to a sample below 90 N,
    % in order to neglect flat fragments related to local maxima and minima
    %significanceThreshold = 90;     % Empirically determined       
    significanceThreshold = 80;     % Empirically determined       
    if (vForce(k) > significanceThreshold)   
         continue;
    end    

    % We consider 2 R samples that precede the current sample.
    % If there is too few samples, because we approach the record beginning,
    % then we decrease R, or shorten range

    R = 16;     % Empirically determined
    if ((k - 2 * R) < 1)
        R = floor((k - 1) / 2);
        if (toBeVerbose)
            warning('findStance: Strike-related sample range contracted to R = %d for k = %d', R, k)
        end
        if (R == 0)
            error('findStance: Strike-related sample range contracted to zero for k = %d', k)            
        end
    end
   
    tt = vForce((k - R) : (k - 1)) - vForce((k - R - R) : (k - 1 - R));
    
    %vForce((k - R) : (k - 1))
    
    t = tt ./ R;

    % By subracting corresponding samples and normalizing the differences
    % we obtaine a series of tangent estimates, or slope estimates.
          
    if ((max(abs(t)) > 0.91))       % Empirically determined
                                    % Tangents should be not too high
                                    % i.e. samples should rise/fall not too quickly
        if (toBeVerbose)            
            fprintf('findStance: Strike-tangent maximum too large %f\n', abs(t))
        end
        continue;
    end
    if (std(abs(t)) > 0.31)     % Empirically determined
                                % Tangents should oscillate not too much
                                % i.e. there should be no transient-related oscillations
        if (toBeVerbose)        
            fprintf('findStance: Strike-tangent std.dev. too large %f\n', std(abs(t)))
        end
        continue
    end
    
    if (toBeVerbose)
        disp(k)
        disp(t)
    end

    strikeIndex = k - 1;
    strikeValue = vForce(strikeIndex);
    break
          
end % for

% The foot off is determined in an analogous way, 
% by going forward from the maximum sample

offIndex = 0;
for k = maxIndex : length(vForce)    
    significanceThreshold = 90; 
    if (vForce(k) > significanceThreshold)
         continue;
    end    
    
    R = 24;
    if ((k + 2 * R) > length(vForce))
        R = floor((length(vForce) - k) / 2);
        if (toBeVerbose)
            warning('findStance: Off-related sample range contracted to R = %d for k = %d', R, k)      
        end
        if (R == 0)
            error('findStance: Off-related sample range contracted to zero for k = %d', k)            
        end        
    end
    
    
    tt = vForce((k + 1) :  (k + R)) - vForce((k + 1 + R) : (k + R + R));
    t = tt ./ R;
    
    if ((max(abs(t)) > 0.5)) 
        if (toBeVerbose)
            fprintf('findStance: Off-tangent maximum too large %f\n', abs(t))
        end
        continue;
    end
    if (std(abs(t)) > 0.16524)
        if (toBeVerbose)
            fprintf('findStance: Off-tangent std. dev. too large %f\n', std(abs(t)))
        end        
        continue;
    end

    offIndex = k + 1;
    offValue = vForce(offIndex);
    break
end % for

   % [strikeIndex, offIndex, strikeValue, offValue, maxIndex, maxValue]

end % function
