# -*- coding: utf-8 -*-
"""
This module-package provides functions that facilitate the use of the data set
of gait measurements described in the article

"A gaitData of measurements of ground reaction forces of normal gait 
of young adults wearing various footwear"
by Marcin Derlatka and Marek Parfieniuk

submitted to the "Scientific Data" journal.

Created on Sep. 1, 2021

@author: Marek Parfieniuk

Copyright (c) 2021 Marek Parfieniuk

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
"""


import numpy as np
import os
import re
import math
import matplotlib.pyplot as plt
import warnings
import traceback


def showMeasurement(gaitMeasurements, measurementIndexOrFileName, toSmoothStanceBoundaries, 
                    toBeVerbose, figureHandle = None) :
    """
    showMeasurement plots 
    - ground reaction forces (GRFs),
    - stance-related parts of the vertical GRF, possibly with smoothed tails
    - center of pressure (CoP),
    - vertical torque
    - vertical lines that indicate boundaries of the stance phase, 
      foot-strike/off events, Initial and refined (black/green colors),
      determined by using the findStance and cutStance functions, respectively
    of a preferred measurement of a given array of gait measurements.
    
    In addition to plots, useful information is presented:
    - measurement index and identifier (name of CSV file)
    - numbers of stance-related samples of both force plates
    - step duration computed as the difference between the indexes of the
      samples that begin the foot strikes of both force plates
    - step duration computed as the difference between the indexes of the
      samples that end the foot offs of both force plates
    - indexes of the boundary samples that end the foot offs of both force plates
    - indexes of the boundary samples that begin the foot strikes of both force plates
    
    The measurement array should be prepared by using the loadMeasurements function.
    
    The measurement can be selected by providing its index in the array
    or its identifier (name of CSV file, including extension).
    
    The function can use an existing figure window, 
    which will be cleared before creating new plots.
    If no figure handle is provided, then the function creates a new window.
    
    Input:
    - gaitMeasurements : array of structures each of which describes one gait measurement
    - measurementIndexOrFileName : index or string that identifies the
      measurement-file
    - toSmoothStanceBoundaries : 0 or 1; disables/enables extrapolation-based  
              refinement of boundaries of the stance-related parts
              of the vertical GRF signals; please see the cutStance function			
    - toBeVerbose : 0 or 1; disables/enables displaying diagnostic messages, 
                    which can be used for debugging.
    - figureHandle : (optional) the handle of a figure in which plots 
    		   should be created.
    
    Output:
    - A window is created or redrawn.
    
    In order to work correctly, this function needs the Tk GUI backend 
    to be installed and associated with the matplotlib library.
    In case of IPython, it maybe necessary to enable a graphics backend that
    allows for opening GUI windows outside the console, by modifying 
        python > preferences > IPython console > Graphics > Graphics backend 
    
    Created on Sep. 3, 2021
    
    @author: Marek Parfieniuk
    """
    

    if (type(measurementIndexOrFileName) == str) :
        measurementFileName = measurementIndexOrFileName
        measurementIndex = 0;        
        for k in range(1, len(gaitMeasurements)) :            
            gaitMeasurements[k].keys
            if (gaitMeasurements[k]['fileName'] == measurementFileName) :
                measurementIndex = k;
                break
    else :
        measurementIndex = int(measurementIndexOrFileName)
        measurementFileName = gaitMeasurements[measurementIndex]['fileName'];    

    print(measurementIndex)
    print(measurementFileName)

    
    measurementRecord = gaitMeasurements[measurementIndex]

    firstVForce = measurementRecord['firstVForce'].copy();
    firstAPForce = measurementRecord['firstAPForce'].copy();
    firstMLForce = measurementRecord['firstMLForce'].copy();
    
    secondVForce = measurementRecord['secondVForce'].copy();
    secondAPForce = measurementRecord['secondAPForce'].copy();
    secondMLForce = measurementRecord['secondMLForce'].copy();
        
    [firstStrikeInitialIndex, firstOffInitialIndex, firstStrikeValue, firstOffValue, firstMaxIndex, firstMaxValue] \
        = findStance(firstVForce, toBeVerbose)

    [firstWave, firstStrikeIndex, firstOffIndex] = cutStance(firstVForce, 
            firstStrikeInitialIndex, firstOffInitialIndex, toSmoothStanceBoundaries)
    
    [secondStrikeInitialIndex, secondOffInitialIndex, secondStrikeValue, secondOffValue, secondMaxIndex, secondMaxValue] \
        = findStance(secondVForce, toBeVerbose)
        
    [secondWave, secondStrikeIndex, secondOffIndex] = cutStance(secondVForce, 
            secondStrikeInitialIndex, secondOffInitialIndex, toSmoothStanceBoundaries)                
        

    [isLeftLegFirst, legOrderRuleIndex] = checkLeftLegFirst(firstMLForce[firstStrikeIndex : firstOffIndex], secondMLForce[secondStrikeIndex : secondOffIndex], 0)

    if (isLeftLegFirst) :        
        firstLabel = 'First plate, left leg'
        secondLabel = 'Second plate, right leg'
        legOrder = "Leg order: left leg on first plate"    
    else :
        firstLabel = 'First plate, right leg'
        secondLabel = 'Second plate, left leg'
        legOrder = "Leg order: right leg on first plate"    

    if (figureHandle == None) :            # No figure handle,
        figureHandle = plt.figure()            # so let us create one.
    else :
        plt.figure(figureHandle.number)
    
    plt.clf()
        
    resizeAndEntitleWindow(figureHandle, 
                           'measurement #{:d} : {:s}'.format(measurementIndex, measurementFileName))
                
    axs = figureHandle.subplots(3, 2, sharey=False)
    
    ax = axs[2, 1]    

    # We neglect samples before and after (up to some margin) the stance phase, as they usually are noisy 
    
    noiseIndexMargin = 20;  
    
    firstAPCoP = measurementRecord['firstAPCoP'] \
                    [(firstStrikeIndex + noiseIndexMargin) : (firstOffIndex - noiseIndexMargin)];
    firstMLCoP = measurementRecord['firstMLCoP'] \
                    [(firstStrikeIndex + noiseIndexMargin) : (firstOffIndex - noiseIndexMargin)];    
    secondAPCoP = measurementRecord['secondAPCoP'] \
                    [(secondStrikeIndex + noiseIndexMargin) : (secondOffIndex - noiseIndexMargin)];
    secondMLCoP = measurementRecord['secondMLCoP'] \
                    [(secondStrikeIndex + noiseIndexMargin) : (secondOffIndex - noiseIndexMargin)];
    
    ax.plot(firstAPCoP * 100, firstMLCoP * 100, 'b', label = firstLabel) #, 'Color', 'b')
    ax.plot(secondAPCoP * 100, secondMLCoP * 100, 'r', label = secondLabel) #, 'Color', 'r')
    ax.set_xlabel('Anterior-posterior coordinate of CoP [cm]')
    ax.set_ylabel('Medio-lateral coordinate of CoP [cm]')
    ax.grid()
    ax.legend()
    
    noiseIndexMargin = 3;
    
    firstVTorque = measurementRecord['firstVTorque']
    firstVTorque[1 : (firstStrikeIndex + noiseIndexMargin)] = 0    
    firstVTorque[(firstOffIndex - noiseIndexMargin) : ] = 0
    
    secondVTorque = measurementRecord['secondVTorque'];
    secondVTorque[1 : (secondStrikeIndex + noiseIndexMargin)] = 0
    secondVTorque[(secondOffIndex - noiseIndexMargin) : ] = 0
    
    ax = axs[1, 1]
    secondVTorque = measurementRecord['secondVTorque'];
    firstVTorque = measurementRecord['firstVTorque'];
    ax.plot(firstVTorque, 'b', label = firstLabel)
    ax.plot([firstStrikeIndex, firstStrikeIndex], [-10, 10], 'k--')
    ax.plot([firstOffIndex, firstOffIndex], [-10, 10], 'k--')    
    ax.plot(secondVTorque, 'r', label = secondLabel)
    ax.plot([secondStrikeIndex, secondStrikeIndex], [-10, 10], 'k--')
    ax.plot([secondOffIndex, secondOffIndex], [-10, 10], 'k--')
    ax.set_ylabel('Vertical torque [N m]')
    ax.set_xlabel("Sample index")
    ax.grid()
    ax.legend()
    ax.set_ylim([-20, 20])

    ax = axs[0, 1]
    ax.text(0.5, 0.5, 'Measurement #{:d} : {:s}'.format(measurementIndex, measurementFileName) + '\n'            
        + '\n'
        + legOrder
        + '\n'
        + '\nFirst-plate stance phase duration: {} ({}) samples'.format(firstOffIndex - firstStrikeIndex + 1, firstOffInitialIndex - firstStrikeInitialIndex  + 1)
        + '\nSecond-plate stance phase duration:   {} ({}) samples'.format(secondOffIndex - secondStrikeIndex + 1, secondOffInitialIndex - secondStrikeInitialIndex + 1)
        + '\nFoot-strike-based step duration:  {} ({}) samples'.format(secondStrikeIndex - firstStrikeIndex + 1, secondStrikeInitialIndex - firstStrikeInitialIndex + 1)
        + '\nFoot-off-based step duration:      {} ({}) samples'.format(secondOffIndex - firstOffIndex + 1, secondOffInitialIndex - firstOffInitialIndex  + 1)
        + '\n'
        + '\nFirst-plate strike/off at: {} / {} ({} / {}) sample'.format(firstStrikeIndex, firstOffIndex, firstStrikeInitialIndex, firstOffInitialIndex)
        + '\nSecond-plate strike/off at: {} / {} ({} / {}) sample'.format(secondStrikeIndex, secondOffIndex, secondStrikeInitialIndex, secondOffInitialIndex)
        + '\n\n    Estimates of strike/off times shown as vertical lines:'
        + '\n      refined = green (initial = black)'
        , fontsize='medium', fontweight='bold'
        , horizontalalignment='center', verticalalignment='center'
        )
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)

    ax = axs[0, 0]

    ax.plot(np.arange(firstStrikeIndex, firstOffIndex + 1), firstWave, 'g')
    ax.plot(firstVForce, 'b', label = firstLabel)
    ax.plot([firstStrikeInitialIndex, firstStrikeInitialIndex], [-100, 600], 'k--')
    ax.plot([firstOffInitialIndex, firstOffInitialIndex], [-100, 600], 'k--')
    ax.plot([firstStrikeIndex, firstStrikeIndex], [-100, 600], 'g--')
    ax.plot([firstOffIndex, firstOffIndex], [-100, 600], 'g--')
    
    ax.plot(np.arange(secondStrikeIndex, secondOffIndex + 1), secondWave, 'g')
    ax.plot(secondVForce, 'r', label = secondLabel)
    ax.plot([secondStrikeInitialIndex, secondStrikeInitialIndex], [-100, 600], 'k--')
    ax.plot([secondOffInitialIndex, secondOffInitialIndex], [-100, 600], 'k--')
    ax.plot([secondStrikeIndex, secondStrikeIndex], [-100, 600], 'g--')   
    ax.plot([secondOffIndex, secondOffIndex], [-100, 600], 'g--')
    
    ax.set_ylabel('Vertical GRF [N]')
    ax.set_xlabel("Sample index")
    ax.grid()
    ax.legend()

    ax = axs[1, 0]
    ax.plot(firstAPForce, 'b', label = firstLabel)
    ax.plot([firstStrikeInitialIndex, firstStrikeInitialIndex], [-150, 150], 'k--')
    ax.plot([firstOffInitialIndex, firstOffInitialIndex], [-150, 150], 'k--')
    ax.plot([firstStrikeIndex, firstStrikeIndex], [-150, 150], 'g--')
    ax.plot([firstOffIndex, firstOffIndex], [-150, 150], 'g--')
    ax.plot(secondAPForce, 'r', label = secondLabel)
    ax.plot([secondStrikeInitialIndex, secondStrikeInitialIndex], [-150, 150], 'k--')
    ax.plot([secondOffInitialIndex, secondOffInitialIndex], [-150, 150], 'k--')
    ax.plot([secondStrikeIndex, secondStrikeIndex], [-150, 150], 'g--')
    ax.plot([secondOffIndex, secondOffIndex], [-150, 150], 'g--')

    ax.set_ylabel('Anterior-posterior GRF [N]')
    ax.set_xlabel("Sample index")
    ax.grid()
    ax.legend()

    ax = axs[2, 0]
    ax.plot(firstMLForce, 'b', label = firstLabel)
    ax.plot([firstStrikeInitialIndex, firstStrikeInitialIndex], [-50, 50], 'k--')
    ax.plot([firstOffInitialIndex, firstOffInitialIndex], [-50, 50], 'k--')
    ax.plot([firstStrikeIndex, firstStrikeIndex], [-50, 50], 'g--')
    ax.plot([firstOffIndex, firstOffIndex], [-50, 50], 'g--')
    ax.plot(secondMLForce, 'r', label = secondLabel)
    ax.plot([secondStrikeInitialIndex, secondStrikeInitialIndex], [-50, 50], 'k--')
    ax.plot([secondOffInitialIndex, secondOffInitialIndex], [-50, 50], 'k--')
    ax.plot([secondStrikeIndex, secondStrikeIndex], [-50, 50], 'g--')
    ax.plot([secondOffIndex, secondOffIndex], [-50, 50], 'g--')

    ax.set_ylabel('Medio-lateral GRF [N]')
    ax.set_xlabel("Sample index")
    ax.grid()
    ax.legend()
    ax.set_ylim([-100, 100])

    print('Shown measurement #{:d} : {:s}'.format(measurementIndex, measurementFileName))

    plt.draw()

    return [figureHandle, measurementRecord]

   
def findStance(vForce, toBeVerbose) :
    """
    findStance determines boundary samples related to the stance phase of gait, 
    of a raw measurement data of the vertical ground reaction force.
    This function assumes that the measurement comprises 
    a single wave between series of insignificant samples.
    
    This function uses constants that have been tuned 
    to match a particular source of measurements 
    (sampling freqency, signal dynamic range). 
    One must refine the constants in order to adapt this function 
    to another source of GRF signals.
    
    Input:
    - vForce : a vector of samples of the vertical ground reaction force.
    - toBeVerbose : 0 or 1. Disables/enables displaying diagnostic messages, 
                    which were used to debug the code.
    
    Output:
    - strikeIndex : The index of the sample that begins the foot strike.
    - offIndex : The index of the sample that ends the foot off.
    - strikeValue : The value of the sample that begins the foot strike.
    - offValue : The value of the sample that ends the foot off.
    - maxIndex: The index of the maximum sample.
    - maxValue: The value of the maximum sample.


    Created on Sep. 27, 2021
                
    @author: Marek Parfieniuk
    """      
    
    # Firstly, the maximum sample is localized.       
    maxIndex = vForce.argmax()
    maxValue = vForce[maxIndex]   
    if (toBeVerbose) :
        print('findStance: maximum {:f} at {:d}\n', maxValue, maxIndex)
    
    # By going back from the maximum sample, we look for the foot strike
    # The foot strike moment is determined by the first sample that is preceded
    # by series of samples that form slope of limited raise/fall.
    # This imples that the sample is followed by a higly rising slope.

    strikeIndex = 0;
    for kk in range(maxIndex + 1, 1, -1) :
        k = kk - 1;
        # We go sample by sample

        
        # We assume that foots strike is related to a sample below 80 N,
        # in order to neglect flat fragments related to local maxima and minima

        significanceThreshold = 80      # Empirically determined            
        if (vForce[k] > significanceThreshold) :     
             continue                           

        # We consider 2 R samples that precede the current sample.        
        # If there is too few samples, because we approach the measurement beginning,
        # then we decrease R, or shorten range

        R = 16;         # Empirically determined
        if (k < (2 * R)) :
            R = math.floor((k - 0) / 2);
            if (toBeVerbose) :
                print('findStance: Strike-related sample range contracted to R = {} for k = {}'.format(R, k))                
            if (R == 0) :
                raise Exception('findStance: Strike-related sample range contracted to zero for {}'.format(k))

        # Slope is determined by comparing R samples before current sample
        # to the erlier R samples.           

        x1 = vForce[(k - R) : k]
        #print(x1)
        x2 = vForce[(k - R - R) : (k - R)]          
        tt = x1 - x2
        t = tt / R
        #print(t)

        # By subracting corresponding samples and normalizing the differences
        # we obtaine a series of tangent estimates, or slope estimates.
    
        if (np.max(np.abs(t)) > 0.91) :     # Empirically determined
                                            # Tangents should be not too high
                                            # i.e. samples should rise/fall not too quickly
            if (toBeVerbose) :              
                print('findStance: Strike-tangent maximum too large {}'.format(np.abs(t)))
            continue
    
        if (np.std(np.abs(t), ddof = 1) > 0.31) :       # Empirically determined
                                                        # Tangents should oscillate not too much
                                                        # i.e. there should be no transient-related oscillations
            if (toBeVerbose) :                          
                print('findStance: Strike-tangent std. dev. too large {}'.format(np.std(np.abs(t), ddof = 1)))
            continue

        if (toBeVerbose) :
            print(k)
            print(t)
            print(tt)
    
        strikeIndex = k - 1;
        strikeValue = vForce[strikeIndex]
        break
    
    # The foot off is determined in an analogous way, 
    # by going forward from the maximum sample

    offIndex = 0;
    for k in range(maxIndex, len(vForce)) :
        significanceThreshold = 90; 
        if (vForce[k] > significanceThreshold) :
             continue
    
        R = 24;
        if ((k + 2 * R + 1) > len(vForce)) :
            R = math.floor((len(vForce) - k - 1) / 2)
            if (toBeVerbose) :
                print('findStance: Off-related sample range contracted to R = {}  for k = {}'.format(R, k))                
            if (R == 0) :
                raise Exception('findStance: Off-related sample range contracted to zero for k = {}'.format(k))
       
        x1 = vForce[(k + 1) : (k + R + 1)]
        x2 = vForce[(k + R + 1) : (k + R + R + 1)]
        tt = x1 - x2
        t = tt / R
        
        if (np.max(np.abs(t)) > 0.5) :
            if (toBeVerbose) :
                print('findStance: Off-tangent maximum too large {}'.format(np.abs(t)))                        
            continue
        
        if (np.std(np.abs(t), ddof = 1) > 0.16524) :
            if (toBeVerbose) :
                print('findStance: Off-tangent std.dev. too large {}'.format(np.std(np.abs(t), ddof = 1)))
            continue        
    
        offIndex = k + 1;
        offValue = vForce[offIndex];
        break    
    
    #print([strikeIndex, offIndex, strikeValue, offValue, maxIndex, maxValue])
    
    return [strikeIndex, offIndex, strikeValue, offValue, maxIndex, maxValue]



def loadMeasurements(fileDirectoryPath, fileNamePattern) :

    """
    loadMeasurements reads CSV files of given directory, 
    only those whose names match a given pattern (regular expression).
    It loads gait measurement data (samples of signals of force plates) 
    from those files into an array of dictionaries.
    Each dictionary is composed of fields-arrays 
    which represent name and columns 
    of one CSV file (one measurement).
    
    The file name (without extension) can be used 
    as the measurement identifier-label.
    
    This function assumes that the directory contains person/participant-related
    subdirectiories '001'...'324' that contain shoe-related subdirectiories 
    that contain only measurement-related .CSV files.
    Errors might result from presence of other subdirectories and files
    in participant-related subdirectories.
    
    This function temporarily changes the working directory. 
    In case of error, it does not return to the starting directory.
    
    Input:
    - fileDirectoryPath : path to directory with measurement files
    - fileNamePattern : pattern (regular expression) for file names
    
    Output:
    - gaitMeasurements : cell array of dictionaries of column vectors 
                          of samples of signals of force plates 
           
    Examples of file-name patterns:
    - r'.*' : All measurements
    - r'.*\.f\..*' : All female-related measurements
    - r'.*\.m\..*' : All male-related measurements
    - r'024.*' : All measurements related to the 24th participant, regardless of shoes
    - r'024.*\.spr.*' : All measurements related to the 24th participant
                           in sport shoes        
    
    Usage example:
    - gaitMeasurements = loadMeasurements(r'c:\\gaitMeasurementsInCSVFiles', r'.*')
    
    Created on Sep. 21, 2021
                
    @author: Marek Parfieniuk
    """    


    currentDirectory= os.getcwd()
        
    fileNameRegex = re.compile(fileNamePattern)
    
    os.chdir(fileDirectoryPath)
    
    gaitMeasurements = np.empty(100000, dtype=object)
    
    # WARNING! We iterate over a fixed range of participant identifiers
    
    fileCount = 0;
    
    for participantIndex in range(1, 325) : #16) : #325) :

        participantDirectory = str("{:03}").format(participantIndex)
        os.chdir(participantDirectory)    
        
        shoeDirectory = os.listdir()
        
        print("Participant #{}".format(participantDirectory))
        
        for shoeDirectory in os.listdir():
            
            print("  Shoe #{}".format(shoeDirectory))
            
            os.chdir(shoeDirectory)
            for fileName in os.listdir() :  
                if fileName.endswith('ptrn.csv') :
                    continue                
                if (fileNameRegex.fullmatch(fileName) == None) :
                    # print("\tSkipped, as it does not match the pattern.")
                    continue

    # An empty matrix is preallocated to store signal samples.
    # It has 10000 rows, more than necessary, so to avoid memory reallocation.
    # After loading data from file, we trim the matrix, 
    # so as to remove empty rows.

                print('   {:s} is processed'.format(fileName))
                    
                data = np.empty([10000, 12], dtype=np.double)
    
                f = open(fileName, "r")
                lineIndex = -1;
                for line in f : # We iterate over lines.
                    if (lineIndex == -1) :
                        lineIndex = 0
                        continue    # We skip the header line
                    
                    t = line.split(',') 
                    data[lineIndex, 0] = float(t[0])
                    data[lineIndex, 1] = float(t[1])
                    data[lineIndex, 2] = float(t[2])
                    data[lineIndex, 3] = float(t[3])
                    data[lineIndex, 4] = float(t[4])
                    data[lineIndex, 5] = float(t[5])
                    data[lineIndex, 6] = float(t[6])
                    data[lineIndex, 7] = float(t[7])
                    data[lineIndex, 8] = float(t[8])
                    data[lineIndex, 9] = float(t[9])
                    data[lineIndex, 10] = float(t[10])
                    data[lineIndex, 11] = float(t[11])
                    lineIndex = lineIndex + 1
                data = data[: lineIndex, :]
                f.close()
                
                t = dict(
                        fileName = fileName,
                             
                        firstVForce = data[:, 0], 
                        firstAPForce = data[:, 1],
                        firstMLForce = data[:, 2],
                        firstVTorque = data[:, 3], 
                        firstAPCoP = data[:, 4], 
                        firstMLCoP = data[:, 5], 
    
                        secondVForce = data[:, 6], 
                        secondAPForce = data[:, 7],
                        secondMLForce = data[:, 8],
                        secondVTorque = data[:, 9], 
                        secondAPCoP = data[:, 10], 
                        secondMLCoP = data[:, 11], 
                        )
                gaitMeasurements[fileCount] = t
                
                fileCount = fileCount + 1      
                        
            os.chdir("..")
        os.chdir("..")
    os.chdir(currentDirectory)    
    
    gaitMeasurements = gaitMeasurements[: fileCount]
    
    print('Done! {} CSV files have been processed.'.format(fileCount))

    return gaitMeasurements



def loadParticipants(filePath) :
    '''
    loadParticipants loads the participant data 
    from a given CSV file into a structure of column vectors, 
    each of which contains data of one kind (index-identifier, weight, etc.)
    of all participants.
    
    loadParticipants returns a dictionary with the following fields:
    - index (string from '001' to '324')
    - gender (as string; 'f' for female; 'm' for male)
    - date of participant measurement, in the 'yyyy-mm-dd' format)
    - age
    - weight
    - height
    each of which is an array that contains the corresponding data
    of all participants (data of one column of the CSV files).
    
    Examples of accessing output data:
    - participantData.gender(13) : gives the gender of the 13th participant
    - participantData.weight(participant.gender == 'f') : gives weights of all females  
    
    The index field can be used to associate participant's data with
    measurements of his/her gait.
    
    Input:
    - filePath : path to a CSV file (participantData.csv) with participant data
    
    Output:
    - participantData : dictionary of arrays that contain participant data

    Data aceess examples:
    - participantData['gender'][13] : gives gender of the 13th participant
    - participantData['weight'][participant['gender' == 'f']] : gives weights of all females  

    Created on Nov. 12, 2021
    
    Author: Marek Parfieniuk
    '''
    
    # The 'object' data type is used to store strings as vector elements.
    # The 'str' data type works inappropriately, as only one, first character 
    # is assigned to vector element.    
    
    identifiers = np.empty(10000, dtype=object)
    genders = np.empty(10000, dtype = object)
    dates = np.empty(10000, dtype = object)
    ages = np.empty(10000, dtype = float)
    weights = np.empty(10000, dtype = float)
    heights = np.empty(10000, dtype = float)
    
    f = open(filePath, "r")
    lineIndex = -1
    for line in f:      # iterate over lines
        if (lineIndex == -1) :
            lineIndex = 0
            continue    # We skip the header line
        
        t = line.split(',')
        
        identifiers[lineIndex] = t[0]
        genders[lineIndex] = t[1]
        dates[lineIndex] = t[2]
        ages[lineIndex] = float(t[3]) 
        weights[lineIndex] = float(t[4]) 
        heights[lineIndex] = float(t[5])  
        lineIndex = lineIndex + 1
    f.close()
    
    identifiers = identifiers[:lineIndex]
    genders = genders[:lineIndex]
    dates = dates[:lineIndex]
    ages = ages[:lineIndex]
    weights = weights[:lineIndex]
    heights = heights[:lineIndex]
    
    participantData = dict(identifiers = identifiers, genders = genders, 
            dates = dates, ages = ages, weights = weights, heights = heights)
   
    return participantData



def summarizeParticipants(participantData) :
    '''    
    summarizeParticipants shows histograms and statistics that characterize 
    distributions of ages, weights, and heights of the persons who served as
    participants of gait measurements.
    
    Additionally, this function creates two figures which show
    - histograms of ages, weights, heights
    - 3D histograms of weight-height combinations; rotated so as to look like
      2D scatter plots
    of all participants, of only females, and of only males.
    
    This function can be used to compare females against males, 
    and to identify participants that could be outliers, 
    especially overweight persons.
    
    Input:
    - participantData : dictionary of participant data, prepared 
                    by using the loadParticipants function.
            
    Output:
    - Text and figures are displayed.
            
    Created on Sep. 28, 2021

    Author: Marek Parfieniuk    
    '''
    
    print('Number of males: {:5d}.'.format(np.sum(participantData['genders'] == 'm')))
    print('Number of females: {:5d}.'.format(np.sum(participantData['genders'] == 'f')))
    print('')
    t = participantData['weights']
    print('Average female''s weigth: {:5.2f} kg.'.format(np.mean(t[participantData['genders'] == 'f'])))
    print('Average male''s weigth: {:5.2f} kg.'.format(np.mean(t[participantData['genders'] == 'm'])))    
    print('')
    t = participantData['heights']
    print('Average female''s heigth: {:5.2f} cm.'.format(np.mean(t[participantData['genders'] == 'f'])))
    print('Average male''s heigth: {:5.2f} cm.'.format(np.mean(t[participantData['genders'] == 'm'])))
    print('')
    print('Average participant''s age: {:5.2f} years.'.format(np.mean(participantData['ages'])))        
    
    
    ageBins = np.arange(15, 30, 1)
    weightBins = np.arange(40, 160, 5)
    heightBins = np.arange(140, 220, 5)

    # 2D histograms
        
    fig, axs = plt.subplots(1, 3, sharey = False, tight_layout = True)
    resizeAndEntitleWindow(fig, 'Participant-related 2D histograms')
    
    
    ax = axs[0]
    ax.hist2d(participantData['weights'][:], participantData['heights'][:], bins = [weightBins, heightBins])
    ax.set_title('Number of participants (both genders)')
    ax.set_xlabel('Weight [kg]')
    ax.set_ylabel('Height [cm]')
    # ax.set_ylim(0, 150)
    ax.grid(True)
    
    
    ax = axs[1]
    ax.hist2d(participantData['weights'][participantData['genders'] == 'f']
        , participantData['heights'][participantData['genders'] == 'f']
        , bins=[weightBins, heightBins])
    ax.set_title('Number of females')
    ax.set_xlabel('Weight [kg]')
    ax.set_ylabel('Height [cm]')
    # ax.set_ylim(0, 150)
    ax.grid(True)        
    
    ax = axs[2]
    [counts, xedges, yedges, im] = ax.hist2d(participantData['weights'][participantData['genders'] == 'm']
        , participantData['heights'][participantData['genders'] == 'm']
        , bins = [weightBins, heightBins])
    ax.set_title('Number of males')
    ax.set_xlabel('Weight [kg]')
    ax.set_ylabel('Height [cm]')
    ax.grid(True)
    plt.colorbar(im)
        
    # 1D histograms
    
    fig, axs = plt.subplots(3, 3, sharey=False, tight_layout=True)
    resizeAndEntitleWindow(fig, 'Participant-related 1D histograms')
    
    # 1st row, age
    
    ax = axs[0, 0]
    t = participantData['ages'][:]
    ax.hist(t, bins = ageBins)
    ax.set_ylabel('Number of participants (both genders)')
    ax.set_xlabel('Age [year]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 150)
    ax.grid(True)
    
    ax = axs[0, 1]
    t = participantData['ages'][participantData['genders'] == 'f']
    ax.hist(t, bins = ageBins)
    ax.set_ylabel('Number of females')
    ax.set_xlabel('Age [year]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 150)
    ax.grid(True)
    
    ax = axs[0, 2]
    t = participantData['ages'][participantData['genders'] == 'm']
    ax.hist(t, bins = ageBins)
    ax.set_ylabel('Number of males')
    ax.set_xlabel('Age [year]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 150)
    ax.grid(True)
    
    # 2nd row, weight
    
    ax = axs[1, 0]
    t = participantData['weights'][:]
    ax.hist(t, bins=weightBins)
    ax.set_ylabel('Number of participants (both genders)')
    ax.set_xlabel('Weight [kg]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 50)
    ax.grid(True)
    
    ax = axs[1, 1]
    t = participantData['weights'][participantData['genders'] == 'f']
    ax.hist(t, bins = weightBins)
    ax.set_ylabel('Number of females')
    ax.set_xlabel('Weight [kg]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 50)
    ax.grid(True)
    
    ax = axs[1, 2]
    t = participantData['weights'][participantData['genders'] == 'm']
    ax.hist(t, bins = weightBins)
    ax.set_ylabel('Number of males')
    ax.set_xlabel('Weight [kg]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 50)
    ax.grid(True)
       
    # 3rd row, height
    
    ax = axs[2, 0]
    t = participantData['heights'][:]
    ax.hist(t, bins = heightBins)
    ax.set_ylabel('Number of participants (both genders)')
    ax.set_xlabel('Height [cm]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 80)
    ax.grid(True)
    
    ax = axs[2, 1]
    t = participantData['heights'][participantData['genders'] == 'f']
    ax.hist(t, bins = heightBins)
    ax.set_ylabel('Number of females')
    ax.set_xlabel('Height [cm]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 80)
    ax.grid(True)
    
    ax = axs[2, 2]
    t = participantData['heights'][participantData['genders'] == 'm']
    ax.hist(t, bins = heightBins)
    ax.set_ylabel('Number of males')
    ax.set_xlabel('Height [cm]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_ylim(0, 80)
    ax.grid(True)
    
    plt.draw()



def cutStance(vForce, strikeInitialIndex, offInitialIndex, toSmoothBoundaries, toBeVerbose = 0) :
    '''
    cutStance cuts out stance-related samples
    from measurement of the vertical ground reaction force.
    The stance-related fragment of the GRF signal must be indicated by
    providing estimated indexes of boundary samples related to foot strike/off events. 
    These indexes should be determined by using the findStance function.
    
    cutStance can refine boundary indexes 
    and replace original samples with extrapolated values,
    so as to return a series of non-negative values which has smoothed tails 
    and begins and ends with zero-valued samples.
    
    The output refined indexes allow for mapping the resulting sample series
    onto the original signal.
    
    Input:
    - vForce : a vector of samples of the vertical ground reaction force.
    - strikeInitialIndex : The initial estimate of the index of the sample 
                      that begins the foot strike.                 .
    - offInitialIndex : The initial estimate of the index of the sample
                      that ends the foot off.
    - toSmoothBoundaries : 0 or 1. Disables/enables smoothing of boundaries 
                      of sample series
    - toBeVerbose : 0 or 1. Disables/enables displaying diagnostic messages, 
                      which were used for debugging.
    
    Output:
    - stanceSamples : A series of samples related to the stance phase of gait.
    - strikeIndex : The index of the (possibly extrapolated) zero sample 
                      that is estimated to precede the foot strike.
    - offIndex : The index of the (possibly extrapolated) zero sample 
                      that is estimated to follow the foot off.

    
     Created on Sep. 27, 2021
    
     @author: Marek Parfieniuk    
     '''
    
    if (not toSmoothBoundaries) :                 # No wave refinement
        strikeIndex = strikeInitialIndex
        offIndex = offInitialIndex
        stanceSamples = vForce[strikeIndex : (offIndex + 1)]
        return [stanceSamples, strikeIndex, offIndex]
    
    # Refinement of the foot-strike-related tail of the wave
    
    vForce = vForce.copy()    # copy the signal as the algorithm modifies samples
    
    strikeValue = vForce[strikeInitialIndex]
    strikeIndex = strikeInitialIndex;
    if (strikeValue > 0) :  # Positive sample extrapolation
        
        # We analyze the signal-slope 
        # at the initial estimate of the foot-strike moment
        # in order to estimate the difference for linear extrapolation

        # We consider two samples before and after the the foot-strike moment
        # so as to get the average difference between subsequent samples
        
        dd = np.diff(vForce[(strikeIndex - 2) : (strikeIndex + 2 + 1)])
        dd = dd[dd > 0]     # Values should decrease toward wave tail
        
        if (len(dd) > 0) :
            d = np.mean(dd)
        else :
            d = 0                
        
        if (d > 0) :
            # Difference estimated successfully 
            if (d < 0.1 * strikeValue) :    # but values cannot 
                d = strikeValue / 10        # decrease too slowly 
        else :
            if (toBeVerbose) :
                warnings.warn('cutVFWave: unsuccessful estimation of difference for strike extrapolation {:f}'.format(d))
            d = strikeValue;    # Difference set so as to cut the wave in one step
                    
        while (True) :                  # Samples extrapolated one-by-one
            if (strikeValue >= d) :     # values decreased by the difference
                strikeValue = strikeValue - d
                strikeIndex = strikeIndex - 1
                vForce[strikeIndex] = strikeValue
            else :                          # until crossing zero
                if (strikeValue > np.abs(strikeValue - d)) :
                    strikeIndex = strikeIndex - 1
                vForce[strikeIndex] = 0
                break;
    elif (strikeValue < 0) :    # When the initial sample is negative
        while (1) :                         # we go backward and omit samples
            t = vForce[strikeIndex + 1]      # until crossing zero
            if (t < 0) :
                strikeValue = t
                strikeIndex = strikeIndex + 1
            else :
                if (np.abs(strikeValue) >= t) :
                    strikeIndex = strikeIndex + 1
                vForce[strikeIndex] = 0
                break;            
    
    # In analogous way, we refine the foot-off-related tail of the wave
    
    offValue = vForce[offInitialIndex]
    offIndex = offInitialIndex
    if (offValue > 0) : # Positive sample extrapolation

        dd = np.diff(vForce[(offIndex + 2) : (offIndex - 2 - 1) : -1])
        dd = dd[dd > 0]
        
        if (len(dd) > 0) :
            d = np.mean(dd)
        else :
            d = 0          
        
        if (d > 0) :
            if (d < 0.1 * offValue) :
                d = offValue / 10;    
        else :
            if (toBeVerbose) :
                warnings.warn('cutVFWave: unsuccessful estimation of difference for off extrapolation {:f}'.format(d))
            d = offValue;
                   
        while (1) :
            if (offValue >= d) :
                offValue = offValue - d
                offIndex = offIndex + 1
                vForce[offIndex] = offValue
            else :
                if (offValue > np.abs(offValue - d)) :
                    offIndex = offIndex + 1
                vForce[offIndex] = 0
                break
    elif (offValue < 0) :
        while (1) :
            t = vForce[offIndex - 1]
            if (t < 0) :
                offValue = t
                offIndex = offIndex - 1
            else :
                if (np.abs(offValue) >= t) :
                    offIndex = offIndex - 1                
                vForce[offIndex] = 0;
                break;                        
    
    stanceSamples = vForce[strikeIndex : (offIndex + 1)]
    stanceSamples[stanceSamples < 0] = 0
        
    return [stanceSamples, strikeIndex, offIndex]



def showPatterns(patternData, fileNamePattern) :
    """
    showPatterns plots stance-related parts of gait measurements 
    whose file names match a given pattern (regular expression).
    In particular, measurements related to a subject and shoe type can be selected.
    
    This function can be used to compare waveforms 
    in order to identify trends and outliers.
    
    Input:
    - gaitPatterns : sstance-related data prepared by using the extractStanceData
                 function
    - fileNamePattern : pattern (regular expression) for file names
            
    Output:
    - Figure is displayed.
            
    Created on May 06, 2022
    
    Author: Marek Parfieniuk
    """
        
    selectedPatterns = []
    
    fileNameRegex = re.compile(fileNamePattern)
               
    for patternIndex in range(0, len(patternData['fileNames'])) :        
        # print(patternData['fileNames'])
        # print(patternData['fileNames'][patternIndex])
        if (fileNameRegex.fullmatch(patternData['fileNames'][patternIndex]) == None) :
            # print("\tSkipped, as it does not match the pattern.")
            continue;    
                           
        t = dict(fileName = patternData['fileNames'][patternIndex],
                 leftMLForce = patternData['leftMLForces'][patternIndex, :],
                 leftAPForce = patternData['leftAPForces'][patternIndex, :],
                 leftVForce = patternData['leftVForces'][patternIndex, :], 
                 rightMLForce = patternData['rightMLForces'][patternIndex, :],
                 rightAPForce = patternData['rightAPForces'][patternIndex, :],
                 rightVForce = patternData['rightVForces'][patternIndex, :]) 

        print('Found measurement {:5d} : {}'.format(patternIndex, t['fileName']))
       
        selectedPatterns.append(t)
        
        if (len(selectedPatterns) > 150) :
            print('More than 150 patterns found. It seems pointless to show so many plots. Please change the pattern for file names.')
            return
        
        
    if (len(selectedPatterns) == 0) :
        print('No measurement found. Please try another file name pattern.')
        return

    [fig, axs] = plt.subplots(3, 3, sharey = False, tight_layout = True, squeeze = False)
    
    resizeAndEntitleWindow(fig, 'Time-normalized stance-related fragments of measurements of files matching "{:s}"'.format(fileNamePattern))

    ax = axs[0][0]
    ax.grid(True)
    for k in range(0, len(selectedPatterns)) :
       ax.plot(selectedPatterns[k]['leftVForce'], label = selectedPatterns[k]['fileName']) 
    ax.set_ylabel("Left-leg vertical GRF [N]")
    ax.set_xlabel("Sample index")
        
    ax = axs[0][1]
    ax.grid(True)
    for k in range(0, len(selectedPatterns)) :
       ax.plot(selectedPatterns[k]['rightVForce'], label = selectedPatterns[k]['fileName']) 
    ax.set_ylabel("Right-leg vertical GRF [N]")
    ax.set_xlabel("Sample index")    
    ax.legend(loc='upper left', bbox_to_anchor=(1.2, 0.5, 0.9, 0.5), borderaxespad=0.1)    

    ax = axs[0][2]
    ax.set_visible(False)

    ax = axs[1][0]
    ax.grid(True)
    for k in range(0, len(selectedPatterns)) :
       ax.plot(selectedPatterns[k]['leftAPForce'], label = selectedPatterns[k]['fileName']) 
    ax.set_ylabel("Left-leg anterior-posterior GRF [N]")
    ax.set_xlabel("Sample index")
        
    ax = axs[1][1]
    ax.grid(True)
    for k in range(0, len(selectedPatterns)) :
       ax.plot(selectedPatterns[k]['rightAPForce'], label = selectedPatterns[k]['fileName']) 
    ax.set_ylabel("Right-leg anterior-posterior GRF [N]")
    ax.set_xlabel("Sample index")
    # ax.legend(loc='center left', bbox_to_anchor=(1, 0.0))    

    ax = axs[1][2]
    ax.set_visible(False)

    ax = axs[2][0]
    ax.grid(True)
    for k in range(0, len(selectedPatterns)) :
       ax.plot(selectedPatterns[k]['leftMLForce'], label = selectedPatterns[k]['fileName']) 
    ax.set_ylabel("Left-leg medio-lateral GRF [N]")
    ax.set_xlabel("Sample index")
    
    ax = axs[2][1]
    ax.grid(True)
    for k in range(0, len(selectedPatterns)) :
       ax.plot(selectedPatterns[k]['rightMLForce'], label = selectedPatterns[k]['fileName']) 
    ax.set_ylabel("Right-leg medio-lateral GRF [N]")
    ax.set_xlabel("Sample index")    

    ax = axs[2][2]
    ax.set_visible(False)


def showStances(stanceData, fileNamePattern) :
    """
    showStances shows stance-related samples of GRF measurements.
    Samples are provided via a structure of arrays that should be created 
    by using the extractStancesFromMeasurements function.
    
    showStances shows only samples related to CSV files whose names
    match a given pattern (regular expression).
    So, the function can be used to show samples related 
    to a particular subject and shoe type.
     
    The showStances function can be used to compare GRF waveforms 
    in order to assess data integrity and to identify trends and outliers.
     
    Input:
    - stanceData : a data structure created by using 
                       the extractStancesFromMeasurements function
    - fileNamePattern : pattern (regular expression) for file names
             
    Output:
    - Figure is displayed.
             
    Created on May 06, 2022
     
    Author: Marek Parfieniuk
    """
        
    selectedRecords = []
    
    fileNameRegex = re.compile(fileNamePattern)
               
    for measurementIndex in range(0, len(stanceData['fileNames'])) :        
        if (fileNameRegex.fullmatch(stanceData['fileNames'][measurementIndex]) == None) :
            # print("\tSkipped, as it does not match the pattern.")
            continue;    
                           
        t = dict(fileName = stanceData['fileNames'][measurementIndex],
                 firstMLForce = stanceData['firstMLForces'][measurementIndex],
                 firstAPForce = stanceData['firstAPForces'][measurementIndex],
                 firstVForce = stanceData['firstVForces'][measurementIndex], 
                 secondMLForce = stanceData['secondMLForces'][measurementIndex],
                 secondAPForce = stanceData['secondAPForces'][measurementIndex],
                 secondVForce = stanceData['secondVForces'][measurementIndex]) 

        print('Found measurement {:5d} : {}'.format(measurementIndex, t['fileName']))
       
        selectedRecords.append(t)
        
        if (len(selectedRecords) > 150) :
            print('More than 150 measurements found. It seems pointless to show so many plots. Please change the pattern for file names.')
            return
        
        
    if (len(selectedRecords) == 0) :
        print('No measurement found. Please try another file name pattern.')
        return

    [fig, axs] = plt.subplots(3, 3, sharey = False, tight_layout = True, squeeze = False)
    
    resizeAndEntitleWindow(fig, 'Stance-related fragments of GRF measurements of files whose names match "{:s}"'.format(fileNamePattern))

    ax = axs[0][0]
    ax.grid(True)
    for k in range(0, len(selectedRecords)) :
       ax.plot(selectedRecords[k]['firstVForce'], label = selectedRecords[k]['fileName']) 
    ax.set_ylabel("First-plate vertical GRF [N]")
    ax.set_xlabel("Sample indexs")
    #ax.legend(loc='center left', bbox_to_anchor=(1, 0.0))
        
    ax = axs[0][1]
    ax.grid(True)
    for k in range(0, len(selectedRecords)) :
       ax.plot(selectedRecords[k]['secondVForce'], label = selectedRecords[k]['fileName']) 
    ax.set_ylabel("Second-plate vertical GRF [N]")
    ax.set_xlabel("Sample index")        
    ax.legend(loc='upper left', bbox_to_anchor=(1.2, 0.5, 0.9, 0.5), borderaxespad=0.1)    

    ax = axs[0][2]
    ax.set_visible(False)


        
    ax = axs[1][0]
    ax.grid(True)
    for k in range(0, len(selectedRecords)) :
       ax.plot(selectedRecords[k]['firstAPForce'], label = selectedRecords[k]['fileName']) 
    ax.set_ylabel("First-plate anterior-posterior GRF [N]")
    ax.set_xlabel("Sample index")
        
    ax = axs[1][1]
    ax.grid(True)
    for k in range(0, len(selectedRecords)) :
       ax.plot(selectedRecords[k]['secondAPForce'], label = selectedRecords[k]['fileName']) 
    ax.set_ylabel("Second-plate anterior-posterior GRF [N]")
    ax.set_xlabel("Sample index")    
   # ax.legend(loc='center left', bbox_to_anchor=(1.2, 1, 1, 2), borderaxespad=0.2)

    ax = axs[1][2]
    ax.set_visible(False)

    ax = axs[2][0]
    ax.grid(True)
    for k in range(0, len(selectedRecords)) :
       ax.plot(selectedRecords[k]['firstMLForce'], label = selectedRecords[k]['fileName']) 
    ax.set_ylabel("First-plate medio-lateral GRF [N]")
    ax.set_xlabel("Sample index")
    
    ax = axs[2][1]
    ax.grid(True)
    for k in range(0, len(selectedRecords)) :
       ax.plot(selectedRecords[k]['secondMLForce'], label = selectedRecords[k]['fileName']) 
    ax.set_ylabel("Second-plate medio-lateral GRF [N]")
    ax.set_xlabel("Sample index")    
    
    ax = axs[2][2]
    ax.set_visible(False)
    


def computeMeans(forceSamples, stanceIndexes, meanOffset) :
    #print(forceSamples)
    #print(stanceIndexes)
    meanIndexes = np.arange(max(stanceIndexes[0] - meanOffset, 0), stanceIndexes[0] + 1)
    
    #print(meanIndexes)
    strikeMean = np.mean(np.abs(forceSamples[meanIndexes]));
    #print(strikeMean)
    meanIndexes = np.arange(stanceIndexes[-1], min(stanceIndexes[-1] + 1 + meanOffset, stanceIndexes[-1] + 1))
    offMean = np.mean(np.abs(forceSamples[meanIndexes]));
    
    return [strikeMean, offMean]

def extractStancesFromMeasurements(gaitMeasurements, meanOffset, toSmoothBoundaries) :
    """
    extractStancesFromMeasurements processes raw measurements of gait 
    so as to obtain stance-related GRF samples and related information:
    indexes/values of boundary/maximum samples, leg order, file name.
    Its results can be used to recognize patterns, 
    to assess data integrity, and to identify trends and outliers.
    
    Raw measurements are provided via an array of structures 
    that should be created by using the loadGaitData function.
    
    Stance-related samples are determined by using the findStance and cutStance functions. 
    The latter function can smooth tails of stance-related fragment taken of GRF vertical force, 
    so boundary samples can differ from the correspoding samples of raw measurements.
    The "Initial" sub-word is used to distinguish results 
    of the findStance function from those of the cutStance results.
    
    extractStancesFromMeasurements returns a structure with fields
    each of which is an array that contains the corresponding data
    (numbers or vectors) of all raw measurements.
    
    Examples of accessing output data:
    - stanceData['firstVForce'][13] : gives stance-related samples 
      of vertical force of first plate of the 13th measurement
    - stanceData['secondStrikeIndex'][3] : gives the index of the sample
      estimated to start the foot strike on the first plate  
    
    Input:
    - gaitMeasurements : array of dictionaries that describe raw measurements
    - meanOffset : how many samples have to be used to compute means
    - toSmoothBoundaries : 0 or 1, disables/enables refinement 
              of boundaries of the stance-related fragment of the vertical force;
               this argument is passed to the cutStance function

    Output:
    - stanceData : dictionary of column vectors with stance-relatd data 

    Created on Sep. 28, 2021
    
    Author: Marek Parfieniuk
    """

    preallocatedLength = len(gaitMeasurements)
    
    fileNames = np.empty(preallocatedLength, dtype = object)
    leftLegFirstFlags = np.empty(preallocatedLength, dtype = int)    
    legOrderRuleIndexes = np.empty(preallocatedLength, dtype = int)    
    measurementLengths = np.empty(preallocatedLength, dtype = int)
        
    firstVForces = np.empty(preallocatedLength, dtype = object)
    firstAPForces = np.empty(preallocatedLength, dtype = object)
    firstMLForces = np.empty(preallocatedLength, dtype = object)

    secondVForces = np.empty(preallocatedLength, dtype = object)
    secondAPForces = np.empty(preallocatedLength, dtype = object)
    secondMLForces = np.empty(preallocatedLength, dtype = object)
        
    firstStrikeInitialIndexes = np.empty(preallocatedLength, dtype = int)
    firstOffInitialIndexes = np.empty(preallocatedLength, dtype = int)
    firstStrikeInitialValues = np.empty(preallocatedLength, dtype = float)
    firstOffInitialValues = np.empty(preallocatedLength, dtype = float)
    firstMaxIndexes = np.empty(preallocatedLength, dtype = int)
    firstMaxValues = np.empty(preallocatedLength, dtype = float)
    firstStrikeIndexes = np.empty(preallocatedLength, dtype = int)
    firstOffIndexes = np.empty(preallocatedLength, dtype = int)
    firstVStrikeMeans = np.empty(preallocatedLength, dtype = float)
    firstAPStrikeMeans = np.empty(preallocatedLength, dtype = float)
    firstMLStrikeMeans = np.empty(preallocatedLength, dtype = float)
    firstVOffMeans = np.empty(preallocatedLength, dtype = float)
    firstAPOffMeans = np.empty(preallocatedLength, dtype = float)
    firstMLOffMeans = np.empty(preallocatedLength, dtype = float)


    secondStrikeInitialIndexes = np.empty(preallocatedLength, dtype = int)
    secondOffInitialIndexes = np.empty(preallocatedLength, dtype = int)
    secondStrikeInitialValues = np.empty(preallocatedLength, dtype = float)
    secondOffInitialValues = np.empty(preallocatedLength, dtype = float)
    secondMaxIndexes = np.empty(preallocatedLength, dtype = int)
    secondMaxValues = np.empty(preallocatedLength, dtype = float)
    secondStrikeIndexes = np.empty(preallocatedLength, dtype = int)
    secondOffIndexes = np.empty(preallocatedLength, dtype = int)        
    secondVStrikeMeans = np.empty(preallocatedLength, dtype = float)
    secondAPStrikeMeans = np.empty(preallocatedLength, dtype = float)
    secondMLStrikeMeans = np.empty(preallocatedLength, dtype = float)
    secondVOffMeans = np.empty(preallocatedLength, dtype = float)
    secondAPOffMeans = np.empty(preallocatedLength, dtype = float)
    secondMLOffMeans = np.empty(preallocatedLength, dtype = float)

        
    for measurementIndex in range(0, len(gaitMeasurements)) :
        measurementRecord = gaitMeasurements[measurementIndex];
        
        if ((measurementIndex % 100) == 0) :
            print('Processing measurement no. {:d}'.format(measurementIndex))

        fileNames[measurementIndex] = measurementRecord['fileName']
        
        measurementLengths[measurementIndex] = len(measurementRecord['secondVForce'])

        [firstStrikeInitialIndexes[measurementIndex], firstOffInitialIndexes[measurementIndex], 
                firstStrikeInitialValues[measurementIndex], firstOffInitialValues[measurementIndex], 
                firstMaxIndexes[measurementIndex], firstMaxValues[measurementIndex]] \
                = findStance(measurementRecord['firstVForce'], 0)
    
        [firstVForces[measurementIndex], firstStrikeIndexes[measurementIndex], 
                firstOffIndexes[measurementIndex]] \
                = cutStance(measurementRecord['firstVForce'], firstStrikeInitialIndexes[measurementIndex],
                firstOffInitialIndexes[measurementIndex], toSmoothBoundaries)
        
        [secondStrikeInitialIndexes[measurementIndex], secondOffInitialIndexes[measurementIndex], 
                 secondStrikeInitialValues[measurementIndex], secondOffInitialValues[measurementIndex], 
                 secondMaxIndexes[measurementIndex], secondMaxValues[measurementIndex]] \
                 = findStance(measurementRecord['secondVForce'], 0)
    
        [secondVForces[measurementIndex], secondStrikeIndexes[measurementIndex],
                secondOffIndexes[measurementIndex]] \
                 = cutStance(measurementRecord['secondVForce'], secondStrikeInitialIndexes[measurementIndex], 
                secondOffInitialIndexes[measurementIndex], toSmoothBoundaries)

        stanceIndexes = np.arange(firstStrikeIndexes[measurementIndex], firstOffIndexes[measurementIndex] + 1)
        #print(stanceIndexes)
        
        firstAPForces[measurementIndex] = measurementRecord['firstAPForce'][stanceIndexes]
        firstMLForces[measurementIndex] = measurementRecord['firstMLForce'][stanceIndexes]

        [firstVStrikeMeans[measurementIndex], firstVOffMeans[measurementIndex]] \
                = computeMeans(measurementRecord['firstVForce'], stanceIndexes, meanOffset)
        [firstAPStrikeMeans[measurementIndex], firstAPOffMeans[measurementIndex]] \
                = computeMeans(measurementRecord['firstAPForce'], stanceIndexes, meanOffset)
        [firstMLStrikeMeans[measurementIndex], firstMLOffMeans[measurementIndex]] \
                = computeMeans(measurementRecord['firstMLForce'], stanceIndexes, meanOffset)

        stanceIndexes = np.arange(secondStrikeIndexes[measurementIndex], secondOffIndexes[measurementIndex] + 1)
        secondAPForces[measurementIndex] = measurementRecord['secondAPForce'][stanceIndexes]
        secondMLForces[measurementIndex] = measurementRecord['secondMLForce'][stanceIndexes]

        [secondVStrikeMeans[measurementIndex], secondVOffMeans[measurementIndex]] \
                = computeMeans(measurementRecord['secondVForce'], stanceIndexes, meanOffset)
        [secondAPStrikeMeans[measurementIndex], secondAPOffMeans[measurementIndex]] \
                = computeMeans(measurementRecord['secondAPForce'], stanceIndexes, meanOffset)
        [secondMLStrikeMeans[measurementIndex], secondMLOffMeans[measurementIndex]] \
                = computeMeans(measurementRecord['secondMLForce'], stanceIndexes, meanOffset)

              
        [leftLegFirstFlags[measurementIndex], legOrderRuleIndexes[measurementIndex]] \
                = checkLeftLegFirst(firstMLForces[measurementIndex], secondMLForces[measurementIndex])

        stanceData = dict(
                fileNames = fileNames,
                measurementLengths = measurementLengths,
                leftLegFirstFlags = leftLegFirstFlags,
                legOrderRuleIndexes = legOrderRuleIndexes,               

                firstVForces = firstVForces,
                firstAPForces = firstAPForces,
                firstMLForces = firstMLForces,
                firstStrikeIndexes = firstStrikeIndexes,
                firstStrikeInitialIndexes = firstStrikeInitialIndexes,
                firstStrikeInitialValues = firstStrikeInitialValues,
                firstOffIndexes = firstOffIndexes,
                firstOffInitialIndexes = firstOffInitialIndexes,
                firstOffInitialValues = firstOffInitialValues,
                firstMaxIndexes = firstMaxIndexes,
                firstMaxValues = firstMaxValues,


                firstVStrikeMeans = firstVStrikeMeans,
                firstAPStrikeMeans = firstAPStrikeMeans,
                firstMLStrikeMeans = firstMLStrikeMeans,
                firstVOffMeans = firstVOffMeans,
                firstAPOffMeans = firstAPOffMeans,
                firstMLOffMeans = firstMLOffMeans,
                
                secondVForces = secondVForces,
                secondAPForces = secondAPForces,
                secondMLForces = secondMLForces,
                secondStrikeIndexes = secondStrikeIndexes,
                secondStrikeInitialValues = secondStrikeInitialValues,               
                secondStrikeInitialIndexes = secondStrikeInitialIndexes,
                secondOffIndexes = secondOffIndexes,
                secondOffInitialIndexes = secondOffInitialIndexes,
                secondOffInitialValues = secondOffInitialValues,
                secondMaxIndexes = secondMaxIndexes,
                secondMaxValues = secondMaxValues,

                secondVStrikeMeans = secondVStrikeMeans,
                secondAPStrikeMeans = secondAPStrikeMeans,
                secondMLStrikeMeans = secondMLStrikeMeans,
                secondVOffMeans = secondVOffMeans,
                secondAPOffMeans = secondAPOffMeans,
                secondMLOffMeans = secondMLOffMeans
                )
    
    return stanceData



def summarizeStances(stanceData) :
    '''
    summarizeStances shows histograms, maxima, minima, 
    that characterize distributions of values and indexes of boundary 
    (maximum, foot-strike-related, foot-off-related) samples
    of measurement parts related to the stance phase of gait.
    
    Additionally, average relations between measurement length 
    and durations of step and stance phase are quantified
    by mean values and standard deviation.
    
    summarizeStances also shows histograms of mean magnitudes 
    of samples that precede/follow the foot-strike/off events.
    These plots show in which GRF signals and how often Means occur
    related to faults in the initial construction of the walkway.
    
    This function can be used to validate patterns, measurements, 
    and algorithms used to detect boundaries of the stance phase.
    One can look for outstanding boundary samples, 
    and then use them to identify and check measurements that might be outliers.
    
    Input:
    - stanceData : stance-related data created by using 
                      the extractStanceData function.
    
    Output:
    - Figures and texts are displayed.
    
    Created on May 25, 2022
    
    Author: Marek Parfieniuk
    '''


    fileNames = stanceData['fileNames']
    firstVForces = stanceData['firstVForces']
    secondVForces = stanceData['secondVForces']
    firstAPForces = stanceData['firstAPForces']
    secondAPForces = stanceData['secondAPForces']

    measurementCount = len(stanceData['firstVForces']);

    firstVMaximums = np.empty(measurementCount, dtype = float)
    firstAPMaximums = np.empty(measurementCount, dtype = float)
    firstAPMinimums = np.empty(measurementCount, dtype = float)
    secondVMaximums = np.empty(measurementCount, dtype = float)
    secondAPMaximums = np.empty(measurementCount, dtype = float)
    secondAPMinimums = np.empty(measurementCount, dtype = float)

    for k in range(0, measurementCount) :
        firstVMaximums[k] = np.max(firstVForces[k])
        firstAPMaximums[k] = np.max(firstAPForces[k])
        firstAPMinimums[k] = np.min(firstAPForces[k])
        secondVMaximums[k] = np.max(secondVForces[k])
        secondAPMaximums[k] = np.max(secondAPForces[k])
        secondAPMinimums[k] = np.min(secondAPForces[k])
            
    
    measurementLengths = stanceData['measurementLengths']

    firstStrikeIndexes = stanceData['firstStrikeIndexes']
    firstOffIndexes = stanceData['firstOffIndexes']
    firstStrikeInitialValues = stanceData['firstStrikeInitialValues']
    firstOffInitialValues = stanceData['firstOffInitialValues']
    
    secondStrikeIndexes = stanceData['secondStrikeIndexes']
    secondOffIndexes = stanceData['secondOffIndexes']
    secondStrikeInitialValues = stanceData['secondStrikeInitialValues']
    secondOffInitialValues = stanceData['secondOffInitialValues']

    t = measurementLengths / (firstOffIndexes - firstStrikeIndexes)
    print('Ratio of measurement duration to stance phase duration (first plate)')
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))
    t = measurementLengths / (secondOffIndexes - secondStrikeIndexes)
    print('Ratio of measurement duration to stance phase duration (second plate)')
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))
    
    t = measurementLengths / (secondStrikeIndexes - firstStrikeIndexes);
    print('Ratio of measurement duration to foot-strike-based step duration (both plates)')
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))

    t = measurementLengths / (secondOffIndexes - firstOffIndexes);
    print('Ratio of measurement duration to foot-strike-based step duration (both plates)')
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))


    t = measurementLengths / 960;    # sample rate was 960 Hz 
    print('Measurement duration [seconds]');
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))

    t = (secondStrikeIndexes - firstStrikeIndexes) / 960;
    print('Foot-strike-based step duration [seconds]');
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))

    t = (secondOffIndexes - firstOffIndexes) / 960;
    print('Foot-off-based step duration [seconds]');
    print(' - mean: {:5.2f}'.format(np.mean(t)))
    print(' - standard deviation: {:5.2f}'.format(np.std(t, ddof = 1)))


    numberOfBins = 30

    fig, axs = plt.subplots(2, 2, sharey = False, tight_layout = True)
    
    resizeAndEntitleWindow(fig, 'Step duration histograms')

    ax = axs[0][0]
    t = secondStrikeIndexes - firstStrikeIndexes
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Foot-strike-based step duration [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(350, 750)
    ax.set_ylim(0, 1700)    

    ax = axs[0][1]
    t = secondOffIndexes - firstOffIndexes
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Foot-off-based step duration [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(350, 750)
    ax.set_ylim(0, 1700)    


    ax = axs[1][0]
    t = (secondStrikeIndexes - firstStrikeIndexes) / 960
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Foot-strike-based step duration [seconds]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0.4, 0.8)
    ax.set_ylim(0, 1700)    

    ax = axs[1][1]
    t = (secondOffIndexes - firstOffIndexes) / 960
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Foot-off-based step duration [seconds]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0.4, 0.8)
    ax.set_ylim(0, 1700)    
 
    
    
    fig, axs = plt.subplots(2, 2, sharey = False, tight_layout = True)
    
    resizeAndEntitleWindow(fig, 'Stance-phase-related general histograms')

    ax = axs[0][0]
    t = measurementLengths
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Measurement duration [samples]\n(The same for both plates)'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(1000, 7000)
    ax.set_ylim(0, 3000)    

    ax = axs[0][1]
    t = (firstOffIndexes - firstStrikeIndexes) - (secondOffIndexes - secondStrikeIndexes)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Difference between durations of stance phases of both plates [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-150, 200)
    ax.set_ylim(0, 3500)    

    ax = axs[1][0]
    t = measurementLengths / (firstOffIndexes - firstStrikeIndexes)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Ratio of measurement duration to stance-phase duration: first plate'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(1, 10)
    ax.set_ylim(0, 2500)    
    

    ax = axs[1][1]
    t = measurementLengths / (secondOffIndexes - secondStrikeIndexes);
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Ratio of measurement duration to stance-phase duration: second plate'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(1, 10)
    ax.set_ylim(0, 2500) 

        
    fig, axs = plt.subplots(2, 2, sharey = False, tight_layout = True)
    
    resizeAndEntitleWindow(fig, 'Stance-phase duration histograms')
    
    ax = axs[0][0]
    t = firstOffIndexes - firstStrikeIndexes
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate stance-phase duration [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(400, 1100)
    ax.set_ylim(0, 1800) 

    ax = axs[0][1]
    t = secondOffIndexes - secondStrikeIndexes
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate stance-phase duration [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(400, 1100)
    ax.set_ylim(0, 1800) 
    
    ax = axs[1][0]
    t = (firstOffIndexes - firstStrikeIndexes) / 960
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate stance-phase duration [seconds]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    #ax.set_xlim(300, 800)
    #ax.set_ylim(0, 1800) 
    
    ax = axs[1][1]
    t = (secondOffIndexes - secondStrikeIndexes) / 960
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate stance-phase duration [seconds]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    #ax.set_xlim(300, 800)
    #ax.set_ylim(0, 1800) 
    
    
    fig, axs = plt.subplots(2, 2, sharey = False, tight_layout = True)

    resizeAndEntitleWindow(fig, 'Histograms of vertical force at stance-phase boundaries')
    
    ax = axs[0][0]
    t = firstStrikeInitialValues
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate vertical force at foot-strike [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-40, 100)
    ax.set_ylim(0, 4000) 
    
    ax = axs[0][1]
    t = secondStrikeInitialValues
    ax.hist(t, bins = math.ceil(numberOfBins / 3))
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate vertical force at foot-strike [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-40, 100)
    ax.set_ylim(0, 4000) 
    
    ax = axs[1][0]
    t = firstOffInitialValues
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate vertical force at foot-off [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-20, 40)
    ax.set_ylim(0, 5000) 
    
    ax = axs[1][1]
    t = secondOffInitialValues
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate vertical force at foot-off [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-20, 40)
    ax.set_ylim(0, 5000) 
    


    fig, axs = plt.subplots(3, 2, sharey = False, tight_layout = True)
    resizeAndEntitleWindow(fig, 'Histograms of extremums of vertical and anterior-posterior forces')


    ax = axs[0][0]
    t = firstVMaximums
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate vertical force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    # ax.set_xlim(0, 1700)
    # ax.set_ylim(0, 1500)    

    ax = axs[0][1]
    t = secondVMaximums
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate vertical force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    # ax.set_xlim(0, 1700)
    # ax.set_ylim(0, 1500)    

    ax = axs[1][0]
    t = firstAPMaximums
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate anterior-posterior force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    # ax.set_xlim(0, 400)
    # ax.set_ylim(0, 1800)    

    ax = axs[1][1]
    t = secondAPMaximums
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate anterior-posterior force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    # ax.set_xlim(0, 400)
    # ax.set_ylim(0, 1800)    

    ax = axs[2][0]
    t = firstAPMinimums
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('First-plate anterior-posterior force: minimum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    # ax.set_xlim(-350, 0)
    # ax.set_ylim(0, 1800)    

    ax = axs[2][1]
    t = secondAPMinimums
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Second-plate anterior-posterior force: minimum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    # ax.set_xlim(-350, 0)
    # ax.set_ylim(0, 1800)




    fig, axs = plt.subplots(3, 2, sharey = False, tight_layout = True)

    resizeAndEntitleWindow(fig, 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events on first plate')
    
    ax = axs[0][0]
    t = stanceData['firstVStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples\nof vertical force of first plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000)  
    
    ax = axs[0][1]
    t = stanceData['firstVOffMeans']
    ax.hist(t, bins = math.ceil(numberOfBins / 3))
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples\nof vertical force of first plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[1][0]
    t = stanceData['firstAPStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples\nof anterior-posterior force of first plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[1][1]
    t = stanceData['firstAPOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples\nof anterior-posterior force of first plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000)  
    
    ax = axs[2][0]
    t = stanceData['firstMLStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples\nof medio-lateral force of first plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    #ax.set_xlim(400, 1700)
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000)  
    
    ax = axs[2][1]
    t = stanceData['firstMLOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples\nof medio-lateral force of first plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000)  


    fig, axs = plt.subplots(3, 2, sharey = False, tight_layout = True)

    resizeAndEntitleWindow(fig, 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events on second plate')
    
    ax = axs[0][0]
    t = stanceData['secondVStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples\nof vertical force of second plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[0][1]
    t = stanceData['secondVOffMeans']
    ax.hist(t, bins = math.ceil(numberOfBins / 3))
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples\nof vertical force of second plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[1][0]
    t = stanceData['secondAPStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples\nof anterior-posterior force of second plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[1][1]
    t = stanceData['secondAPOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples\nof anterior-posterior force of second plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[2][0]
    t = stanceData['secondMLStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples\nof medio-lateral force of second plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    #ax.set_xlim(400, 1700)
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 
    
    ax = axs[2][1]
    t = stanceData['secondMLOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples\nof medio-lateral force of second plate [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 40)
    ax.set_ylim(0, 6000) 

     
    plt.plot()
   


def checkLeftLegFirst(firstPlateMLStanceSamples, secondPlateMLStanceSamples, toBeVerbose = 0) :

    '''
    checkLeftLegFirst checks if the first of two given sequences of samples 
    of the medio-lateral force is related to the left leg-foot.  
    
    Given sequences should be related to two force plates, 
    one of which contacted the left foot, while another contacted the right foot.
    
    Given sequences must comprise only stance-related samples, 
    but need not to be time-normalized, or to comprise as many samples.
    
    The algorithm is based on trying decision rules, from more to less strict ones. 
    Rules are based on comparing the means of the beginning, mid, and ending
    framents of time-normalized versions of the input sequences.
    
    This function allows for automatically associating participant's legs 
    with force plates and measurements.
    During experiments participants were allowed to put either foot 
    on the first of two force plates, which were arranged 
    one after another along the walkway.
    The leg order was not observed, so our data set of gait measurements 
    contains no explicite information about leg-plate relations.
    
    Input:
    - firstPlateMLForceStanceSamples : a vector of stance-related samples 
          of the medio-lateral ground reaction force of one force plate.
    - secondPlateMLForceStanceSamples : a signal (vector of samples) 
          of the medio-lateral ground reaction force of another force plate
    - toBeVerbose : 0 or 1. Disables/enables displaying diagnostic messages, 
                    which were used to debug the code.
    
    Output:
    - result : 1 if firstPlateMLForceStanceSamples are related to the left foot
               0 otherwise
    - ruleIndex : index of the decision rule behind the result
    
    Created on May 04, 2022
    
    Author: Marek Parfieniuk        
    '''
    
    
    compareLength = 100; 

    sampleCount = len(firstPlateMLStanceSamples); 
    firstPlateSamples = np.interp(np.arange(0, compareLength) / (compareLength - 1), 
                            np.arange(0, sampleCount) / (sampleCount - 1), firstPlateMLStanceSamples);
    
    sampleCount = len(secondPlateMLStanceSamples); 
    secondPlateSamples = np.interp(np.arange(0, compareLength) / (compareLength - 1), 
                            np.arange(0, sampleCount) / (sampleCount - 1), secondPlateMLStanceSamples);
    
    ruleIndex = 0;
    
    # Indexes of three parts of a time series.
    # Unequal parts are based on dividing a time series into 7 fragments.
    # Two no-care fragments are omitted.
    
    d = np.rint(compareLength / 7)
    begIdx = np.arange(0, d, dtype=int)
    midIdx = np.arange(2 * d - 1, 5 * d, dtype=int)
    endIdx = np.arange(6 * d - 1, compareLength, dtype=int)
    
    midMean1 = np.mean(firstPlateSamples[midIdx])
    midStd1 = np.std(firstPlateSamples[midIdx])
    midMean2 = np.mean(secondPlateSamples[midIdx])
    midStd2 = np.std(secondPlateSamples[midIdx])
    
    frc1 = 0.45;    # Tuned based on measurement no. 93355. Could be lower for most measurements.
    
    # A rule is applied if no previous rule gives a decision.

    if (ruleIndex == 0) :
        begGreaterCount1 = np.sum(firstPlateSamples[begIdx] > (midMean1 + midStd1))
        endGreaterCount1 = np.sum(firstPlateSamples[endIdx] > (midMean1 + midStd1))
        begLowerCount2 = np.sum(secondPlateSamples[begIdx] < (midMean2 - midStd2))
        endLowerCount2 = np.sum(secondPlateSamples[endIdx] < (midMean2 - midStd2))
        
        if ((begGreaterCount1 > (frc1 * d)) and (endGreaterCount1 > (frc1 * d)) 
                and (begLowerCount2 > (frc1 * d)) and (endLowerCount2 > (frc1 * d))) :
            ruleIndex = -1      # right on first
    
    if (ruleIndex == 0) :
        begLowerCount1 = np.sum(firstPlateSamples[begIdx] < (midMean1 - midStd1))
        endLowerCount1 = np.sum(firstPlateSamples[endIdx] < (midMean1 - midStd1))
        begGreaterCount2 = np.sum(secondPlateSamples[begIdx] > (midMean2 + midStd2))
        endGreaterCount2 = np.sum(secondPlateSamples[endIdx] > (midMean2 + midStd2))
        
        if ((begGreaterCount2 > (frc1 * d)) and (endGreaterCount2 > (frc1 * d)) 
                and (begLowerCount1 > (frc1 * d)) and (endLowerCount1 > (frc1 * d))) :
            ruleIndex = 1       # left on first
    
    if (ruleIndex == 0) :   
        begGreaterCount1 = sum(firstPlateSamples[begIdx] > (midMean1 + midStd1)) 
        begLowerCount2 = sum(secondPlateSamples[begIdx] < (midMean2 - midStd2))
        
        if ((begGreaterCount1 > (frc1 * d)) and (begLowerCount2 > (frc1 * d))) :
            ruleIndex = -3      # right on first
    if (ruleIndex == 0) :    
        begLowerCount1 = sum(firstPlateSamples[begIdx] < (midMean1 - midStd1))
        begGreaterCount2 = sum(secondPlateSamples[begIdx] > (midMean2 + midStd2))
       
        if ((begLowerCount1 > (frc1 * d)) and (begGreaterCount2 > (frc1 * d))) :
            ruleIndex = 3       # left on first
    if (ruleIndex == 0) :
        begGreaterCount1 = sum(firstPlateSamples[begIdx] > midMean1)
        endGreaterCount1 = sum(firstPlateSamples[endIdx] > midMean1)
        begLowerCount2 = sum(secondPlateSamples[begIdx] < midMean2)
        endLowerCount2 = sum(secondPlateSamples[endIdx] < midMean2)
        
        if ((begGreaterCount1 > (frc1 * d)) and (endGreaterCount1 > (frc1 * d))  
                and (begLowerCount2 > (frc1 * d)) and (endLowerCount2 > (frc1 * d))) :
            ruleIndex = -5      # right on first
    
    if (ruleIndex == 0) :
        begLowerCount1 = sum(firstPlateSamples[begIdx] < midMean1)
        endLowerCount1 = sum(firstPlateSamples[endIdx] < midMean1)
        begGreaterCount2 = sum(secondPlateSamples[begIdx] > midMean2)
        endGreaterCount2 = sum(secondPlateSamples[endIdx] > midMean2)
        
        if ((begGreaterCount2 > (frc1 * d)) and (endGreaterCount2 > (frc1 * d)) 
                and (begLowerCount1 > (frc1 * d)) and (endLowerCount1 > (frc1 * d))) :
            ruleIndex = 5       # left on first
    
    if (ruleIndex == 0) :   # This rule is always decisive.
        if (midMean1 < midMean2) :
            ruleIndex = -4      # right on first
        else :
            ruleIndex = 4   
    
    result = (ruleIndex >= 0)
    
    return [result, ruleIndex]




def resizeAndEntitleWindow(fig, title) :
    """
    resizeAndEntitleWindow sets the title of window of given figure.
    It also enlarges window so that is should occupy more area 
    of high-resolution screen.

    This function highly depends on GUI backend behind matplotlib,
    and on GUI configuration of a Python runtime.
    In case of errors, it only shows warnings.
    """
    try :
        # fig.canvas.set_window_title(title)
        if fig.canvas.manager is not None:
            fig.canvas.manager.set_window_title(title)
        else :
            fig.canvas.set_window_title(title)
        manager = plt.get_current_fig_manager()        
        manager.window.setGeometry(50, 50, 1500, 1000)
        
    except :
        traceback.print_exc()
        print('Something is wrong with GUI backends behind matplotlib')



def loadPatterns(fileDirectoryPath, fileNamePattern, patternSize) :
    
    """
    loadPatterns reads PTRN.CSV files of given directory, 
    only those whose names match a given pattern (regular expression).
    Each file is assumed to describe one gait pattern i.e. 
    samples of the time-normalized stance-related fragment 
    of a gait-GRF measurement described 
    by the correspoding CSV file of the essential data set. 
    
    Data are loaded from files into an array of structures.
    Each structure describes one gait-pattern
    and comprises fields
    each of which is a vector or matrix that contains the corresponding data
    of all files.
    In "...Force" fields, one row comprises samples 
    of the corresponding signal of one file. 
    
    The file name (including extension) can be used 
    as a pattern identifier-label.
    
    This function assumes that the given directory contains person/subject-related
    subdirectiories '001'...'324' that contain shoe-related subdirectiories 
    that contain only pattern-related PTRN.CSV files.
    Errors will result from presence of other subdirectories and files
    in subject-related subdirectories.
    
    This function temporarily changes the working directory. 
    In case of error it does not return to the starting directory.
    
    Input:
    - fileDirectoryPath : path to directory of gait-pattern files
    - fileNamePattern : pattern (regular expression) of file names
    - patternSize : expected number of samples in each signal fragment 
    
    Output:
    - patternData : cell array of structures that describe gait-GRF patterns
    
    Examples of file-name patterns:
    - r'.*' : All measurements
    - r'.*\.f\..*' : All female-related measurements
    - r'.*\.m\..*' : All male-related measurements
    - r'024.*' : All measurements related to the 24th participant, regardless of shoes
    - r'024.*\.spr.*' : All measurements related to the 24th participant
                           in sport shoes        
    
    Usage example:
    - patternData = loadPatterns(r'c:\\gaitDataInCSVFiles', r'.*', 0)
    
    Created on May 08, 2022
                
    @author: Marek Parfieniuk
    """    


    currentDirectory= os.getcwd()

    patternCount = 100000

    fileNames = np.empty(patternCount, dtype = object)
    leftVForces = np.empty([patternCount, patternSize], dtype = float)
    leftAPForces = np.empty([patternCount, patternSize], dtype = float)
    leftMLForces = np.empty([patternCount, patternSize], dtype = float)

    rightVForces = np.empty([patternCount, patternSize], dtype = float)
    rightAPForces = np.empty([patternCount, patternSize], dtype = float)
    rightMLForces = np.empty([patternCount, patternSize], dtype = float)
    
    leftVStrikeMeans = np.empty(patternCount, dtype = float)
    leftMLStrikeMeans = np.empty(patternCount, dtype = float)
    leftAPStrikeMeans = np.empty(patternCount, dtype = float)
    
    rightVStrikeMeans = np.empty(patternCount, dtype = float)
    rightMLStrikeMeans = np.empty(patternCount, dtype = float)
    rightAPStrikeMeans = np.empty(patternCount, dtype = float)
     
    leftVOffMeans = np.empty(patternCount, dtype = float)
    leftMLOffMeans = np.empty(patternCount, dtype = float)
    leftAPOffMeans = np.empty(patternCount, dtype = float)
    
    rightVOffMeans = np.empty(patternCount, dtype = float)
    rightMLOffMeans = np.empty(patternCount, dtype = float) 
    rightAPOffMeans = np.empty(patternCount, dtype = float)
     
    leftStrikeIndexes = np.empty(patternCount, dtype = int)
    leftOffIndexes = np.empty(patternCount, dtype = int)
    rightStrikeIndexes = np.empty(patternCount, dtype = int) 
    rightOffIndexes = np.empty(patternCount, dtype = int)

        
    fileNameRegex = re.compile(fileNamePattern)
    
    os.chdir(fileDirectoryPath)
    
    
    fileIndex = -1
    
    # WARNING! We iterate over a fixed range of participant identifiers
    for participantIndex in range(1, 325) : #325) :

        participantDirectory = str("{:03}").format(participantIndex)
        
        os.chdir(participantDirectory)    
        
        shoeDirectory = os.listdir()
        
        print("{} : participant is processed.".format(participantDirectory))
        
        for shoeDirectory in os.listdir():
            
            print("   {} : shoe is processed.".format(shoeDirectory))
            
            os.chdir(shoeDirectory)
            for fileName in os.listdir() :     
                if not fileName.endswith('ptrn.csv') :
                    continue                                
                
                if (fileNameRegex.fullmatch(fileName) == None) :
                    # print("\tSkipped, as it does not match the pattern.")
                    continue

    # An empty matrix is preallocated to store signal samples.
    # It has 10000 rows, more than necessary, so to avoid memory reallocation.
    # After loading data from file, we trim the matrix, 
    # so as to remove empty rows.

                fileIndex = fileIndex + 1                

                print('   {:s} is processed'.format(fileName))
                
                # Preallocate materix for data
                data = np.empty([patternSize, 14], dtype=np.double)
    
                f = open(fileName, "r")
                lineIndex = -1;
                for line in f : # We iterate over lines.
                    if (lineIndex == -1) :
                        lineIndex = 0
                        continue    # We skip the header line
                    
                    t = line.split(',') 
                    data[lineIndex, 0] = float(t[0])
                    data[lineIndex, 1] = float(t[1])
                    data[lineIndex, 2] = float(t[2])
                    data[lineIndex, 3] = float(t[3])
                    data[lineIndex, 4] = float(t[4])                    
                    data[lineIndex, 5] = float(t[5])
                    if (len(t) > 6) :
                        data[lineIndex, 6] = float(t[6])
                        data[lineIndex, 7] = float(t[7])
                        data[lineIndex, 8] = float(t[8])
                        data[lineIndex, 9] = float(t[9])
                        data[lineIndex, 10] = float(t[10])                    
                        data[lineIndex, 11] = float(t[11])
                        data[lineIndex, 12] = int(t[12])                    
                        data[lineIndex, 13] = int(t[13])
                    
                    lineIndex = lineIndex + 1
                data = data[:lineIndex, :]
                f.close()
                
                fileNames[fileIndex] = fileName
                leftVForces[fileIndex, :] = data[:, 0]
                leftAPForces[fileIndex, :] = data[:, 1]
                leftMLForces[fileIndex, :] = data[:, 2]

                rightVForces[fileIndex, :] = data[:, 3]
                rightAPForces[fileIndex, :] = data[:, 4]
                rightMLForces[fileIndex, :] = data[:, 5]
                
                leftVStrikeMeans[fileIndex] = data[0, 6]
                leftAPStrikeMeans[fileIndex] = data[0, 7]
                leftMLStrikeMeans[fileIndex] = data[0, 8]
                
                rightVStrikeMeans[fileIndex] = data[0, 9]
                rightAPStrikeMeans[fileIndex] = data[0, 10]
                rightMLStrikeMeans[fileIndex] = data[0, 11]
                 
                leftVOffMeans[fileIndex] = data[lineIndex - 1, 6]
                leftAPOffMeans[fileIndex] = data[lineIndex - 1, 7]
                leftMLOffMeans[fileIndex] = data[lineIndex - 1, 8]
                
                rightVOffMeans[fileIndex] = data[lineIndex - 1, 9]
                rightAPOffMeans[fileIndex] = data[lineIndex - 1, 10]
                rightMLOffMeans[fileIndex] = data[lineIndex - 1, 11]
                 
                leftStrikeIndexes[fileIndex] = data[0, 12]
                leftOffIndexes[fileIndex] = data[lineIndex - 1, 12]
                rightStrikeIndexes[fileIndex] = data[0, 13]
                rightOffIndexes[fileIndex] = data[lineIndex - 1, 13]


            patternData = dict(
                fileNames = fileNames[: fileIndex],
                leftVForces = leftVForces[: fileIndex, :],
                leftAPForces = leftAPForces[: fileIndex, :],
                leftMLForces = leftMLForces[: fileIndex, :],
                
                rightVForces = rightVForces[: fileIndex, :],
                rightAPForces = rightAPForces[: fileIndex, :],
                rightMLForces = rightMLForces[: fileIndex, :],
                    
                leftVStrikeMeans = leftVStrikeMeans[: fileIndex],
                leftMLStrikeMeans = leftMLStrikeMeans[: fileIndex],
                leftAPStrikeMeans = leftAPStrikeMeans[: fileIndex],
                    
                rightVStrikeMeans = rightVStrikeMeans[: fileIndex],
                rightMLStrikeMeans = rightMLStrikeMeans[: fileIndex],
                rightAPStrikeMeans = rightAPStrikeMeans[: fileIndex],
                     
                leftVOffMeans = leftVOffMeans[: fileIndex],
                leftMLOffMeans = leftMLOffMeans[: fileIndex],
                leftAPOffMeans = leftAPOffMeans[: fileIndex],
                    
                rightVOffMeans = rightVOffMeans[: fileIndex],
                rightMLOffMeans = rightMLOffMeans[: fileIndex],
                rightAPOffMeans = rightAPOffMeans[: fileIndex],
                     
                leftStrikeIndexes = leftStrikeIndexes[: fileIndex],
                leftOffIndexes = leftOffIndexes[: fileIndex],
                rightStrikeIndexes = rightStrikeIndexes[: fileIndex],
                rightOffIndexes = rightOffIndexes[: fileIndex]
            )
                        
            os.chdir("..")
        os.chdir("..")
    os.chdir(currentDirectory)    
    
    print('Done! {} PTRN.CSV files have been processed.'.format(fileIndex + 1))

    return patternData


def convertStancesIntoPatterns(stanceData, patternSize) :
    """
    convertStancesIntoPatterns converts data of stance-related fragments 
    of GRF signals into patterns suitable for recognizing persons and shoe
    types.
    
    The input structure should be prepared by using the extractStancesFromMeasurements
    function.
    
    The main effects of this function are
    - it returns data assigned to legs, taking as input data assigned to force plates.
    - it returns sample series of the same, given length, regardless of how many
      samples were in input signals; it time-normalizes GRF signals 
      by interpolating samples,
    
    This function returns a structure of fiels
    each of which is a vector that contains the corresponding data
    (numbers or vectors) of all input records.
    
    Input:
    - stanceData : array of records of gait measurements
    - patternSize : number of samples that result from signal
      time-normalization.
    
    Output:
    - patternData : structure of column vectors with pattern data 
    
    Created on Sep. 28, 2021
    
    Author: Marek Parfieniuk
    """

    measurementCount = len(stanceData['fileNames'])
    
    fileNames = np.empty(measurementCount, dtype = object)
    leftVForces = np.empty([measurementCount, patternSize], dtype = float)
    leftAPForces = np.empty([measurementCount, patternSize], dtype = float)
    leftMLForces = np.empty([measurementCount, patternSize], dtype = float)

    rightVForces = np.empty([measurementCount, patternSize], dtype = float)
    rightAPForces = np.empty([measurementCount, patternSize], dtype = float)
    rightMLForces = np.empty([measurementCount, patternSize], dtype = float)
    
    leftVStrikeMeans = np.empty(measurementCount, dtype = float)
    leftMLStrikeMeans = np.empty(measurementCount, dtype = float)
    leftAPStrikeMeans = np.empty(measurementCount, dtype = float)
    
    rightVStrikeMeans = np.empty(measurementCount, dtype = float)
    rightMLStrikeMeans = np.empty(measurementCount, dtype = float)
    rightAPStrikeMeans = np.empty(measurementCount, dtype = float)
     
    leftVOffMeans = np.empty(measurementCount, dtype = float)
    leftMLOffMeans = np.empty(measurementCount, dtype = float)
    leftAPOffMeans = np.empty(measurementCount, dtype = float)
    
    rightVOffMeans = np.empty(measurementCount, dtype = float)
    rightMLOffMeans = np.empty(measurementCount, dtype = float) 
    rightAPOffMeans = np.empty(measurementCount, dtype = float)
     
    leftStrikeIndexes = np.empty(measurementCount, dtype = int)
    leftOffIndexes = np.empty(measurementCount, dtype = int)
    rightStrikeIndexes = np.empty(measurementCount, dtype = int) 
    rightOffIndexes = np.empty(measurementCount, dtype = int)
    
    normalizedLength = patternSize
    normalizedRelativeTimes = np.arange(0, normalizedLength) / (normalizedLength - 1)
    
    for measurementIndex in range(0, measurementCount) :
        if ((measurementIndex % 10) == 0) :
            print('Processing measurement no. {}\n'.format(measurementIndex))
    
        fileNames[measurementIndex] = stanceData['fileNames'][measurementIndex]    
        
    #        %keyboard
        
    # %     if (length(stanceData.firstVForce[measurementIndex]) > normalizedLength)
    # %        error('Normalization would shorten record') 
    # %     end
        
        originalRelativeTimes = np.arange(0, len(stanceData['firstVForces'][measurementIndex])) / (len(stanceData['firstVForces'][measurementIndex]) - 1)   
        firstNormalizedVForce = np.interp(normalizedRelativeTimes, originalRelativeTimes, stanceData['firstVForces'][measurementIndex])
        firstNormalizedAPForce = np.interp(normalizedRelativeTimes, originalRelativeTimes, stanceData['firstAPForces'][measurementIndex])
        firstNormalizedMLForce = np.interp(normalizedRelativeTimes, originalRelativeTimes, stanceData['firstMLForces'][measurementIndex])
    
    # %     if (length(stanceData.secondVForce[measurementIndex]) > normalizedLength)
    # %        error('Normalization would shorten record') 
    # %     end    
        
        originalRelativeTimes = np.arange(0, len(stanceData['secondVForces'][measurementIndex])) / (len(stanceData['secondVForces'][measurementIndex]) - 1)
        secondNormalizedVForce = np.interp(normalizedRelativeTimes, originalRelativeTimes, stanceData['secondVForces'][measurementIndex])
        secondNormalizedAPForce = np.interp(normalizedRelativeTimes, originalRelativeTimes, stanceData['secondAPForces'][measurementIndex])
        secondNormalizedMLForce = np.interp(normalizedRelativeTimes, originalRelativeTimes, stanceData['secondMLForces'][measurementIndex])
    
        
        #%[result, ruleIndex] = isLeftLegFirst(firstMLWave, secondMLWave, 0)     
    
       #  %if (isLeftLegMedioLateralForce(recordData.secondVForce))
         
        if (stanceData['leftLegFirstFlags'][measurementIndex]) :      
            leftVForces[measurementIndex, :] = firstNormalizedVForce
            leftAPForces[measurementIndex, :] = firstNormalizedAPForce
            leftMLForces[measurementIndex, :] = firstNormalizedMLForce
             
            leftStrikeIndexes[measurementIndex] = stanceData['firstStrikeIndexes'][measurementIndex]
            leftOffIndexes[measurementIndex] = stanceData['firstOffIndexes'][measurementIndex]
             
            leftVStrikeMeans[measurementIndex] = stanceData['firstVStrikeMeans'][measurementIndex]
            leftAPStrikeMeans[measurementIndex] = stanceData['firstAPStrikeMeans'][measurementIndex]
            leftMLStrikeMeans[measurementIndex] = stanceData['firstMLStrikeMeans'][measurementIndex]
            leftVOffMeans[measurementIndex] = stanceData['firstVOffMeans'][measurementIndex]
            leftAPOffMeans[measurementIndex] = stanceData['firstAPOffMeans'][measurementIndex]
            leftMLOffMeans[measurementIndex] = stanceData['firstMLOffMeans'][measurementIndex]         
             
            rightVForces[measurementIndex, :] = secondNormalizedVForce
            rightAPForces[measurementIndex, :] = secondNormalizedAPForce
            rightMLForces[measurementIndex, :] = secondNormalizedMLForce
             
            rightStrikeIndexes[measurementIndex] = stanceData['secondStrikeIndexes'][measurementIndex]
            rightOffIndexes[measurementIndex] = stanceData['secondOffIndexes'][measurementIndex]
    
            rightVStrikeMeans[measurementIndex] = stanceData['secondVStrikeMeans'][measurementIndex]
            rightAPStrikeMeans[measurementIndex] = stanceData['secondAPStrikeMeans'][measurementIndex]
            rightMLStrikeMeans[measurementIndex] = stanceData['secondMLStrikeMeans'][measurementIndex]
            rightVOffMeans[measurementIndex] = stanceData['secondVOffMeans'][measurementIndex]
            rightAPOffMeans[measurementIndex] = stanceData['secondAPOffMeans'][measurementIndex]
            rightMLOffMeans[measurementIndex] = stanceData['secondMLOffMeans'][measurementIndex]         
        else :
             leftVForces[measurementIndex, :] = secondNormalizedVForce
             leftAPForces[measurementIndex, :] = secondNormalizedAPForce
             leftMLForces[measurementIndex, :] = secondNormalizedMLForce
             leftStrikeIndexes[measurementIndex] = stanceData['secondStrikeIndexes'][measurementIndex]
             leftOffIndexes[measurementIndex] = stanceData['secondOffIndexes'][measurementIndex]
    
             leftVStrikeMeans[measurementIndex] = stanceData['secondVStrikeMeans'][measurementIndex]
             leftAPStrikeMeans[measurementIndex] = stanceData['secondAPStrikeMeans'][measurementIndex]
             leftMLStrikeMeans[measurementIndex] = stanceData['secondMLStrikeMeans'][measurementIndex]
             leftVOffMeans[measurementIndex] = stanceData['secondVOffMeans'][measurementIndex]
             leftAPOffMeans[measurementIndex] = stanceData['secondAPOffMeans'][measurementIndex]
             leftMLOffMeans[measurementIndex] = stanceData['secondMLOffMeans'][measurementIndex]         
             
             rightVForces[measurementIndex, :] = firstNormalizedVForce
             rightAPForces[measurementIndex, :] = firstNormalizedAPForce
             rightMLForces[measurementIndex, :] = firstNormalizedMLForce
             
             rightStrikeIndexes[measurementIndex] = stanceData['firstStrikeIndexes'][measurementIndex]
             rightOffIndexes[measurementIndex] = stanceData['firstOffIndexes'][measurementIndex]    
             
             rightVStrikeMeans[measurementIndex] = stanceData['firstVStrikeMeans'][measurementIndex]
             rightAPStrikeMeans[measurementIndex] = stanceData['firstAPStrikeMeans'][measurementIndex]
             rightMLStrikeMeans[measurementIndex] = stanceData['firstMLStrikeMeans'][measurementIndex]
             rightVOffMeans[measurementIndex] = stanceData['firstVOffMeans'][measurementIndex]
             rightAPOffMeans[measurementIndex] = stanceData['firstAPOffMeans'][measurementIndex]
             rightMLOffMeans[measurementIndex] = stanceData['firstMLOffMeans'][measurementIndex]          
    
    
    patternData = dict(
        fileNames = fileNames[: measurementIndex],
        leftVForces = leftVForces[: measurementIndex, :],
        leftAPForces = leftAPForces[: measurementIndex, :],
        leftMLForces = leftMLForces[: measurementIndex, :],
        
        rightVForces = rightVForces[: measurementIndex, :],
        rightAPForces = rightAPForces[: measurementIndex, :],
        rightMLForces = rightMLForces[: measurementIndex, :],
            
        leftVStrikeMeans = leftVStrikeMeans[: measurementIndex],
        leftMLStrikeMeans = leftMLStrikeMeans[: measurementIndex],
        leftAPStrikeMeans = leftAPStrikeMeans[: measurementIndex],
            
        rightVStrikeMeans = rightVStrikeMeans[: measurementIndex],
        rightMLStrikeMeans = rightMLStrikeMeans[: measurementIndex],
        rightAPStrikeMeans = rightAPStrikeMeans[: measurementIndex],
             
        leftVOffMeans = leftVOffMeans[: measurementIndex],
        leftMLOffMeans = leftMLOffMeans[: measurementIndex],
        leftAPOffMeans = leftAPOffMeans[: measurementIndex],
            
        rightVOffMeans = rightVOffMeans[: measurementIndex],
        rightMLOffMeans = rightMLOffMeans[: measurementIndex],
        rightAPOffMeans = rightAPOffMeans[: measurementIndex],
             
        leftStrikeIndexes = leftStrikeIndexes[: measurementIndex],
        leftOffIndexes = leftOffIndexes[: measurementIndex],
        rightStrikeIndexes = rightStrikeIndexes[: measurementIndex],
        rightOffIndexes = rightOffIndexes[: measurementIndex]
    )

    return patternData   

def summarizePatterns(patternData) :
    '''
    summarizePatterns shows histograms, maxima, minima 
    that characterize leg-assigned time-normalized stance-related fragments
    of GRF signals.
    
    Additionally, lengths of stance-related fragments average relations between 
    measurement length and durations of step and stance phase are quantified
    by mean values and standard deviation.
    
    summarizePatterns also shows histograms of mean magnitudes 
    of samples that precede/follow the foot-strike/off events.
    These plots show in which GRF signals and how often artifacts occur
    related to faults in the initial construction of the walkway.
    
    This function can be used to validate patterns and raw measurements.
    One can look for outstanding patterns 
    and then use them to identify and check measurements that might be outliers.
    
    Input:
    - patternData : stance-related data created by using 
                       the extractStancesFromMeasurements function.
    
    Output:
    - Figures and texts are displayed.
    
    Created on May 26, 2022
    
    Author: Marek Parfieniuk    
    '''


    fileNames = patternData['fileNames']
    leftVForces = patternData['leftVForces']
    rightVForces = patternData['rightVForces']
    
    leftStrikeIndexes = patternData['leftStrikeIndexes']
    leftOffIndexes = patternData['leftOffIndexes']
    
    rightStrikeIndexes = patternData['rightStrikeIndexes']
    rightOffIndexes = patternData['rightOffIndexes']



    numberOfBins = 30

    fig, axs = plt.subplots(1, 2, sharey = False, tight_layout = True)
    resizeAndEntitleWindow(fig, 'Stance-phase histograms')

    ax = axs[0]
    t = leftOffIndexes - leftStrikeIndexes
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Left-leg stance-phase duration [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(400, 1100)
    ax.set_ylim(0, 1700)    

    ax = axs[1]
    t = rightOffIndexes - rightStrikeIndexes
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Right-leg stance-phase duration [samples]'
                  + '\nMin: {:5d}, max: {:5d}'.format(np.min(t), np.max(t)))
    ax.set_xlim(400, 1100)
    ax.set_ylim(0, 1700)    


    fig, axs = plt.subplots(3, 2, sharey = False, tight_layout = True)
    resizeAndEntitleWindow(fig, 'Histograms of extremums of vertical and anterior-posterior forces')


    ax = axs[0][0]
    t = np.max(patternData['leftVForces'], axis = 1)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Left-leg vertical force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 1700)
    ax.set_ylim(0, 1500)    

    ax = axs[0][1]
    t = np.max(patternData['rightVForces'], axis = 1)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Right-leg vertical force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 1700)
    ax.set_ylim(0, 1500)    

    ax = axs[1][0]
    t = np.max(patternData['leftAPForces'], axis = 1)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Left-leg anterior-posterior force: maximum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 400)
    ax.set_ylim(0, 1800)    

    ax = axs[1][1]
    t = np.max(patternData['rightAPForces'], axis = 1)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Right-leg anterior-posterior force: minimum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(0, 400)
    ax.set_ylim(0, 1800)    

    ax = axs[2][0]
    t = np.min(patternData['leftAPForces'], axis = 1)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Left-leg anterior-posterior force: minimum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-350, 0)
    ax.set_ylim(0, 1800)    

    ax = axs[2][1]
    t = np.min(patternData['rightAPForces'], axis = 1)
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Right-leg anterior-posterior force: minimum value [N]'
                  + '\nMin: {:5.2f}, max: {:5.2f}'.format(np.min(t), np.max(t)))
    ax.set_xlim(-350, 0)
    ax.set_ylim(0, 1800)

    
    fig, axs = plt.subplots(3, 2, sharey = False, tight_layout = True)
    
    resizeAndEntitleWindow(fig, 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events of left leg')

    ax = axs[0][0]
    t = patternData['leftVStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples' + '\nof vertical force of left leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[0][1]
    t = patternData['leftVOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples' + '\nof vertical force of left leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[1][0]
    t = patternData['leftAPStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples' + '\nof anterior-posterior force of left leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[1][1]
    t = patternData['leftAPOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples' + '\nof anterior-posterior force of left leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[2][0]
    t = patternData['leftMLStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples' + '\nof medio-lateral force of left leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[2][1]
    t = patternData['leftMLOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples' + '\nof medio-lateral force of left leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    



    fig, axs = plt.subplots(3, 2, sharey = False, tight_layout = True)
    
    resizeAndEntitleWindow(fig, 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events of right leg')

    ax = axs[0][0]
    t = patternData['rightVStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples' + '\nof vertical force of right leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[0][1]
    t = patternData['rightVOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples' + '\nof vertical force of right leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[1][0]
    t = patternData['rightAPStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples' + '\nof anterior-posterior force of right leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[1][1]
    t = patternData['rightAPOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples' + '\nof anterior-posterior force of right leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[2][0]
    t = patternData['rightMLStrikeMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of pre-foot-strike samples' + '\nof medio-lateral force of right leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

    ax = axs[2][1]
    t = patternData['rightMLOffMeans']
    ax.hist(t, bins = numberOfBins)
    ax.grid(True)
    ax.set_ylabel('Number of measurements')
    ax.set_xlabel('Mean magnitude of post-foot-off samples' + '\nof medio-lateral force of right leg [N]'
                  + '\nMax: {:5.2f}'.format(np.max(t)))
    ax.set_xlim(0, 50)
    ax.set_ylim(0, 12000)    

   
