function gaitMeasurements = loadMeasurements(fileDirectoryPath, fileNamePattern)

% loadMeasurements reads CSV files of given directory, 
% only those whose names match a given pattern (regular expression).
% It loads gait measurement data (samples of signals of force plates) 
% from those files into an array of structures.
% Each structure is composed of fields-vectors 
% which represent name and columns 
% of one CSV file (one measurement).
%
% The file name (without extension) can be used 
% as the measurement identifier-label.
%
% This function assumes that the directory contains person/participant-related
% subdirectiories '001'...'324' that contain shoe-related subdirectiories 
% that contain only measurement-related .CSV files.
% Errors might result from presence of other subdirectories and files
% in participant-related subdirectories.
%
% This function temporarily changes the working directory. 
% In case of error, it does not return to the starting directory.
%
% Input:
% - fileDirectoryPath : path to directory with measurement files
% - fileNamePattern : pattern (regular expression) for file names
%
% Output:
% - gaitMeasurements : cell array of structures of column vectors 
%                       of samples of signals of force plates 
% 
% Examples of file-name patterns:
% - '.*' : All measurements
% - '.*\.f\..*' : All female-related measurements
% - '.*\.m\..*' : All male-related measurements
% - '024.*'  : All measurements related to the 024th participant, regardless of shoes            
% - '024.*\.spr.*' : All measurements related to the 024th participant
%                       in sport shoes
% 
% Usage example:
% - gaitMeasurements = loadMeasurements('c:\gaitMeasurementsInCSVFiles', '.*')
%
% Created on Sep. 28, 2021
% 
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>


currentDirectory = cd(fileDirectoryPath);

% Preallocate empty cell matrix with more rows than data files.
gaitMeasurements = cell(13702, 1);      
fileCount = 0;

% Attention! This loop is based on fixed, hard-coded number of participants.
for participantIndex = 1 : 324 
    participantDirectory = sprintf('%03d', participantIndex);
    
    fprintf('%s : participant is processed.\n', participantDirectory);
           
    cd(participantDirectory);   
    shoeDirectoryList = dir();    
    for shoeDirectoryIndex = 1 : length(shoeDirectoryList)
        shoeDirectory = shoeDirectoryList(shoeDirectoryIndex).name;
        if (startsWith(shoeDirectory, '.'))
            continue;    % Skip '.' and '..' special directiories
        end
        
        fprintf('  #%s : shoe is processed.\n', shoeDirectory)
                      
        cd(shoeDirectory);   
        fileList = dir('*.csv');            
        for fileIndex = 1 : length(fileList)            
            fileName = fileList(fileIndex).name;
            if (endsWith(fileName, 'ptrn.csv'))
                continue;    % Skip files with gait patters 
            end                                        
            if (startsWith(fileName, '.'))
                continue;    % Skip '.' and '..' special directiories
            end                            
            %fileName = fileName(1 : (end - 4)); % to remove CSV extension
            if (isempty(regexpi(fileName, fileNamePattern)))
                % fprintf('Skipped files whose names do not match the pattern
                continue;    
            end
            
            fprintf('    %s is processed.\n', fileName)
            
            data = zeros(10000, 12);           

            lineIndex = -1;            
            
            fid = fopen(fileName, 'r');
            while 1
                fileLine = fgetl(fid);
                if (~ischar(fileLine)) % End-of-file
                    break;      
                end
                lineIndex = lineIndex + 1;
                if (lineIndex == 0)
                    continue;       % To skip the header line.
                end
                data(lineIndex, :) = sscanf(fileLine, '%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f', 12);
            end
            fclose(fid);
            
            measurementRecord.fileName = fileName;
            
            measurementRecord.firstVForce = data(1 : lineIndex, 1);
            measurementRecord.firstAPForce = data(1 : lineIndex, 2); 
            measurementRecord.firstMLForce = data(1 : lineIndex, 3); 
            measurementRecord.firstVTorque = data(1 : lineIndex, 4);             
            measurementRecord.firstAPCoP = data(1 : lineIndex, 5); 
            measurementRecord.firstMLCoP = data(1 : lineIndex, 6);
                                  
            measurementRecord.secondVForce = data(1 : lineIndex, 7); 
            measurementRecord.secondAPForce = data(1 : lineIndex, 8); 
            measurementRecord.secondMLForce = data(1 : lineIndex, 9);             
            measurementRecord.secondVTorque = data(1 : lineIndex, 10); 
            measurementRecord.secondAPCoP = data(1 : lineIndex, 11); 
            measurementRecord.secondMLCoP = data(1 : lineIndex, 12); 
                       
            fileCount = fileCount + 1;
            gaitMeasurements{fileCount} = measurementRecord;                        
        end
        cd('..')
    end
    cd('..')
end

% Trim matrix to remove empty cells
gaitMeasurements = gaitMeasurements(1 : fileCount);    

cd(currentDirectory)

fprintf(1, 'Done! %d CSV files have been processed.\n', fileCount)

end

