function participantData = loadParticipants(filePath)

% loadParticipants loads the participant data 
% from a given CSV file into a structure of column vectors, 
% each of which contains data of one kind (index-identifier, weight, etc.)
% of all participants.
%
% loadParticipants returns a structure with the following fields:
% - index (string from '001' to '324')
% - gender (as string; 'f' for female; 'm' for male)
% - date of participant measurement, in the 'yyyy-mm-dd' format)
% - age
% - weight
% - height
% each of which is an array that contains the corresponding data
% of all participants (data of one column of the CSV files).
%
% Examples of accessing output data:
% - participantData.gender(13) : gives the gender of the 13th participant
% - participantData.weight(participant.gender == 'f') : gives weights of all females  
%
% The index field can be used to associate participant's data with
% measurements of his/her gait.
%
% Input:
% - filePath : path to a CSV file (participantData.csv) with participant data
%
% Output:
% - participantData : structure of arrays that contain participant data
%
% Created on Nov. 12, 2021
% 
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Pahrfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

fileID = fopen(filePath);
fileColumns = textscan(fileID,'%s %s %s %f %f %f %*[^\n]', 'Delimiter', ',', 'Headerlines', 1);
fclose(fileID);

participantData.indexs = string(fileColumns{1});
participantData.genders = string(fileColumns{2});
participantData.dates = string(fileColumns{3});
participantData.ages = fileColumns{4};
participantData.weights = fileColumns{5};
participantData.heights = fileColumns{6};

end

