function patternData = loadPatterns(fileDirectoryPath, fileNamePattern, patternSize)

% loadPatterns reads PTRN.CSV files of given directory, 
% only those whose names match a given pattern (regular expression).
% Each file is assumed to describe one gait pattern i.e. 
% samples of the time-normalized stance-related fragment 
% of a gait-GRF measurement described 
% by the correspoding CSV file of the essential data set. 
%
% Data are loaded from files into an array of structures.
% Each structure describes one gait-pattern
% and comprises fields
% each of which is a vector or matrix that contains the corresponding data
% of all files.
% In "...Force" fields, one row comprises samples 
% of the corresponding signal of one file. 
%
% The file name (including extension) can be used 
% as a pattern identifier-label.
%
% This function assumes that the given directory contains person/subject-related
% subdirectiories '001'...'324' that contain shoe-related subdirectiories 
% that contain only pattern-related PTRN.CSV files.
% Errors will result from presence of other subdirectories and files
% in subject-related subdirectories.
%
% This function temporarily changes the working directory. 
% In case of error it does not return to the starting directory.
%
% Input:
% - fileDirectoryPath : path to directory of gait-pattern files
% - fileNamePattern : pattern (regular expression) of file names
% - patternSize : expected number of samples in each signal fragment 
%
% Output:
% - patternData : cell array of structures that describe gait-GRF patterns 
% 
% Examples of file-name patterns:
% - '.*' : All records
% - '.*\.f\..*' : All female-related records
% - '.*\.m\..*' : All male-related records
% - '024.*'  : All records related to the 024th subject, regardless of shoes%              
% - '024.*\.spr.*' : All records related to the 024th subject
%                       in sport shoes
% 
% Usage example:
% - patternData = loadPatterns('c:\gaitDataInCSVFiles', '.*', 0)
%
% Created on May 8, 2022
% 
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>


currentDirectory = cd(fileDirectoryPath)    

% Preallocate matrices.
measurementCount = 13702;

fileNames = cell(measurementCount, 1);
leftVForces = zeros(measurementCount, patternSize);
rightVForces = zeros(measurementCount, patternSize);
leftAPForces = zeros(measurementCount, patternSize);
rightAPForces = zeros(measurementCount, patternSize);
leftMLForces = zeros(measurementCount, patternSize);
rightMLForces = zeros(measurementCount, patternSize);
leftOffIndexes = zeros(measurementCount, 1);
rightOffIndexes = zeros(measurementCount, 1);
leftVStrikeMeans = zeros(measurementCount, 1);
leftAPStrikeMeans = zeros(measurementCount, 1);
leftMLStrikeMeans = zeros(measurementCount, 1);
leftVOffMeans = zeros(measurementCount, 1);
leftAPOffMeans = zeros(measurementCount, 1);
leftMLOffMeans = zeros(measurementCount, 1);
rightVStrikeMeans = zeros(measurementCount, 1);
rightAPStrikeMeans = zeros(measurementCount, 1);
rightMLStrikeMeans = zeros(measurementCount, 1);
rightVOffMeans = zeros(measurementCount, 1);
rightAPOffMeans = zeros(measurementCount, 1);
rightMLOffMeans = zeros(measurementCount, 1);
leftStrikeIndexes = zeros(measurementCount, 1);
rightStrikeIndexes = zeros(measurementCount, 1);





fileCount = 0;

% Attention! This loop is based on fixed, hard-coded number of subjects.
%for participantIndex = 1 : 3 %324 
for participantIndex = 1 : 324 
    participantDirectory = sprintf('%03d', participantIndex);
    
    fprintf('%s : participant is processed.\n', participantDirectory);
           
    cd(participantDirectory);   
    shoeDirectoryList = dir();    
    for shoeDirectoryIndex = 1 : length(shoeDirectoryList)
        shoeDirectory = shoeDirectoryList(shoeDirectoryIndex).name;
        if (startsWith(shoeDirectory, '.'))
            continue;    % Skip '.' and '..' special directiories
        end
        
        fprintf('  #%s : shoe is processed.\n', shoeDirectory)
                      
        cd(shoeDirectory);   
        fileList = dir('*.ptrn.csv');            
        for fileIndex = 1 : length(fileList)            
            fileName = fileList(fileIndex).name;
            if (startsWith(fileName, '.'))
                continue;    % Skip '.' and '..' special directiories
            end            
            if (isempty(regexpi(fileName, fileNamePattern)))
                % fprintf('Skipped files whose names do not match the pattern
                continue;    
            end
            fileCount = fileCount + 1;
            
            fprintf('    %s is processed.\n', fileName)
            
            % Preallocate matrix for data.
            data = zeros(patternSize, 14);           

            lineIndex = -1;                        
            try
                fid = fopen(fileName, 'r');
                while 1
                    fileLine = fgetl(fid);
                    if (~ischar(fileLine)) % End-of-file
                        break;      
                    end
                    lineIndex = lineIndex + 1;                
                    if (lineIndex == 0)
                        continue;       % To skip the header line.
                    end
                    if (lineIndex > patternSize)
                        error('File comprises more data lines than expected, more that %d (pattern size)\n', patternSize);
                    end
                    
%                    keyboard
                    
                    t = sscanf(fileLine, '%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d');
                    data(lineIndex, 1 : numel(t)) = t;
                end
                fclose(fid);
            catch ME
                fclose(fid);
                rethrow(ME)
            end
            
            if (lineIndex < patternSize)
                error('There are only %d data lines in this file, while it is expeced to comprise %d (pattern size) data lines\n', lineIndex, patternSize);
            end

            %gaitPatterns.fileName{fileCount} = fileName(1 : (end - 9)); % to trim the PTRN.CSV extension;
            fileNames{fileCount} = fileName;
            
            leftVForces(fileCount, :) = data(1 : lineIndex, 1); 
            leftAPForces(fileCount, :) = data(1 : lineIndex, 2);
            leftMLForces(fileCount, :) = data(1 : lineIndex, 3);
            rightVForces(fileCount, :) = data(1 : lineIndex, 4); 
            rightAPForces(fileCount, :) = data(1 : lineIndex, 5);
            rightMLForces(fileCount, :) = data(1 : lineIndex, 6);                      

            leftVStrikeMeans(fileCount) = data(1, 7); 
            leftAPStrikeMeans(fileCount) = data(1, 8);
            leftMLStrikeMeans(fileCount) = data(1, 9);
            rightVStrikeMeans(fileCount) = data(1, 10); 
            rightAPStrikeMeans(fileCount) = data(1, 11);
            rightMLStrikeMeans(fileCount) = data(1, 12);                      

            leftVOffMeans(fileCount) = data(lineIndex, 7); 
            leftAPOffMeans(fileCount) = data(lineIndex, 8);
            leftMLOffMeans(fileCount) = data(lineIndex, 9);
            rightVOffMeans(fileCount) = data(lineIndex, 10); 
            rightAPOffMeans(fileCount) = data(lineIndex, 11);
            rightMLOffMeans(fileCount) = data(lineIndex, 12);                      
            
            leftStrikeIndexes(fileCount) = data(1, 13); 
            leftOffIndexes(fileCount) = data(lineIndex, 13); 
            rightStrikeIndexes(fileCount) = data(1, 14); 
            rightOffIndexes(fileCount) = data(lineIndex, 14);
        end
        cd('..')
    end
    cd('..')
end

% Trim arrays to remove empty rows

patternData.fileNames = fileNames(1 : fileCount);
patternData.leftVForces = leftVForces(1 : fileCount, :);
patternData.rightVForces = rightVForces(1 : fileCount, :);
patternData.leftAPForces = leftAPForces(1 : fileCount, :);
patternData.rightAPForces = rightAPForces(1 : fileCount, :);
patternData.leftMLForces = leftMLForces(1 : fileCount, :);
patternData.rightMLForces = rightMLForces(1 : fileCount, :);
patternData.leftStrikeIndexes = leftStrikeIndexes(1 : fileCount);
patternData.rightStrikeIndexes = rightStrikeIndexes(1 : fileCount);
patternData.leftOffIndexes = leftOffIndexes(1 : fileCount);
patternData.rightOffIndexes = rightOffIndexes(1 : fileCount);
patternData.leftVStrikeMeans = leftVStrikeMeans(1 : fileCount);
patternData.leftAPStrikeMeans = leftAPStrikeMeans(1 : fileCount);
patternData.leftMLStrikeMeans = leftMLStrikeMeans(1 : fileCount);
patternData.leftVOffMeans = leftVOffMeans(1 : fileCount);
patternData.leftAPOffMeans = leftAPOffMeans(1 : fileCount);
patternData.leftMLOffMeans = leftMLOffMeans(1 : fileCount);
patternData.rightVStrikeMeans = rightVStrikeMeans(1 : fileCount);
patternData.rightAPStrikeMeans = rightAPStrikeMeans(1 : fileCount);
patternData.rightMLStrikeMeans = rightMLStrikeMeans(1 : fileCount);
patternData.rightVOffMeans = rightVOffMeans(1 : fileCount);
patternData.rightAPOffMeans = rightAPOffMeans(1 : fileCount);
patternData.rightMLOffMeans = rightMLOffMeans(1 : fileCount);

cd(currentDirectory)

fprintf(1, 'Done! %d PTRN.CSV files have been processed.\n', fileCount)

end

