% This script demonstrates how to use the Matlab routines that accompany
% the data set of gait measurements described in the article
% 
% "A dataset of measurements of ground reaction forces of normal gait 
% of young adults wearing various footwear"
% 
% by Marcin Derlatka and Marek Parfieniuk
%
% submitted to the "Scientific Data" journal.
%
% Created on Sep. 27, 2021
%
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

fprintf('\n-------------------------------------------\n\n');
fprintf('Welcome to a demo of Matlab routines that accompany\n');
fprintf('the dataset of gait measurements described in the article\n\n');
fprintf('"A dataset of measurements of ground reaction forces of normal gait\n');
fprintf('of young adults wearing various footwear"\n\n');
fprintf('by Marcin Derlatka and Marek Parfieniuk\n\n');
fprintf('submitted to the "Scientific Data" journal.\n');

if (~exist('fileDirectoryPath', 'var'))
    fileDirectoryPath = cd;     % default = current directory 
end

fprintf('\n-------------------------------------------\n')
fprintf('Please enter the path to a directory which contains\n')
fprintf('the participantData.csv file and gaitMeasurements.mat file\n')
fprintf('or gaitMeasurementsInCSVFiles subdirectory.\n')
fprintf('Or, simply press the ENTER key in order to use the recent directory\n(%s).\n', fileDirectoryPath)
s = input('>>>', 's');
if (~isempty(s))
    fileDirectoryPath = s;
end

toReloadData = ~exist('participantData', 'var');
toSummarizeData = toReloadData;

if (~toReloadData)
    fprintf('\n-------------------------------------------\n')
    fprintf('In order to reload the participant data, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not reload the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toReloadData = ((s == 'y') | (s == 'Y'));
    end
end    
    
if (toReloadData)
    fprintf('\n-------------------------------------------\n')
    fprintf('Loading participant data.\n')
    participantData = loadParticipants([fileDirectoryPath '/participantData.csv']);
    fprintf('OK. Participant data loaded.\n')
    toSummarizeData = 1;
end    

if (~toSummarizeData)
    fprintf('\n-------------------------------------------\n')
    fprintf('In order to summarize the participant data, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not summarize the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toSummarizeData = ((s == 'y') || (s == 'Y'));
    end
end    

if (toSummarizeData)
    fprintf('\n-------------------------------------------\n')    
    summarizeParticipants(participantData)
end

fprintf('\n-------------------------------------------\n')
toReloadData = ~exist('gaitMeasurements', 'var');
if (~toReloadData)
    fprintf('In order to reload gait measurements from CSV files, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not reload the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toReloadData = ((s == 'y') || (s == 'Y'));
    end
end    
    
if (toReloadData)
    matFilePath = [fileDirectoryPath '/gaitMeasurements.mat'];
    if (exist(matFilePath, 'file') == 2)
        fprintf('Loading gait measurements from MAT file. This may take several minutes.\n')
        load(matFilePath);  % gaitMeasurements should be the only variable in the MAT file.
    else 
        fprintf('Loading gait measurements from CSV files. This may take a dozen or so minutes.\n')
        gaitMeasurements = loadMeasurements([fileDirectoryPath '/gaitMeasurementsInCSVFiles'], '.*');
    end
    fprintf('OK. %d gait measurements loaded.\n', length(gaitMeasurements))
end    

fprintf('\n-------------------------------------------\n\n');
fprintf('Now, You can review measurements one by one.\n');
fprintf('Please enter the index of a measurement to be presented,\n');
fprintf('or the name of the CSV file (without .CSV extension).\n')
fprintf('Or, please press "Enter" in order to review a randomly selected measurement.\n')
fprintf('Or, please enter "q" in order to stop reviewing measurements.\n')
while (1)
    try 
        toSmoothStanceBoundaries = 1;
        toBeVerbose = 0;
        s = input('>>> ', 's');
        if (isempty(s))
            showMeasurement(gaitMeasurements, randi(length(gaitMeasurements)), toSmoothStanceBoundaries, toBeVerbose, figure)
        else
            if (s == 'q')
                break;                
            end
            idx = str2double(s);
            if (isnan(idx))
                showMeasurement(gaitMeasurements, s, toSmoothStanceBoundaries, toBeVerbose, figure)
            else
                showMeasurement(gaitMeasurements, idx, toSmoothStanceBoundaries, toBeVerbose, figure)
            end
        end
    catch ex        
        disp(getReport(ex,'extended','hyperlinks','off'))
%        keyboard
    end            
end

toReloadData = ~exist('stanceData', 'var');
if (~toReloadData)
    fprintf('\n-------------------------------------------\n')
    fprintf('In order to extract data of stance-related parts of measurements, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not extract waves.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toReloadData = ((s == 'y') || (s == 'Y'));
    end
end    
    
if (toReloadData)
    fprintf('\n-------------------------------------------\n\n');
    fprintf('Extracting data of stance-related parts of GRF signals. These data can be used as patterns in research\n');
    fprintf('related to pattern recognition and classification.\n')
    stanceData = extractStancesFromMeasurements(gaitMeasurements, 100, 1);
    fprintf('OK. Stance-related data extracted.\n')
    toSummarizeData = 1;
end

if (~toSummarizeData)
    fprintf('\n-------------------------------------------\n')
    fprintf('In order to summarize the stance-related data, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not summarize the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toSummarizeData = ((s == 'y') || (s == 'Y'));
    end
end

if (toSummarizeData)
    fprintf('\n-------------------------------------------\n')
    summarizeStances(stanceData)
end

fprintf('\n-------------------------------------------\n\n');
fprintf('Now, You can review stance-related parts of measurements\n');
fprintf('of ground reaction forces.');
fprintf('Please enter a regular expression that determines identifiers\n');
fprintf('of a subset of measurements. For example:\n')
fprintf('- 027.* : selects all measurements related to the 27th participant.\n')
fprintf('- 027.*spr.* : selects measurements related to the 27th participant wearing sport shoes.\n')
fprintf('- 027.*\\.2\\..* : selects measurements related to the 27th participant wearing the 2nd footwear.\n')
fprintf('Or, please enter "q" in order to stop reviewing stance-related parts of GRF signals.\n')
while (1)
    try 
        s = input('>>> ', 's');
        if (s == 'q')
            break;                
        end
        idx = str2double(s);
        if (~isnan(idx))
            s = [s '.*'];
        end                
        showStances(stanceData, s)
    catch ex
        fprintf(getReport(ex,'extended','hyperlinks','off'))        
%        keyboard
    end            
end

fprintf('\n-------------------------------------------\n\n');
fprintf('Measurement demo has ended.\n')
fprintf('\n-------------------------------------------\n\n');


    
