# -*- coding: utf-8 -*-
"""
runDemo demonstrates how to use functions of the gait.py module-package
that accompanies the dataset 
of gait measurements described in the article

"A dataset of measurements of ground reaction forces of normal gait 
of young adults wearing various footwear"
by Marcin Derlatka and Marek Parfieniuk

submitted to the "Scientific Data" journal.

Created on Sep. 26, 2021

@author: Marek Parfieniuk

Copyright (c) 2021 Marek Parfieniuk

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
"""


import gait
import matplotlib.pyplot as plt
import os
import random
import re
import traceback



print('\n-------------------------------------------\n')
print('Welcome to a demo of Python routines that accompany')
print('the dataset of gait measurements described in the article\n')
print('"A dataset of measurements of ground reaction forces of normal gait')
print('of young adults wearing various footwear"')
print('by Marcin Derlatka and Marek Parfieniuk\n')
print('submitted to the "Scientific Data" journal.')
print('\n-------------------------------------------\n')
print('This demo creates figures that present data.')
print('Figures should be reviewed AFTER the execution of the demo.')
print('They may be irreponsible when the script is working,')
print('even when it only waits for user input.')
print('')
print('This is related to some interaction-related limitations')
print('of the matplotlib library, and of Python itself')
print('A more interactive script could be created, but it would be less portable,')
print('being dependent on how Python is configured, requiring a particular GUI toolkit.')
print('')
print('For related information, please see:')
print('https://matplotlib.org/stable/users/interactive.html')
print('')
print('In order to review figures, please ensure that the Python interpreter')
print('enters interactive mode after executing the script.')
print('Running the demo from the command line, one should write')
print('   python -i runDemo.py')
print('Or one can run the demo under the IPython interactive shell.')


if (not ('fileDirectoryPath' in vars())) :
    fileDirectoryPath = os.getcwd()      # default = current directory  
    
print('\n-------------------------------------------\n')
print('Please enter the path to the data directory, which contains')
print('the participantData.csv file and the ''001''...''324'' subdirectories with gait data in CSV files.')
print('Or, simply press the ENTER key in order to use the recent directory\n({:s}).'.format(fileDirectoryPath))
s = input('>>>')
if (s != '') :
    fileDirectoryPath = s

toReloadData = (not ('participantData' in vars()))
toSummarizeData = toReloadData

if (not toReloadData) :
    print('\n-------------------------------------------\n')
    print('In order to reload the participant data, please enter the "y" letter.')
    print('Or, simply press the ENTER to not reload the data.')
    s = input('>>>')
    toReloadData = ((s == 'y') or (s == 'Y'))
    
if (toReloadData) :
    print('\n-------------------------------------------\n')
    print('Loading participant data ...')
    participantData = gait.loadParticipants(fileDirectoryPath + r'/participantData.csv')
    print('OK. Participant data loaded.')
    toSummarizeData = True;

if (not toSummarizeData) :
    print('\n-------------------------------------------\n')
    print('In order to summarize the participant data, please enter the "y" letter.')
    print('Or, simply press the ENTER to not summarize the data.')
    s = input('>>>')
    toSummarizeData = ((s == 'y') or (s == 'Y'))

if (toSummarizeData) :
    print('\n-------------------------------------------\n')
    gait.summarizeParticipants(participantData)
    plt.pause(1)
    
    
#plt.show(block=True);

toReloadData = (not ('gaitMeasurements' in vars()))

if (not toReloadData) :
    print('\n-------------------------------------------\n')
    print('In order to reload the gait data from CSV files, please enter the "y" letter.')
    print('Or, simply press the ENTER to not reload the data.')
    s = input('>>>')
    toReloadData = ((s == 'y') or (s == 'Y'))    
    
if (toReloadData) :    
    print('\n-------------------------------------------\n')    
    print('Loading gait data from CSV files. This may take a dozen or so minutes.')
    gaitMeasurements = gait.loadMeasurements(fileDirectoryPath + r'/GaitMeasurementsInCSVFiles', r'.*');
    print('OK. Gait data loaded.')

print('\n-------------------------------------------\n')
print('Now, You can review records one by one. Please enter')
print('the identifier or index of a record to be presented.')
print('Or, please press "Enter" directly in order to get a randomly-selected record.')
print('Or, please enter "q" in order to stop reviewing records.')
while (True) :
    plt.pause(1)
    try :
        toSmoothStanceBoundaries = True
        toBeVerbose = False
        s = input(">>> ")
        if (s == '') :
            gait.showMeasurement(gaitMeasurements, random.randint(1, len(gaitMeasurements))
                , toSmoothStanceBoundaries, toBeVerbose, plt.figure())
        else :
            if (s == 'q') :
                break
            elif (re.fullmatch('\d+', s)) :
                gait.showMeasurement(gaitMeasurements, int(s), toSmoothStanceBoundaries, toBeVerbose, plt.figure())
            else :
                gait.showMeasurement(gaitMeasurements, s, toSmoothStanceBoundaries, toBeVerbose, plt.figure())
                
        plt.pause(1)
    except :
        traceback.print_exc()

toReloadData = (not ('stanceData' in vars()))
toSummarizeData = toReloadData

if (not toReloadData) :
    print('\n-------------------------------------------\n')
    print('In order to extract stance-related waves from measurements, please enter the "y" letter.')
    print('Or, simply press the ENTER to not extract waves.')
    s = input('>>>')
    toReloadData = ((s == 'y') or (s == 'Y'))    

if (toReloadData) : 
    print('\n-------------------------------------------\n')
    print('Extracting stance-related waves, which can be used as patterns in research')
    print('related to pattern recognition and classification.')
    stanceData = gait.extractStancesFromMeasurements(gaitMeasurements, 100, True)
    print('OK. Stance-related data extracted.')
    toSummarizeData = True


if (not toSummarizeData) :
    print('\n-------------------------------------------\n')
    print('In order to summarize the stance-related data, please enter the "y" letter.')
    print('Or, simply press the ENTER to not summarize the data.')
    s = input('>>>')
    toSummarizeData = ((s == 'y') or (s == 'Y'))

if (toSummarizeData) :
    gait.summarizeStances(stanceData)
    plt.pause(1)

print('\n-------------------------------------------\n')
print('Now, You can review waves-patterns extracted from measurements')
print('of the vertical ground reaction forces.')
print('Please enter a regular expression that determines identifiers')
print('of a subset of patterns-waves. For example:')
print('- 027.* : selects all patterns related to the 27th participant.')
print('- 027.*spr.* : selects patterns related to the 27th participant wearing sport shoes.')
print('- 027.*\.2\..* : selects patterns related to the 27th participant wearing the 2nd footwear.')
print('Or, please enter "q" in order to stop revewing patterns.')
while (True) :
    plt.pause(1)
    try :
        s = input(">>> ")
        if (s == 'q') :
            break;   
        if (re.fullmatch('\d+', s)) :
             s = s + '.*'
        gait.showStances(stanceData, s)
        plt.pause(1)
    except :
        traceback.print_exc()

print('\n-------------------------------------------\n')
print('Demo has ended. Now, You can interact with the figures.')
print('\n-------------------------------------------\n')
