% This script demonstrates how to use Matlab routines to load and show gait patterns 
% (stance-related processed data)
% that accompany the dataset of gait measurements (raw data) described in the article
% 
% "A dataset of measurements of ground reaction forces of normal gait 
% of young adults wearing various footwear"
%     by Marcin Derlatka and Marek Parfieniuk
% submitted to the "Scientific Data" journal.
%
% Created on May 06, 2022
%
% Author: Marek Parfieniuk

% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

fprintf('\n-------------------------------------------\n\n');
fprintf('Welcome to a demo of gait patterns (preprocessed stance-related data)\n'); 
fprintf('and Matlab routines that accompany\n');
fprintf('the dataset of gait measurements (raw data) described in the article\n\n');
fprintf('"A dataset of measurements of ground reaction forces of normal gait\n');
fprintf('of young adults wearing various footwear"\n\n');
fprintf('by Marcin Derlatka and Marek Parfieniuk\n\n');
fprintf('submitted to the "Scientific Data" journal.\n');

if (~exist('fileDirectoryPath', 'var'))
    fileDirectoryPath = cd;     % default = current directory  
end

fprintf('\n-------------------------------------------\n')
fprintf('Please enter the path to a directory which contains\n')
fprintf('the participantData.csv file and gaitPatterns.mat file ')
fprintf('or gaitPatternsInCSVFiles subdirectory.\n')
%fprintf('or the ''001''...''324'' subdirectories with gait patterns in PTRN.CSV files.\n')
fprintf('Or, simply press the ENTER key in order to use the recent directory\n(%s).\n', fileDirectoryPath)
s = input('>>>', 's');
if (~isempty(s))
    fileDirectoryPath = s;
end

toReloadData = ~exist('participantData', 'var');
toSummarizeData = toReloadData;

if (~toReloadData)
    fprintf('\n-------------------------------------------\n')
    fprintf('In order to reload participant data, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not reload the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toReloadData = ((s == 'y') | (s == 'Y'));
    end
end    
    
if (toReloadData)
    if exist([fileDirectoryPath '/participantData.csv'], 'file') ~= 2
        error('The participantData.csv file does not found in the given directory.')
    end        
    
    fprintf('\n-------------------------------------------\n')
    fprintf('Loading participant data.\n')
    participantData = loadParticipants([fileDirectoryPath '/participantData.csv']);
    fprintf('OK. Participant data loaded.\n')
    toSummarizeData = 1;
end    

fprintf('\n-------------------------------------------\n')
toReloadData = ~exist('gaitPatternData', 'var');
if (~toReloadData)
    fprintf('In order to reload gait patterns from files, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not reload the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toReloadData = ((s == 'y') || (s == 'Y'));
    end
end    
    
if (toReloadData)    
    matFilePath = [fileDirectoryPath '/gaitPatterns.mat'];
    if (exist(matFilePath, 'file') == 2)
        fprintf('Loading gait patterns from MAT file. This may take a dozen or so minutes.\n')
        load(matFilePath);  % gaitPatterns should be only variable in the MAT file.
    else 
        fprintf('Loading gait patterns from PTRN.CSV files. This may take a dozen or so minutes.\n')
        gaitPatternData = loadPatterns([fileDirectoryPath '/gaitPatternsInCSVFiles'], '.*', 1100);
    end
    fprintf('OK. Gait patterns loaded.\n')
end    

if (~toSummarizeData)
    fprintf('\n-------------------------------------------\n')
    fprintf('In order to summarize the pattern data, please enter the "y" letter.\n')
    fprintf('Or, simply press the ENTER to not summarize the data.\n')
    s = input('>>>', 's');
    if (~isempty(s))
        toSummarizeData = ((s == 'y') || (s == 'Y'));
    end
end

if (toSummarizeData)
%    fprintf('\n-------------------------------------------\n')
    summarizePatterns(gaitPatternData);
end

fprintf('\n-------------------------------------------\n\n');
fprintf('Now, You can review patterns (leg-assigned time-normalized stance-related fragments of GRF signals)\n');
fprintf('of the ground reaction forces.');
fprintf('Please enter a regular expression (regex) that determines\n');
fprintf('a subset of patterns-files to be presented. For example:\n')
fprintf('- 027.* : selects all patterns related to the 27th participant.\n')
fprintf('- 027.*spr.* : selects patterns related to the 27th participant wearing sport shoes.\n')
fprintf('- 027.*\\.2\\..* : selects patterns related to the 27th participant wearing the 2nd footwear.\n')
fprintf('Or, please enter "q" in order to stop reviewing patterns.\n')
while (1)
    try 
        s = input('>>> ', 's');
        if (s == 'q')
            break;                
        end
        s = [s '.*'];
        showPatterns(gaitPatternData, s)
    catch ex
        fprintf(getReport(ex,'extended','hyperlinks','off'))        
%        keyboard
    end            
end

fprintf('\n-------------------------------------------\n\n');
fprintf('Demo has ended.\n')
fprintf('\n-------------------------------------------\n\n');


    
