# -*- coding: utf-8 -*-
"""
This script demonstrates how to use functions of the gait.py module-package
to load and show gait patterns (preprocessed stance-related data) 
that accompanies the dataset 
of gait measurements (raw data) described in the article

"A dataset of measurements of ground reaction forces of normal gait 
of young adults wearing various footwear"
by Marcin Derlatka and Marek Parfieniuk

submitted to the "Scientific Data" journal.

Created on May 9, 2021

@author: Marek Parfieniuk

Copyright (c) 2022 Marek Parfieniuk

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
"""


import gait
import matplotlib.pyplot as plt
import os
import re
import traceback


print('\n-------------------------------------------\n')
print('Welcome to a demo of gait patterns (preprocessed stance-related data) and Python routines that accompany')
print('the dataset of gait measurements (raw data) described in the article\n')
print('"A dataset of measurements of ground reaction forces of normal gait')
print('of young adults wearing various footwear"')
print('by Marcin Derlatka and Marek Parfieniuk\n')
print('submitted to the "Scientific Data" journal.')
print('\n-------------------------------------------\n')
print('This demo creates figures that present data.')
print('Figures should be reviewed AFTER the execution of the demo.')
print('They may be irreponsible when the script is working,')
print('even when it only waits for user input.')
print('')
print('This is related to some interaction-related limitations')
print('of the matplotlib library, and of Python itself')
print('A more interactive script could be created, but it would be less portable,')
print('being dependent on how Python is configured, requiring a particular GUI toolkit.')
print('')
print('For related information, please see:')
print('https://matplotlib.org/stable/users/interactive.html')
print('')
print('In order to review figures, please ensure that the Python interpreter')
print('enters interactive mode after executing the script.')
print('Running the demo from the command line, one should write')
print('   python -i runDemo.py')
print('Or one can run the demo under the IPython interactive shell.')

if (not ('fileDirectoryPath' in vars())) :
    fileDirectoryPath = os.getcwd()      # default = current directory 
    
print('\n-------------------------------------------\n')
print('Please enter the path to the data directory which contains')
print('the participantData.csv file and the gaitPatternsInCSVFiles subdirectory')
#      ''000''...''324'' subdirectories with gait patterns in PTRN.CSV files.')
print('Or, simply press the ENTER key in order to use the recent directory\n({:s}).'.format(fileDirectoryPath))
s = input('>>>')
if (s != '') :
    fileDirectoryPath = s

toReloadData = (not ('participantData' in vars()))
toSummarizeData = toReloadData

if (not toReloadData) :
    print('\n-------------------------------------------\n')
    print('In order to reload participant data, please enter the "y" letter.')
    print('Or, simply press the ENTER to not reload the data.')
    s = input('>>>')
    toReloadData = ((s == 'y') or (s == 'Y'))
    
if (toReloadData) :
    print('\n-------------------------------------------\n')
    print('Loading participant data ...')
    participantData = gait.loadParticipants(fileDirectoryPath + r'/participantData.csv')
    print('OK. Participant data loaded.')
    toSummarizeData = True;

if (not toSummarizeData) :
    print('\n-------------------------------------------\n')
    print('In order to summarize the participant data, please enter the "y" letter.')
    print('Or, simply press the ENTER to not summarize the data.')
    s = input('>>>')
    toSummarizeData = ((s == 'y') or (s == 'Y'))

if (toSummarizeData) :
    print('\n-------------------------------------------\n')
    gait.summarizeParticipants(participantData)

toReloadData = (not ('gaitPatternData' in vars()))

if (not toReloadData) :
    print('\n-------------------------------------------\n')
    print('In order to reload gait patterns from PTRN.CSV files, please enter the "y" letter.')
    print('Or, simply press the ENTER to not reload the data.')
    s = input('>>>')
    toReloadData = ((s == 'y') or (s == 'Y'))    
    
if (toReloadData) :    
    print('\n-------------------------------------------\n')    
    print('Loading gait patterns from CSV files. This may take a dozen or so minutes.')
    gaitPatternData = gait.loadPatterns(fileDirectoryPath + r'/gaitPatternsInCSVFiles',  r'.*', 1100);
    print('OK. Gait patterns loaded.')

if (not toSummarizeData) :
    print('\n-------------------------------------------\n')
    print('In order to summarize the pattern data, please enter the "y" letter.')
    print('Or, simply press the ENTER to not summarize the data.')
    s = input('>>>')
    toSummarizeData = ((s == 'y') or (s == 'Y'))

if (toSummarizeData) :
    print('\n-------------------------------------------\n')
    gait.summarizePatterns(gaitPatternData)

print('\n-------------------------------------------\n')
print('Now, You can review patterns one by one. Please enter a regular expression (regex)')
print('that determines a subset of patterns-files to be presented. For example:')
print('- 027.* : selects all patterns related to the 27th participant.')
print('- 027.*spr.* : selects patterns related to the 27th participant wearing sport shoes.')
print('- 027.*\.2\..* : selects patterns related to the 27th participant wearing the 2nd footwear.')
print('Or, please enter "q" in order to stop revewing patterns.')
while (True) :
    plt.pause(1)
    try :
        s = input(">>> ")
        if (s == 'q') :
            break;   
        if (re.fullmatch('\d+', s)) :
             s = s + '.*'
        gait.showPatterns(gaitPatternData, s)
        plt.pause(1)
    except :
        traceback.print_exc()

print('\n-------------------------------------------\n')
print('Demo has ended. Now, You can interact with the figures.')
print('\n-------------------------------------------\n')
