function showMeasurement(gaitMeasurements, measurementIndexOrFileName, ...
                    toSmoothStanceBoundaries, toBeVerbose, figureHandle)

% showMeasurement plots 
% - ground reaction forces (GRFs),
% - stance-related parts of the vertical GRF, possibly with smoothed tails
% - center of pressure (CoP),
% - vertical torque
% - vertical lines that indicate boundaries of the stance phase, 
%   foot-strike/off events, Initial and refined (black/green colors),
%   determined by using the findStance and cutStance functions, respectively
% of a preferred measurement of a given array of gait measurements.
%
% In addition to plots, useful information is presented:
% - measurement index and identifier (name of CSV file)
% - numbers of stance-related samples of both force plates
% - step duration computed as the difference between the indexes of the
%   samples that begin the foot strikes of both force plates
% - step duration computed as the difference between the indexes of the
%   samples that end the foot offs of both force plates
% - indexes of the boundary samples that end the foot offs of both force plates
% - indexes of the boundary samples that begin the foot strikes of both force plates
%
% The measurement array should be prepared by using the loadMeasurements function.
%
% The measurement can be selected by providing its index in the array
% or its identifier (name of CSV file, including extension).
%
% The function can use an existing figure window, 
% which will be cleared before creating new plots.
% If no figure handle is provided, then the function creates a new window.
%
% Input:
% - gaitMeasurements : array of structures each of which describes one gait measurement
% - measurementIndexOrFileName : index or string that identifies the
%   measurement-file
% - toSmoothStanceBoundaries : 0 or 1; disables/enables extrapolation-based  
%           refinement of boundaries of the stance-related parts
%           of the vertical GRF signals; please see the cutStance function			
% - toBeVerbose : 0 or 1; disables/enables displaying diagnostic messages, 
%                 which can be used for debugging.
% - figureHandle : (optional) the handle of a figure in which plots 
%		   should be created.
%
% Output:
% - A window is created or redrawn.
%	
% Created on Sep. 28, 2021
%     
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>


if (ischar(measurementIndexOrFileName))
    measurementFileName = measurementIndexOrFileName;
    if (~endsWith(measurementFileName, '.csv'))
        measurementFileName = [measurementFileName '.csv'];
    end    
    measurementIndex = 0;
    for k = 1 : length(gaitMeasurements)
        if strcmpi(gaitMeasurements{k}.fileName, measurementFileName)
            measurementIndex = k;
            break;
        end
    end
else 
    measurementIndex = measurementIndexOrFileName;
    measurementFileName = gaitMeasurements{measurementIndex}.fileName;
end

measurementRecord = gaitMeasurements{measurementIndex};

[firstStrikeInitialIndex, firstOffInitialIndex, firstStrikeInitialValue, ...
    firstOffInitialValue, firstMaxIndex, firstMaxValue] ...
    = findStance(measurementRecord.firstVForce, 0); %toBeVerbose);

[firstVStanceSamples, firstStrikeIndex, firstOffIndex] ...
    = cutStance(measurementRecord.firstVForce, firstStrikeInitialIndex ...
        , firstOffInitialIndex, toSmoothStanceBoundaries, toBeVerbose);

[secondStrikeInitialIndex, secondOffInitialIndex, secondStrikeInitialValue, ...
    secondOffInitialValue, secondMaxIndex, secondMaxValue] ...
    = findStance(measurementRecord.secondVForce, 0); %, toBeVerbose);

[secondVStanceSamples, secondStrikeIndex, secondOffIndex] ...
    = cutStance(measurementRecord.secondVForce, secondStrikeInitialIndex ...
        , secondOffInitialIndex, toSmoothStanceBoundaries, toBeVerbose);

firstMLStanceSamples = measurementRecord.firstMLForce(firstStrikeIndex : firstOffIndex);
secondMLStanceSamples = measurementRecord.secondMLForce(secondStrikeIndex : secondOffIndex);

[isLeftLegFirst, legOrderRuleIndex] = checkLeftLegFirst(firstMLStanceSamples, secondMLStanceSamples, 0);

fprintf(1, 'IsLeftLegFirst %d based on rule %d\n', isLeftLegFirst, legOrderRuleIndex);

if (isLeftLegFirst)
    firstLabel = 'First plate, left leg';
    secondLabel = 'Second plate, right leg';
    legOrder = 'Leg order: left leg on first plate';
else    
    firstLabel = 'First plate, right leg';
    secondLabel = 'Second plate, left leg';
    legOrder = 'Leg order: right leg on first plate';    
end

if (nargin < 5)                % No figure handle,
    figureHandle = figure;      % so let us create one.
    set(figureHandle, 'Units', 'normalized', 'Position', [0 0 1 1])
end

figure(figureHandle)
clf

figureName = sprintf('Measurement #%d : %s', measurementIndex, measurementFileName);
set(figureHandle,'name', figureName, 'numbertitle', 'off')
%set(figureHandle,'Position',[100 100 1000 500]);
set(figureHandle, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);  
%title(strrep(fileName, '\', '\\'))

subplot(3, 2, 2)
box on
grid on
hold on
text(0.5, 0.5, { ...
        sprintf('Measurement #%d : %s', measurementIndex, measurementFileName), ...
        '', ...
        legOrder, ...
        '', ...
        sprintf('First-plate stance phase duration: %d (%d) samples', firstOffIndex - firstStrikeIndex + 1, ...
                firstOffInitialIndex - firstStrikeInitialIndex + 1), ...
        sprintf('Second-plate stance phase duration:  %d (%d) samples', secondOffIndex - secondStrikeIndex + 1, ...
                secondOffInitialIndex - secondStrikeInitialIndex + 1), ...
        sprintf('Foot-strike-based step duration: %d (%d) samples', secondStrikeIndex - firstStrikeIndex + 1, ...
                secondStrikeInitialIndex - firstStrikeInitialIndex + 1), ...
        sprintf('Foot-off-based step duration:    %d (%d) samples', secondOffIndex - firstOffIndex + 1, ...
                secondOffInitialIndex - firstOffInitialIndex + 1), ...
        '', ...
        sprintf('First-plate foot strike/off at: %d / %d (%d / %d) sample', ...
                firstStrikeIndex, firstOffIndex, firstStrikeInitialIndex, firstOffInitialIndex), ...
        sprintf('Second-plate foot strike/off at: %d / %d (%d / %d) sample', ...
                secondStrikeIndex, secondOffIndex, secondStrikeInitialIndex, secondOffInitialIndex), ...
        '    Estimates of strike/off times shown as vertical lines:', ...
        '       refined = green (initial = black)'
    }, 'HorizontalAlignment', 'center', 'FontWeight', 'bold'); %, 'fontsize', 12);
axis off

subplot(3, 2, 1)
box on
grid on
hold on
% title(sprintf('second-leg stance phase duration: %d samples\nfirst-leg stance phase duration: %d samples\nFoot-strike-based step duration: %d samples\nFoot-off-based step duration: %d samples', ...
%     secondOffIndex - secondStrikeIndex, firstOffIndex - firstStrikeIndex, firstStrikeIndex - secondStrikeIndex, firstOffIndex - secondOffIndex), 'HorizontalAlignment', 'first');

plot(firstStrikeIndex : firstOffIndex, firstVStanceSamples, 'Color','g');
fp = plot(measurementRecord.firstVForce, 'Color','b');
line([firstStrikeInitialIndex firstStrikeInitialIndex], [-100 600], 'Color','k', 'LineStyle','--')
line([firstOffInitialIndex firstOffInitialIndex], [-100 600], 'Color','k', 'LineStyle','--')
line([firstStrikeIndex firstStrikeIndex], [-100 600], 'Color','g', 'LineStyle','--')
line([firstOffIndex firstOffIndex], [-100 600], 'Color','g', 'LineStyle','--')

plot(secondStrikeIndex : secondOffIndex, secondVStanceSamples, 'Color','g');
sp = plot(measurementRecord.secondVForce, 'Color','r');
line([secondStrikeInitialIndex secondStrikeInitialIndex], [-100 600], 'Color','k', 'LineStyle','--')
line([secondOffInitialIndex secondOffInitialIndex], [-100 600], 'Color','k', 'LineStyle','--')
line([secondStrikeIndex secondStrikeIndex], [-100 600], 'Color','g', 'LineStyle','--')
line([secondOffIndex secondOffIndex], [-100 600], 'Color','g', 'LineStyle','--')

ylabel('Vertical GRF [N]')
xlabel('Sample index')
legend([fp, sp], {firstLabel, secondLabel})
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 2, 3)
box on
grid on
hold on
fp = plot(measurementRecord.firstAPForce, 'Color', 'b');
line([firstStrikeInitialIndex firstStrikeInitialIndex], [-150 150], 'Color','k', 'LineStyle','--')
line([firstOffInitialIndex firstOffInitialIndex], [-150 150], 'Color','k', 'LineStyle','--')
line([firstStrikeIndex firstStrikeIndex], [-150 150], 'Color','g', 'LineStyle','--')
line([firstOffIndex firstOffIndex], [-150 150], 'Color','g', 'LineStyle','--')

sp = plot(measurementRecord.secondAPForce, 'Color', 'r');
line([secondStrikeInitialIndex secondStrikeInitialIndex], [-150 150], 'Color','k', 'LineStyle','--')
line([secondOffInitialIndex secondOffInitialIndex], [-150 150], 'Color','k', 'LineStyle','--')
line([secondStrikeIndex secondStrikeIndex], [-150 150], 'Color','g', 'LineStyle','--')
line([secondOffIndex secondOffIndex], [-150 150], 'Color','g', 'LineStyle','--')

ylabel('Anterior-posterior GRF [N]')
xlabel('Sample index')
legend([fp, sp], {firstLabel, secondLabel})
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 2, 5)
box on
grid on
hold on
fp = plot(measurementRecord.firstMLForce, 'Color','b');
line([firstStrikeInitialIndex firstStrikeInitialIndex], [-50 50], 'Color','k', 'LineStyle','--');
line([firstOffInitialIndex firstOffInitialIndex], [-50 50], 'Color','k', 'LineStyle','--');
line([firstStrikeIndex firstStrikeIndex], [-50 50], 'Color','g', 'LineStyle','--');
line([firstOffIndex firstOffIndex], [-50 50], 'Color','g', 'LineStyle','--');
sp = plot(measurementRecord.secondMLForce, 'Color','r');
line([secondStrikeInitialIndex secondStrikeInitialIndex], [-50 50], 'Color','k', 'LineStyle','--');
line([secondOffInitialIndex secondOffInitialIndex], [-50 50], 'Color','k', 'LineStyle','--');
line([secondStrikeIndex secondStrikeIndex], [-50 50], 'Color','g', 'LineStyle','--');
line([secondOffIndex secondOffIndex], [-50 50], 'Color','g', 'LineStyle','--');

ylabel('Medio-lateral GRF [N]')
xlabel('Sample index')
t = axis; t(3) = -100; t(4) = 100; axis(t);
legend([fp, sp], {firstLabel, secondLabel})
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 2, 6)
box on
grid on
hold on

% We neglect samples before and after (up to some margin) the stance phase, as they usually are noisy 

noiseIndexMargin = 20;
firstAPCoP = measurementRecord.firstAPCoP((firstStrikeIndex + noiseIndexMargin) : (firstOffIndex - noiseIndexMargin));
firstMLCoP = measurementRecord.firstMLCoP((firstStrikeIndex + noiseIndexMargin) : (firstOffIndex - noiseIndexMargin));
secondAPCoP = measurementRecord.secondAPCoP((secondStrikeIndex + noiseIndexMargin) : (secondOffIndex - noiseIndexMargin));
secondMLCoP = measurementRecord.secondMLCoP((secondStrikeIndex + noiseIndexMargin) : (secondOffIndex - noiseIndexMargin));

fp = plot(firstAPCoP * 100, firstMLCoP * 100, 'Color', 'b');
sp = plot(secondAPCoP * 100, secondMLCoP * 100, 'Color', 'r');

xlabel('Anterior-posterior coordinate of CoP [cm]')
ylabel('Medio-lateral coordinate of CoP [cm]')
legend([fp, sp], {firstLabel, secondLabel})

noiseIndexMargin = 3;

firstVTorque = measurementRecord.firstVTorque;
firstVTorque(1 : (firstStrikeIndex + noiseIndexMargin)) = 0;
firstVTorque((firstOffIndex - noiseIndexMargin) : end) = 0;

secondVTorque = measurementRecord.secondVTorque;
secondVTorque(1 : (secondStrikeIndex + noiseIndexMargin)) = 0;
secondVTorque((secondOffIndex - noiseIndexMargin) : end) = 0;


subplot(3, 2, 4)
box on
grid on
hold on
fp = plot(firstVTorque, 'Color', 'b');
sp = plot(secondVTorque, 'Color','r');
%line([firstStrikeInitialIndex firstStrikeInitialIndex], [-10 10], 'Color','k', 'LineStyle','--')
%line([firstOffInitialIndex firstOffInitialIndex], [-10 10], 'Color','k', 'LineStyle','--')
line([firstStrikeIndex firstStrikeIndex], [-10 10], 'Color','g', 'LineStyle','--')
line([firstOffIndex firstOffIndex], [-10 10], 'Color','g', 'LineStyle','--')
%line([secondStrikeInitialIndex secondStrikeInitialIndex], [-10 10], 'Color','k', 'LineStyle','--')
%line([secondOffInitialIndex secondOffInitialIndex], [-10 10], 'Color','k', 'LineStyle','--')
line([secondStrikeIndex secondStrikeIndex], [-10 10], 'Color','g', 'LineStyle','--')
line([secondOffIndex secondOffIndex], [-10 10], 'Color','g', 'LineStyle','--')

t = axis; t(3) = -20; t(4) = 20; axis(t);
ylabel('Vertical torque [N \cdot m]')
xlabel('Sample index')
legend([fp, sp], {firstLabel, secondLabel})


fprintf('Shown measurement #%d : %s\n', measurementIndex, measurementFileName)