function showPatterns(patternData, fileNamePattern)

% showPatterns plots stance-related parts of gait measurements 
% whose file names match a given pattern (regular expression).
% In particular, measurements related to a subject and shoe type can be selected.
% 
% This function can be used to compare waveforms 
% in order to identify trends and outliers.
% 
% Input:
% - gaitPatterns : sstance-related data prepared by using the extractStanceData
%              function
% - fileNamePattern : pattern (regular expression) for file names
%         
% Output:
% - Figure is displayed.
%         
% Created on May 08, 2022
% 
% Author: Marek Parfieniuk
%
% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

selectedPatterns = {};

%keyboard

for patternIndex = 1 : length(patternData.fileNames)    
    
    if (isempty(regexpi(patternData.fileNames{patternIndex}, fileNamePattern)))
        % fprintf('Skipped files whose names do not match the pattern')
        continue;    
    end
    
    t.fileName = patternData.fileNames{patternIndex}; 
    t.leftVForce = patternData.leftVForces(patternIndex, :); 
    t.rightVForce = patternData.rightVForces(patternIndex, :); 
    t.leftAPForce = patternData.leftAPForces(patternIndex, :); 
    t.rightAPForce = patternData.rightAPForces(patternIndex, :); 
    t.leftMLForce = patternData.leftMLForces(patternIndex, :); 
    t.rightMLForce = patternData.rightMLForces(patternIndex, :); 

    
    fprintf('Found measurement %5d : %s\n', patternIndex, t.fileName);    
    
    selectedPatterns{end + 1} = t;
    
    if (length(selectedPatterns) > 150)
        fprintf('More than 150 patterns found. It seems pointless to show so many plots. Please change the pattern for file names.\n')
        return
    end
    
end

%if (min(size(selectedPatterns)) == 0)
if (length(selectedPatterns) == 0)
    fprintf('No measurement found. Please try another file name pattern.\n')
    return
end

%keyboard

figure
t = ['Time-normalized stance-related fragments of measurements of files matching ' fileNamePattern];
set(gcf, 'Name', t, 'NumberTitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]); 

subplot(3, 3, 1)
hold on
box on
grid on
for k = 1 : length(selectedPatterns)
   plot(selectedPatterns{k}.leftVForce, 'DisplayName', selectedPatterns{k}.fileName) 
end
ylabel('Left-leg vertical GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');
title('Left leg')

subplot(3, 3, 2)
hold on
box on
grid on
for k = 1 : length(selectedPatterns)
   plot(selectedPatterns{k}.rightVForce, 'DisplayName', selectedPatterns{k}.fileName) 
end
legend(gca, 'Position', [0.7 0.1 0.2 0.8])
ylabel('Right-leg vertical GRF [N]')
xlabel('Sample index')
h = zoom;
set(h,'Motion','both','Enable','on');
title('Right leg')

subplot(3, 3, 4)
hold on
box on
grid on
for k = 1 : length(selectedPatterns)
   plot(selectedPatterns{k}.leftAPForce, 'DisplayName', selectedPatterns{k}.fileName) 
end
ylabel('Left-leg anterior-posterior GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 3, 5)
hold on
box on
grid on
for k = 1 : length(selectedPatterns)
   plot(selectedPatterns{k}.rightAPForce, 'DisplayName', selectedPatterns{k}.fileName) 
end
ylabel('Right-leg anterior-posterior GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 3, 7)
hold on
box on
grid on
for k = 1 : length(selectedPatterns)
   plot(selectedPatterns{k}.leftMLForce, 'DisplayName', selectedPatterns{k}.fileName) 
end
ylabel('Left-leg medio-lateral GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 3, 8)
hold on
box on
grid on
for k = 1 : length(selectedPatterns)
   plot(selectedPatterns{k}.rightMLForce, 'DisplayName', selectedPatterns{k}.fileName) 
end
ylabel('Right-leg medio-lateral GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

end % function