function showStances(stanceData, fileNamePattern)

% showStances shows stance-related samples of GRF measurements.
% Samples are provided via a structure of arrays that should be created 
% by using the extractStancesFromMeasurements function.
%
% showStances shows only samples related to CSV files whose names
% match a given pattern (regular expression).
% So, the function can be used to show samples related 
% to a particular subject and shoe type.
% 
% The showStances function can be used to compare GRF waveforms 
% in order to assess data integrity and to identify trends and outliers.
% 
% Input:
% - stanceData : a data structure created by using 
%                   the extractStancesFromMeasurements function
% - fileNamePattern : pattern (regular expression) for file names
%         
% Output:
% - Figure is displayed.
%         
% Created on May 06, 2022
% 
% Author: Marek Parfieniuk

% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

selectedRecords = {};

%keyboard

for measurementIndex = 1 : length(stanceData.fileNames)
    if (isempty(regexpi(stanceData.fileNames{measurementIndex}, fileNamePattern)))
        % fprintf('Skipped files whose names do not match the pattern')
        continue;    
    end
        
    t.firstVForce = stanceData.firstVForces{measurementIndex}; 
    t.secondVForce = stanceData.secondVForces{measurementIndex}; 
    t.firstAPForce = stanceData.firstAPForces{measurementIndex}; 
    t.secondAPForce = stanceData.secondAPForces{measurementIndex}; 
    t.firstMLForce = stanceData.firstMLForces{measurementIndex}; 
    t.secondMLForce = stanceData.secondMLForces{measurementIndex}; 

    t.fileName = stanceData.fileNames{measurementIndex}; 
    selectedRecords{end + 1} = t;
    
    if (length(selectedRecords) > 150)
        fprintf('More than 150 measurements found. It seems pointless to show so many plots. Please change the pattern for file names.\n')
        return
    end    
    
    fprintf('Found measurement %5d : %s\n', measurementIndex, t.fileName);
end

if (min(size(selectedRecords)) == 0)
    fprintf('No measurement found. Please try another file name pattern.\n')
    return
end

%keyboard

figure
t = ['Stance-related parts of GRF measurements of files whose names match ' fileNamePattern];
set(gcf, 'Name', t, 'NumberTitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]); 

subplot(3, 3, 1)
hold on
box on
grid on
for k = 1 : length(selectedRecords)
   plot(selectedRecords{k}.firstVForce, 'DisplayName', selectedRecords{k}.fileName) 
end
ylabel('First-plate vertical GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');
title('First force plate')

subplot(3, 3, 2)
hold on
box on
grid on
for k = 1 : length(selectedRecords)
   plot(selectedRecords{k}.secondVForce, 'DisplayName', selectedRecords{k}.fileName) 
end
legend(gca, 'Position', [0.7 0.1 0.2 0.8])
ylabel('Second-plate vertical GRF [N]')
xlabel('Sample index')
h = zoom;
set(h,'Motion','both','Enable','on');
title('Second force plate')

subplot(3, 3, 4)
hold on
box on
grid on
for k = 1 : length(selectedRecords)
   plot(selectedRecords{k}.firstAPForce, 'DisplayName', selectedRecords{k}.fileName) 
end
ylabel('First-plate anterior-posterior GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 3, 5)
hold on
box on
grid on
for k = 1 : length(selectedRecords)
   plot(selectedRecords{k}.secondAPForce, 'DisplayName', selectedRecords{k}.fileName) 
end
ylabel('Second-plate anterior-posterior GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 3, 7)
hold on
box on
grid on
for k = 1 : length(selectedRecords)
   plot(selectedRecords{k}.firstMLForce, 'DisplayName', selectedRecords{k}.fileName) 
end
ylabel('First-plate medio-lateral GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

subplot(3, 3, 8)
hold on
box on
grid on
for k = 1 : length(selectedRecords)
   plot(selectedRecords{k}.secondMLForce, 'DisplayName', selectedRecords{k}.fileName) 
end
ylabel('Second-plate medio-lateral GRF [N]')
xlabel('Sample index')
%legend(gca, 'Location', 'best')
h = zoom;
set(h,'Motion','both','Enable','on');

end % function