function summarizeParticipants(participantData)

% summarizeParticipants shows histograms and statistics that characterize 
% distributions of ages, weights, and heights of the persons who served as
% participants of gait measurements.
% 
% Additionally, this function creates two figures which show
% - histograms of ages, weights, heights
% - 3D histograms of weight-height combinations; rotated so as to look like
%   2D scatter plots
% of all participants, of only females, and of only males.
%
% This function can be used to compare females against males, 
% and to identify participants that could be outliers, 
% especially overweight persons.
% 
% Input:
% - participantData : structure of participant data, prepared 
%                 by using the loadParticipants function.
%         
% Output:
% - Text and figures are displayed.
%         
% Created on Sep. 28, 2021
% 
% Author: Marek Parfieniuk

% Copyright (c) 2021 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>

fprintf('Number of males: %d.\n', sum(participantData.genders == 'm'));
fprintf('Number of females: %d.\n', sum(participantData.genders == 'f'));
fprintf('\n');
fprintf('Average female''s weigth: %5.2f kg.\n' ...
            , mean(participantData.weights(participantData.genders == 'f')));
fprintf('Average male''s weigth: %5.2f kg.\n' ...
            , mean(participantData.weights(participantData.genders == 'm')));
fprintf('\n');
fprintf('Average female''s heigth: %5.2f cm.\n' ...
            , mean(participantData.heights(participantData.genders == 'f')));
fprintf('Average male''s heigth: %5.2f cm.\n' ...
            , mean(participantData.heights(participantData.genders == 'm')));
fprintf('\n');
fprintf('Average participant''s age: %5.2f years.\n', mean(participantData.ages));

% 2D histograms

figure
set(gcf, 'Name', 'Participant-related 2D histograms', 'NumberTitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

binEdges = {40 : 5 : 160; 150 : 5 : 210};

subplot(1, 3, 1)
t = [participantData.weights, participantData.heights];
hist3(t, 'CdataMode','auto', 'Edges', binEdges)
title('Number of participants (both genders)')
ylabel('Height [cm]')
xlabel('Weight [kg]')
colorbar
colormap pink
view(2)
box on;
grid on;

subplot(1, 3, 2)
t = [participantData.weights(participantData.genders == 'f') ...
        , participantData.heights(participantData.genders == 'f')];
hist3(t,'CdataMode','auto', 'Edges', binEdges)
title('Number of females')
ylabel('Height [cm]')
xlabel('Weight [kg]')
colorbar
colormap pink
view(2)
box on;
grid on;

subplot(1, 3, 3)
t = [participantData.weights(participantData.genders == 'm'), ...
        participantData.heights(participantData.genders == 'm')];
hist3(t,'CdataMode','auto', 'Edges', binEdges)
title('Number of males')
ylabel('Height [cm]')
xlabel('Weight [kg]')
colorbar
colormap pink
view(2)
box on;
grid on;

% 1D histograms

figure
set(gcf, 'name', 'Participant-related 1D histograms', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 3, 1)
t = participantData.ages;
histogram(t)
xlabel({'Age [years]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of participants (both genders)')
grid on
box on

subplot(3, 3, 2)
t = participantData.ages(participantData.genders == 'f');
histogram(t)
xlabel({'Age [years]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of females')
grid on
box on

subplot(3, 3, 3)
t = participantData.ages(participantData.genders == 'm');
histogram(t)
xlabel({'Age [years]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of males')
grid on
box on

% weight 

subplot(3, 3, 4)
t = participantData.weights;
histogram(t)
xlabel({'Weight [kg]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of participants (both genders)')
grid on
box on

subplot(3, 3, 5)
t = participantData.weights(participantData.genders == 'f');
histogram(t)
xlabel({'Weight [kg]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of females')
grid on
box on

subplot(3, 3, 6)
t = participantData.weights(participantData.genders == 'm');
histogram(t)
xlabel({'Weight [kg]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of males')
grid on
box on

% height

subplot(3, 3, 7)
t = participantData.heights;
histogram(t)
xlabel({'Height [cm]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of participants (both genders)')
grid on
box on

subplot(3, 3, 8)
t = participantData.heights(participantData.genders == 'f');
histogram(t)
xlabel({'Height [cm]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of females')
grid on
box on

subplot(3, 3, 9)
t = participantData.heights(participantData.genders == 'm');
histogram(t)
xlabel({'Height [cm]', ...
    sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
ylabel('Number of males')
grid on
box on


end % function

