function summarizePatterns(patternData)

% summarizePatterns shows histograms, maxima, minima 
% that characterize leg-assigned time-normalized stance-related fragments
% of GRF signals.
%
% Additionally, lengths of stance-related fragments average relations between 
% measurement length and durations of step and stance phase are quantified
% by mean values and standard deviation.
%
% summarizePatterns also shows histograms of mean magnitudes 
% of samples that precede/follow the foot-strike/off events.
% These plots show in which GRF signals and how often artifacts occur
% related to faults in the initial construction of the walkway.
%
% This function can be used to validate patterns and raw measurements.
% One can look for outstanding patterns 
% and then use them to identify and check measurements that might be outliers.
%
% Input:
% - patternData : stance-related data created by using 
%                   the extractStancesFromMeasurements function.
%
% Output:
% - Figures and texts are displayed.
%
% Created on May 25, 2022
% 
% Author: Marek Parfieniuk

% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>


numberOfBins = 30;


figure
set(gcf, 'name', 'Stance-phase histograms', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(1, 2, 1);
t = patternData.leftOffIndexes - patternData.leftStrikeIndexes;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Left-leg stance-phase duration [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([400 1100 0 1700])

subplot(1, 2, 2);
t = patternData.rightOffIndexes - patternData.rightStrikeIndexes;
histogram(t, numberOfBins)
grid on
box on
ylabel('Number of measurements')
xlabel({'Right-leg stance-phase duration [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([400 1100 0 1700])



figure
set(gcf, 'name', 'Histograms of extremums of vertical and anterior-posterior forces', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 2, 1)
t = max(patternData.leftVForces, [], 2);
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Left-leg vertical force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0 1700 0 1500])

subplot(3, 2, 2)
t = max(patternData.rightVForces, [], 2);
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Right-leg vertical force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0 1700 0 1500])

subplot(3, 2, 3)
t = max(patternData.leftAPForces, [], 2);
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Left-leg anterior-posterior force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0 400 0 1800])

subplot(3, 2, 4)
t = max(patternData.rightAPForces, [], 2);
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Right-leg anterior-posterior force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0 400 0 1800])

subplot(3, 2, 5)
t = min(patternData.leftAPForces, [], 2);
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Left-leg anterior-posterior force: minimum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-350 0 0 1800])

subplot(3, 2, 6)
t = min(patternData.rightAPForces, [], 2);
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Right-leg anterior-posterior force: minimum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-350 0 0 1800])





figure
set(gcf, 'name', 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events of left leg', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 2, 1);
t = patternData.leftVStrikeMeans;
histogram(t, numberOfBins * 3)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of vertical force of left leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 2);
t = patternData.leftVOffMeans;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of vertical force of left leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 3);
t = patternData.leftAPStrikeMeans;
histogram(t, numberOfBins * 4)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of anterior-posterior force of left leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 4);
t = patternData.leftAPOffMeans;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of anterior-posterior force of left leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 5);
t = patternData.leftMLStrikeMeans;
histogram(t, numberOfBins * 2)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of medio-lateral force of left leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 6);
t = patternData.leftMLOffMeans;
histogram(t, numberOfBins * 3)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of medio-lateral force of left leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])



figure
set(gcf, 'name', 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events on right leg', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 2, 1);
t = patternData.rightVStrikeMeans;
histogram(t, numberOfBins * 2)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike stamples', 'of vertical force of right leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 2);
t = patternData.rightVOffMeans;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of vertical force of right leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 3);
t = patternData.rightAPStrikeMeans;
histogram(t, numberOfBins * 3)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of anterior-posterior force of right leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 4);
t = patternData.rightAPOffMeans;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of anterior-posterior force of right leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 5);
t = patternData.rightMLStrikeMeans;
histogram(t, numberOfBins * 3)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of medio-lateral force of right leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

subplot(3, 2, 6);
t = patternData.rightMLOffMeans;
histogram(t, numberOfBins * 3)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of medio-lateral force of right leg [N]' ...
    , sprintf('Max: %5.2f', max(t))})
axis([0 30 0 8000])

return





    