function summarizeStances(stanceData)

% summarizeStances shows histograms, maxima, minima, 
% that characterize distributions of values and indexes of boundary 
% (maximum, foot-strike-related, foot-off-related) samples
% of measurement parts related to the stance phase of gait.
%
% Additionally, average relations between measurement length 
% and durations of step and stance phase are quantified
% by mean values and standard deviation.
%
% summarizeStances also shows histograms of mean magnitudes 
% of samples that precede/follow the foot-strike/off events.
% These plots show in which GRF signals and how often Means occur
% related to faults in the initial construction of the walkway.
%
% This function can be used to validate patterns, measurements, 
% and algorithms used to detect boundaries of the stance phase.
% One can look for outstanding boundary samples, 
% and then use them to identify and check measurements that might be outliers.
%
% Input:
% - stanceData : stance-related data created by using 
%                   the extractStanceData function.
%
% Output:
% - Figures and texts are displayed.
%
% Created on May 25, 2022
% 
% Author: Marek Parfieniuk

% Copyright (c) 2022 Marek Parfieniuk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>


fileNames = stanceData.fileNames;
firstVForces = stanceData.firstVForces;
secondVForces = stanceData.secondVForces;

measurementLengths = stanceData.measurementLengths;

firstStrikeIndexes = stanceData.firstStrikeIndexes;
firstOffIndexes = stanceData.firstOffIndexes;
firstStrikeInitialValues = stanceData.firstStrikeInitialValues;
firstOffInitialValues = stanceData.firstOffInitialValues;


secondStrikeIndexes = stanceData.secondStrikeIndexes;
secondOffIndexes = stanceData.secondOffIndexes;
secondStrikeInitialValues = stanceData.secondStrikeInitialValues;
secondOffInitialValues = stanceData.secondOffInitialValues;

measurementCount = length(stanceData.firstVForces);

firstVMaximums = zeros(measurementCount, 1);
firstAPMaximums = zeros(measurementCount, 1);
firstAPMinimums = zeros(measurementCount, 1);
secondVMaximums = zeros(measurementCount, 1);
secondAPMaximums = zeros(measurementCount, 1);
secondAPMinimums = zeros(measurementCount, 1);

for k = 1 : measurementCount,
   firstVMaximums(k) = max(stanceData.firstVForces{k}); 
   firstAPMaximums(k) = max(stanceData.firstAPForces{k});
   firstAPMinimums(k) = min(stanceData.firstAPForces{k});
   secondVMaximums(k) = max(stanceData.secondVForces{k}); 
   secondAPMaximums(k) = max(stanceData.secondAPForces{k});
   secondAPMinimums(k) = min(stanceData.secondAPForces{k});
end


t = measurementLengths ./ (firstOffIndexes - firstStrikeIndexes);
fprintf(1, 'Ratio of measurement duration to stance phase duration (first plate)\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))
t = measurementLengths ./ (secondOffIndexes - secondStrikeIndexes);
fprintf(1, 'Ratio of measurement duration to stance phase duration (second plate)\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))
t = measurementLengths ./ (secondStrikeIndexes - firstStrikeIndexes);
fprintf(1, 'Ratio of measurement duration to foot-strike-based step duration (both plates)\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))
t = measurementLengths ./ (secondOffIndexes - firstOffIndexes);
fprintf(1, 'Ratio of measurement duration to foot-off-based step duration (both plates)\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))

t = measurementLengths / 960;
fprintf(1, 'Measurement duration [seconds]\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))

t = (secondStrikeIndexes - firstStrikeIndexes) / 960;
fprintf(1, 'Foot-strike-based step duration [seconds]\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))

t = (secondOffIndexes - firstOffIndexes) / 960;
fprintf(1, 'Foot-off-based step duration [seconds]\n');
fprintf(1, ' - mean: %5.2f\n', mean(t))
fprintf(1, ' - standard deviation: %5.2f\n', std(t))


numberOfBins = 30;

figure
set(gcf, 'name', 'Step duration histograms', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(2, 2, 1);
t = secondStrikeIndexes - firstStrikeIndexes;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Foot-strike-based step duration [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([350 750 0 1700])

subplot(2, 2, 2);
t = secondOffIndexes - firstOffIndexes;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Foot-off-based step duration [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([350 750 0 1700])

subplot(2, 2, 3);
t = (secondStrikeIndexes - firstStrikeIndexes) / 960; % 960 Hz is sampling frequency
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Foot-strike-based step duration [seconds]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0.4 0.8 0 1700])

subplot(2, 2, 4);
t = (secondOffIndexes - firstOffIndexes) / 960; % 960 Hz is sampling frequency
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Foot-off-based step duration [seconds]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0.4 0.8 0 1700])




%keyboard

figure
set(gcf, 'name', 'Stance-phase-related general histograms', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(2, 2, 1);
t = measurementLengths;
histogram(t, numberOfBins)
text()
grid on
ylabel('Number of measurements')
xlabel({'Measurement duration [samples]', '(The same for both plates) ' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([1000 7000 0 3000])

subplot(2, 2, 2);
t = (firstOffIndexes - firstStrikeIndexes) - (secondOffIndexes - secondStrikeIndexes);
histogram(t, numberOfBins)
grid on
ylabel('Number of measurements')
xlabel({'Difference between durations of stance phases of both plates [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([-150 200 0 3500])

subplot(2, 2, 3);
t = measurementLengths ./ (firstOffIndexes - firstStrikeIndexes);
histogram(t, numberOfBins)
grid on
ylabel('Number of measurements')
xlabel({'Ratio of measurement duration to stance-phase duration', '(First plate)' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([1 10 0 2500])

subplot(2, 2, 4);
t = measurementLengths ./ (secondOffIndexes - secondStrikeIndexes);
histogram(t, numberOfBins)
grid on
ylabel('Number of measurements')
xlabel({'Ratio of measurement duration to stance-phase duration', '(Second plate)' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([1 10 0 2500])

figure
set(gcf, 'name', 'Stance-phase duration histograms', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(2, 2, 1);
t = firstOffIndexes - firstStrikeIndexes;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate stance-phase duration [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([400 1100 0 1800])

subplot(2, 2, 2);
t = secondOffIndexes - secondStrikeIndexes;
histogram(t, numberOfBins)
grid on
box on
ylabel('Number of measurements')
xlabel({'Second-plate stance-phase duration [samples]' ...
    , sprintf('Min: %5d, max: %5d', min(t), max(t))})
axis([400 1100 0 1800])

subplot(2, 2, 3);
t = (firstOffIndexes - firstStrikeIndexes) / 960;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate stance-phase duration [seconds]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0.5 1.1 0 1800])

subplot(2, 2, 4);
t = (secondOffIndexes - secondStrikeIndexes) / 960;
histogram(t, numberOfBins)
grid on
box on
ylabel('Number of measurements')
xlabel({'Second-plate stance-phase duration [seconds]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([0.5 1.1 0 1800])



figure
set(gcf, 'name', 'Histograms of vertical force at stance-phase boundaries', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(2, 2, 1);
t = firstStrikeInitialValues;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate vertical force: foot-strike value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-40 100 0 4000])

subplot(2, 2, 2);
t = secondStrikeInitialValues;
histogram(t, ceil(numberOfBins / 3))
box on
grid on
ylabel('Number of measurements')
xlabel({'Second-plate vertical force: foot-strike value  [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-40 100 0 4000])

subplot(2, 2, 3);
t = firstOffInitialValues;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate vertical force: foot-off value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-20 40 0 5000])

subplot(2, 2, 4);
t = secondOffInitialValues;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Second-plate vertical force: foot-off value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-20 40 0 5000])



figure
set(gcf, 'name', 'Histograms of extremums of vertical and anterior-posterior forces', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 2, 1)
t = firstVMaximums;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate vertical force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([400 1700 0 1500])

subplot(3, 2, 2)
t = secondVMaximums;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Second-plate vertical force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([400 1700 0 1500])

subplot(3, 2, 3)
t = firstAPMaximums;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate anterior-posterior force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([40 400 0 1600])

subplot(3, 2, 4)
t = secondAPMaximums;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Second-plate anterior-posterior force: maximum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([40 400 0 1600])

subplot(3, 2, 5)
t = firstAPMinimums;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'First-plate anterior-posterior force: minimum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-350 0 0 1800])

subplot(3, 2, 6)
t = secondAPMinimums;
histogram(t, numberOfBins)
box on
grid on
ylabel('Number of measurements')
xlabel({'Second-plate anterior-posterior force: minimum value [N]' ...
    , sprintf('Min: %5.2f, max: %5.2f', min(t), max(t))})
axis([-350 0 0 1800])

%keyboard


figure
set(gcf, 'name', 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events on first plate', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 2, 1);
t = stanceData.firstVStrikeMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of vertical force of first plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 2);
t = stanceData.firstVOffMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of vertical force of first plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 3);
t = stanceData.firstAPStrikeMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of anterior-posterior force of first plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 4);
t = stanceData.firstAPOffMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of anterior-posterior force of first plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 5);
t = stanceData.firstMLStrikeMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of medio-lateral force of first plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 6);
t = stanceData.firstMLOffMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of medio-lateral force of first plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])



figure
set(gcf, 'name', 'Histograms of mean magnitudes of samples that precede/follow the foot-strike/off events on second plate', 'numbertitle', 'off')
set(gcf, 'Units', 'normalized', 'Position', [0.1, 0.1, 0.8, 0.8]);

subplot(3, 2, 1);
t = stanceData.secondVStrikeMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of vertical force of second plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 2);
t = stanceData.secondVOffMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of vertical force of second plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 3);
t = stanceData.secondAPStrikeMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of anterior-posterior force of second plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 4);
t = stanceData.secondAPOffMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of anterior-posterior force of second plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 5);
t = stanceData.secondMLStrikeMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of pre-foot-strike samples', 'of medio-lateral force of second plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

subplot(3, 2, 6);
t = stanceData.secondMLOffMeans;
histogram(t)
box on
grid on
ylabel('Number of measurements')
xlabel({'Mean magnitude of post-foot-off samples', 'of medio-lateral force of second plate [N]' ...
    , sprintf('Max: %4.3f', max(t))})
axis([0 40 0 6000])

return





    